#include "compStudy.h"

void compStudy::comp(const ImportData& idata)
{
	//inSample(idata);
	
	string stages = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);
	
	
	string filename = "AbsResults" + instanceName;
	string tree = to_string((_Longlong)1);

	string path = directory + "ScenarioTrees\\" + instanceName + "\\" + stages + "_stage\\" + scenarios + "_scenarios\\" + "Tree_" + tree + "\\";	//The path were the scenario tree is saved

	nodeData node(numberOfStages, numberOfScenarios, idata, instalment, buildTime, path, branchPeriod, discountFactor, interest);

	

	nodeModel nodeMFRP(node, idata, addOption);

	string outPutDirectory = directory + "Solutions";

	compareData comparison(idata, node, outPutDirectory, addOptionString, instanceName, branch, stages, scenarios, tree);

	writeOutput(nodeMFRP, node, idata, outPutDirectory, filename, stages, scenarios, tree);

 	startingL = nodeMFRP.L;

	startingWorstCashFlow = nodeMFRP.worstCashFlow;


	string robustFileName = "RobustResultsX" + instanceName;
	double increment = 0.1;

	//testingRobustnes(node, idata, robustFileName, tree, increment, addOption);

	string absoluteRobustFileName = "AbsRobRes" + instanceName;
	//testingAbsoluteRobustnes(node, idata, absoluteRobustFileName, tree, increment, addOption);

	string CvarName = "CVarRes" + instanceName;

	testingCVAR(node, idata, CvarName, tree, increment);
	
/*
	for (int i = 1; i <= numberOfTrees; i++) {
		string tree = to_string((_ULonglong)i);


		string path = directory + "ScenarioTrees\\" + stages + "_stage\\" + scenarios + "_scenarios\\" + "Tree_" + tree + "\\";	//The path were the scenario tree is saved

		nodeData node(numberOfStages, numberOfScenarios, idata, instalment, buildTime, path, branchPeriod);
		//node.createTree(inputData, filePath, stages, scenarios);

		nodeModel nodeMFRP(node, idata, 1, 0.6);

		string outPutDirectory = directory + "Solutions\\" + stages + "_stage\\";

		writeOutput(nodeMFRP, node, idata, outPutDirectory, filename, stages, scenarios, tree);
	}
	
	*/

	
}

void compStudy::testingRobustnes(const nodeData& nodeData, const ImportData & idata, const string & filename, const string& tree, double& increment, const int& addOption)
{
	int modelOption = 1;
	string stages = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);

	double incrementLimit = 0.01;

	double L = startingL;

	bool flag;		//Set equal to the feasible parameter. changes to false when the solution does not give a solution



	nodeModel robustModel(nodeData, idata, modelOption, addOption, L);
	flag = robustModel.feasible;

	string outPutDirectory = directory + "Solutions";

	writeOutput(robustModel, nodeData, idata, outPutDirectory, filename, stages, scenarios, tree);

	string robustnesName = outPutDirectory + "\\" + instanceName + "\\AddOption_" + addOptionString + "\\" +"BranchPeriod_" + branch + "\\" + stages + "_stage\\" + scenarios + "_scenarios" + "\\" + "Tree_" + tree+ "\\testingL.txt";

	writeFactor(robustModel, L, robustnesName);


	while (flag) {

		L += increment;

		flag = testForL(nodeData, idata, filename, tree, increment, incrementLimit, modelOption, addOption, stages, scenarios, outPutDirectory, L, robustnesName, flag);
		
	}



}

void compStudy::testingAbsoluteRobustnes(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree, double & increment, const int & addOption)
{
	string stages = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);

	double incrementLimit = 0.1;

	double wCF = startingWorstCashFlow;

	bool flag;		//Set equal to the feasible parameter. changes to false when the solution does not give a solution



	nodeModel robustModel(nodeData, idata, modelOption, addOption, wCF);
	flag = robustModel.feasible;

	string outPutDirectory = directory + "Solutions";

	writeOutput(robustModel, nodeData, idata, outPutDirectory, filename, stages, scenarios, tree);

	string robustnesName = outPutDirectory + "\\" + instanceName + "\\AddOption_" + addOptionString + "\\" + "BranchPeriod_" + branch + "\\" + stages + "_stage\\" + scenarios + "_scenarios" + "\\" + "Tree_" + tree + "\\testingAbs.txt";

	writeFactor(robustModel, wCF, robustnesName);


	while (flag) {

		wCF += abs(startingWorstCashFlow * increment);

		flag = testForAbsoluteL(nodeData, idata, filename, tree, increment, incrementLimit, modelOption, addOption, stages, scenarios, outPutDirectory, wCF, robustnesName, flag);

	}
}

void compStudy::testingCVAR(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree, double & increment)
{
	string stages = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);

	double incrementLimit = 0.1;

	double wCF = startingWorstCashFlow;

	bool flag;		//Set equal to the feasible parameter. changes to false when the solution does not give a solution



	nodeModel robustModel(nodeData, idata, modelOption, addOption, wCF, confidenceLevel);
	flag = robustModel.feasible;

	string outPutDirectory = directory + "Solutions";

	writeOutput(robustModel, nodeData, idata, outPutDirectory, filename, stages, scenarios, tree);

	string robustnesName = outPutDirectory + "\\" + instanceName + "\\AddOption_" + addOptionString + "\\" + "BranchPeriod_" + branch + "\\" + stages + "_stage\\" + scenarios + "_scenarios" + "\\" + "Tree_" + tree + "\\testingAbs.txt";

	writeFactor(robustModel, wCF, robustnesName);


	while (flag) {

		wCF += abs(startingWorstCashFlow * increment);

		flag = testForCvarL(nodeData, idata, filename, tree, increment, incrementLimit, modelOption, addOption, stages, scenarios, outPutDirectory, wCF, confidenceLevel, robustnesName, flag);

	}
}

bool compStudy::testForL(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree, double & increment, const double& incrementLimit, const int& modelOption,
	const int & addOption, const string& stages, const string & scenarios, const string& outPutDirectory, double& L, const string& robustnesName, bool flag)
{
	if (flag) {
		nodeModel robustModelTesting(nodeData, idata, modelOption, addOption, L);
		flag = robustModelTesting.feasible;
		if (flag) {
			double relativeL = (abs(L - startingWorstCashFlow) / abs(startingWorstCashFlow)) + 1;
			string robustFileName = filename + "L="+ to_string((_Longlong)relativeL);
			writeOutput(robustModelTesting, nodeData, idata, outPutDirectory, robustFileName, stages, scenarios, tree);
			appendFactor(robustModelTesting, L, robustnesName);
		}
		else if (increment >= incrementLimit) {
			L -= increment;
			increment = increment / 5;
			flag = true;
		}

	}

	return flag;
}

bool compStudy::testForAbsoluteL(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree, double & increment, const double & incrementLimit, const int & modelOption, const int & addOption, const string & stages, const string & scenarios, const string & outPutDirectory, double & L, const string & robustnesName, bool flag)
{
	if (flag) {
		nodeModel robustModelTesting(nodeData, idata, modelOption, addOption, L);
		flag = robustModelTesting.feasible;
		if (flag) {
			double relativeL = (abs(L - startingWorstCashFlow) / abs(startingWorstCashFlow)) + 1.0;
			string robustFileName = filename + "L=" + to_string(static_cast<long double>(relativeL));
			writeOutput(robustModelTesting, nodeData, idata, outPutDirectory, robustFileName, stages, scenarios, tree);
			appendFactor(robustModelTesting, L, robustnesName);
		}
		else if (increment >= incrementLimit) {
			L -= abs(increment*startingWorstCashFlow);
			increment -= 0.2;
			flag = true;
		}

	}

	return flag;
}

bool compStudy::testForCvarL(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree, double & increment, const double & incrementLimit, const int & modelOption, const int & addOption, const string & stages, const string & scenarios, const string & outPutDirectory, double & L, const double & confidenceLevel, const string & robustnesName, bool flag)
{
	if (flag) {
		nodeModel robustModelTesting(nodeData, idata, modelOption, addOption, L, confidenceLevel);
		flag = robustModelTesting.feasible;
		if (flag) {
			double relativeL = (abs(L - startingWorstCashFlow) / abs(startingWorstCashFlow)) + 1.0;
			string robustFileName = filename + "L=" + to_string(static_cast<long double>(relativeL));
			writeOutput(robustModelTesting, nodeData, idata, outPutDirectory, robustFileName, stages, scenarios, tree);
			appendFactor(robustModelTesting, L, robustnesName);
		}
		else if (increment >= incrementLimit) {
			L -= abs(increment*startingWorstCashFlow);
			increment -= 0.2;
			flag = true;
		}

	}

	return flag;
}



void compStudy::inSample(const ImportData& idata)
{
	inSampleObjective.resize(numberOfTrees+1);
	inSampleObjective[0] = 0;

	string stages = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);

	string outputDirectory = directory + "Solutions";
	string scenarioTrees = directory + "ScenarioTrees";

	for (int i = 1; i <= numberOfTrees; i++) {
		string tree = to_string((_Longlong)i);
		string path = scenarioTrees +"\\" + instanceName + "\\" + stages + "_stage\\" + scenarios + "_scenarios\\" + "Tree_" + tree + "\\";
		string filename = "Results" + instanceName;

		nodeData node(numberOfStages, numberOfScenarios, idata, instalment, buildTime, path, branchPeriod, discountFactor, interest);

		compareData comparison(idata, node, outputDirectory, addOptionString, instanceName, branch, stages, scenarios, tree);

		nodeModel MFRP(node, idata);

		if (MFRP.feasible == true) {

			writeOutput(MFRP, node, idata, outputDirectory, filename, stages, scenarios, tree);

			inSampleObjective[i] = MFRP.Profit;
		}
		else {
			inSampleObjective[i] = 0;
		}
	}

	string resultsName = "InSampleTest";

	writeInSample(inSampleObjective, resultsName);

	

}

void compStudy::writeOutput(nodeModel& model, const nodeData& nodeData, const ImportData& idata, const string& outputDirectory, const string& filename, const string& stage, const string& scenarios, const string& tree)
{
	if (CreateDirectory(outputDirectory.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) { // Creating the folder solutions
		
		string fullPath =  outputDirectory +  "\\" + instanceName;

		if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder "instanceName"

			fullPath = fullPath + "\\" + "AddOption_" + addOptionString;

			if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {

				fullPath = fullPath + "\\" + "BranchPeriod_" + branch;

				if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder "BranchPeriod_"

					fullPath += "\\" + stage + "_stage";

					if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {		//Creating the folder _stage

						fullPath += "\\" + scenarios + "_scenarios";

						if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder _scenarios

							fullPath = fullPath + "\\" + "Tree_" + tree;

							if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder Tree_
								string resultFile = fullPath + "\\" + filename  + ".txt";
								string compareFile = fullPath + "\\" + "comparingDemandToCap" + "_" + stage + "_Stage_" + scenarios + "_Scenarios_" + "Tree_" + tree + ".txt";
								
								model.aggregateResults(nodeData, idata);
								model.writeToFile(resultFile, nodeData, idata);
								model.compareDemandToCap(compareFile, nodeData, idata);
							}
							else {
								string error = "Failed creating directory : " + fullPath;
								char* cstr = new char[error.length() + 1];
								strcpy_s(cstr, error.length() + 1, error.c_str());
								throw cstr;
							}



						}
						else {
							string error = "Failed creating directory : " + fullPath;
							char* cstr = new char[error.length() + 1];
							strcpy_s(cstr, error.length() + 1, error.c_str());
							throw cstr;
						}

					}
					else {
						string error = "Failed creating directory : " + fullPath;
						char* cstr = new char[error.length() + 1];
						strcpy_s(cstr, error.length() + 1, error.c_str());
						throw cstr;
					}
				}
				else {
					string error = "Failed creating directory : " + fullPath;
					char* cstr = new char[error.length() + 1];
					strcpy_s(cstr, error.length() + 1, error.c_str());
					throw cstr;
				}

			}
			else {
				string error = "Failed creating directory : " + fullPath;
				char* cstr = new char[error.length() + 1];
				strcpy_s(cstr, error.length() + 1, error.c_str());
				throw cstr;
			}

		}
			
		else {
			string error = "Failed creating directory : " + fullPath;
			char* cstr = new char[error.length() + 1];
			strcpy_s(cstr, error.length() + 1, error.c_str());
			throw cstr;
		}
	}
	else {
		string error = "Failed creating directory : " + outputDirectory;
		char* cstr = new char[error.length() + 1];
		strcpy_s(cstr, error.length() + 1, error.c_str());
		throw cstr;
	}

}

void compStudy::writeFactor(const nodeModel & model,const double& L, const string & filename)
{
	ofstream output(filename);

	output << "L \t" << "Expected Profit" << endl;

	output << L << "\t" << model.Profit << endl;

	output.close();

}

void compStudy::appendFactor(const nodeModel & model,const double& L, const string & filename)
{
	ofstream output(filename, ios::app);

	output << L << "\t" << model.Profit << endl;

	output.close();
}

void compStudy::writeInSample(const vector<double>& inSample, const string & filename)
{

	string stage = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);

	string outPutDirectory = directory + "Solutions";

	if (CreateDirectory(outPutDirectory.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) { // Creating the folder solutions

		string fullPath = outPutDirectory + "\\" + instanceName;

		if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder "instanceName"

			fullPath = fullPath + "\\" + "AddOption_" + addOptionString;

			if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {

				fullPath = fullPath + "\\" + "BranchPeriod_" + branch;

				if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder "BranchPeriod_"

					fullPath += "\\" + stage + "_stage";

					if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {		//Creating the folder _stage

						fullPath += "\\" + scenarios + "_scenarios";

						if (CreateDirectory(fullPath.c_str(), NULL) || ERROR_ALREADY_EXISTS == GetLastError()) {	//Creating the folder _scenarios

							string resultFile = fullPath + "\\" + filename + "_" + stage + "_Stage_" + scenarios + "_Scenarios.txt";

							ofstream output(resultFile);

							output << "Tree\t" << "Objective value" << endl;

							for (int i = 1; i <= numberOfTrees; i++) {

								output << i << "\t" << inSample[i] << endl;

							}

							output.close();

							appendInSample(inSample, resultFile);


						}

						else {
							string error = "Failed creating directory : " + fullPath;
							char* cstr = new char[error.length() + 1];
							strcpy_s(cstr, error.length() + 1, error.c_str());
							throw cstr;
						}
					}

					else {
						string error = "Failed creating directory : " + fullPath;
						char* cstr = new char[error.length() + 1];
						strcpy_s(cstr, error.length() + 1, error.c_str());
						throw cstr;
					}
				}
				else {
					string error = "Failed creating directory : " + fullPath;
					char* cstr = new char[error.length() + 1];
					strcpy_s(cstr, error.length() + 1, error.c_str());
					throw cstr;
				}
			}
			else {
				string error = "Failed creating directory : " + fullPath;
				char* cstr = new char[error.length() + 1];
				strcpy_s(cstr, error.length() + 1, error.c_str());
				throw cstr;
			}

			}
			else {
				string error = "Failed creating directory : " + fullPath;
				char* cstr = new char[error.length() + 1];
				strcpy_s(cstr, error.length() + 1, error.c_str());
				throw cstr;
			}
		}
		else {
			string error = "Failed creating directory : " + outPutDirectory;
			char* cstr = new char[error.length() + 1];
			strcpy_s(cstr, error.length() + 1, error.c_str());
			throw cstr;
		}
}

void compStudy::appendInSample(const vector<double>& inSample, const string & filename)
{
	double diff = 0;

	for (int i = 1; i <= numberOfTrees; i++) {
		for (int j = i; j <= numberOfTrees; j++) {
			double temp = abs((inSample[j] - inSample[i]) / inSample[i]);

			if (temp > diff) {
				diff = temp;
			}

		}
	}

	ofstream output(filename, ios::app);

	output << endl;
	output << "Maximum relative difference ((after-before)/before):\t" << diff << endl;

	output.close();


}
