#pragma once

#include "importData.h"
#include "nodeData.h"
#include "nodeModel.h"
#include "compareData.h"


class compStudy {
public:
	compStudy(const ImportData& idata, const double& instalment, const int& buildTime, const int & numberOfStages, 
		const int& numberOfScenarios, const int& branchPeriod, const int& numberOfTrees, const string& directory, 
		const string& instanceName, const double& discountFactor, const int& modelOption, const int& addOption, const double& confidenceLevel, const double& interest): instalment(instalment), buildTime(buildTime), 
		numberOfStages(numberOfStages), numberOfScenarios(numberOfScenarios), branchPeriod(branchPeriod), numberOfTrees(numberOfTrees), 
		directory(directory), instanceName(instanceName), discountFactor(discountFactor), branch(to_string((_Longlong)branchPeriod)),
		modelOption(modelOption), addOption(addOption), confidenceLevel(confidenceLevel), addOptionString(to_string((_Longlong)addOption)), interest(interest)	{ comp(idata); }

	compStudy(const ImportData& idata, const double& instalment, const int& buildTime, const int & numberOfStages,
		const int& numberOfScenarios, const vector<int>& branchPeriods, const int& numberOfTrees, const string& directory,
		const string& instanceName, const double& discountFactor,const int& modelOption, const int& addOption, const double& confidenceLevel, const double& interest) : instalment(instalment), buildTime(buildTime),
		numberOfStages(numberOfStages), numberOfScenarios(numberOfScenarios), branchPeriods(branchPeriods), numberOfTrees(numberOfTrees),
		directory(directory), instanceName(instanceName), discountFactor(discountFactor), branch(to_string((_Longlong)branchPeriods[0])), modelOption(modelOption),
		addOption(addOption), confidenceLevel(confidenceLevel), addOptionString(to_string((_Longlong)addOption)), interest(interest), branchPeriod(branchPeriods[0]) {comp(idata); }

	void comp(const ImportData& idata);

	void testingRobustnes(const nodeData& nodeData, const ImportData& idata, const string& filename, const string& tree, double & increment, const int& addOption);		//Testing for incresing ratio L

	void testingAbsoluteRobustnes(const nodeData& nodeData, const ImportData & idata, const string& filename, const string& tree, double & increment, const int& addOption);	//Testing for increasing absolute cash flow

	void testingCVAR(const nodeData& nodeData, const ImportData & idata, const string& filename, const string& tree, double & increment);

	bool testForL(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree,
		double & increment, const double& incrementLimit, const int& modelOption, const int & addOption, const string& stages, 
		const string & scenarios, const string& outPutDirectory, double& L, const string& robustnesName, bool flag);

	bool testForAbsoluteL(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree,
		double & increment, const double& incrementLimit, const int& modelOption, const int & addOption, const string& stages,
		const string & scenarios, const string& outPutDirectory, double& L, const string& robustnesName, bool flag);

	bool testForCvarL(const nodeData & nodeData, const ImportData & idata, const string & filename, const string & tree,
		double & increment, const double& incrementLimit, const int& modelOption, const int & addOption, const string& stages,
		const string & scenarios, const string& outPutDirectory, double& L, const double & confidenceLevel, const string& robustnesName, bool flag);

	void inSample(const ImportData& idata);

	void writeOutput(nodeModel& model, const nodeData& nodeData, const ImportData& idata, const string& outputDirectory, const string& filename, const string& stage, const string& scenario,
		const string& tree); //Writing output
	
	void writeFactor(const nodeModel& model,const double& L, const string& filename);
	void appendFactor(const nodeModel& model, const double& L, const string& filename);
	void writeInSample(const vector<double>& inSample, const string& filename);
	void appendInSample(const vector<double>& inSample, const string& filename);


	double startingL;		//Starting robustnes factor
	double startingWorstCashFlow;	//The worst case cash flow in the basic model
	vector<double> inSampleObjective;	//Saving the objective function of each tree

private:
	const double discountFactor;
	const double instalment;
	const int buildTime;
	const int branchPeriod;
	const vector<int> branchPeriods;
	const int numberOfStages;
	const int numberOfScenarios;
	const int numberOfTrees;
	const int modelOption;
	const int addOption;
	const double confidenceLevel;
	const double interest;
	const string addOptionString;
	const string directory;
	const string instanceName;
	const string branch;
};