#include "createData.h"

void CreateData::createData(const string & fileID, const int & timeHorizon, const string & fileNameAddOn)
{
	ImportData idata(fileID, timeHorizon);

	string fileName = fileNameAddOn+fileID;
	ofstream dataFile;
	dataFile.open(fileName);

	//Writing the set of ships
	dataFile << "Ships:";
	writeSet(dataFile, idata.Ships);

	//Writing the set of trades
	dataFile << "Trades:";
	writeSet(dataFile, idata.Trades);

	//Writing the set of contracted trades
	dataFile << "TradesC:";
	writeSet(dataFile, idata.TradesC);

	//Writing the set of optional trades
	dataFile << "TradesO:";
	writeSet(dataFile, idata.TradesO);
	
	//Writing the set of products
	dataFile << "Cargo:";
	writeSet(dataFile, idata.Product);

	//Writing Number of scenarios
	dataFile << "Scenario: ";
	writeInteger(dataFile, idata.Scenario);

	//Writing DeltasOM/the scenarios
	dataFile << "DeltasOM:";
	writeParameter(dataFile, idata.DeltasOM);

	//Writing the set of loops
	dataFile << "Routes:";
	writeSet(dataFile, idata.Routes);

	//Writing the set of ships in time t
	dataFile << "Vof:";
	writeSet(dataFile, idata.VoF);

	//Writing the set of new builds in time t
	dataFile << "VN:";
	writeSet(dataFile, idata.VN);

	//Writing the subset of loops done by ship v in time t
	dataFile << "RofVT:";
	writeSet(dataFile, idata.RofVT, idata.Ships);

	//Writing the subset of loops servicing trade i by ship v in time t
	dataFile << "RofIVT:";
	writeSet(dataFile, idata.RofIVT, idata.Trades, idata.Ships);

	//Writing the subset of trades availeble at t
	dataFile << "TradesOf:";
	writeSet(dataFile, idata.TradesOf);
	
	//Writing the subset of contracted trades available at t
	dataFile << "TradesCOf:";
	writeSet(dataFile, idata.TradesCOf);

	//Writing the subset of optional trades available at t
	dataFile << "TradesOOf:";
	writeSet(dataFile, idata.TradesOOf);

	//Writing the space charter cost
	dataFile << "CspaceC:";
	writeParameter(dataFile, idata.CspaceC, idata.Trades, idata.Product);

	//Writing the new build cost
	dataFile << "CNB:";
	writeParameter(dataFile, idata.CNB, idata.Ships);

	//Writing the second hand price
	dataFile << "CSH:";
	writeParameter(dataFile, idata.secondHand, idata.Ships);

	//Writing the charter in cost
	dataFile << "CCI:";
	writeParameter(dataFile, idata.CCI, idata.Ships);

	//Writing the lay-up savings
	dataFile << "RLU:";
	writeParameter(dataFile, idata.RLU, idata.Ships);

	//Writing the scrapping value
	dataFile << "RSC:";
	writeParameter(dataFile, idata.RSC, idata.Ships);

	//Writing the sunset value
	dataFile << "RSV:";
	writeParameter(dataFile, idata.RSV, idata.Ships);

	//Writing demand
	dataFile << "D:";
	writeParameterDemand(dataFile, idata.Demand, idata.Trades, idata.Product);

	//Writing minimum frequency
	dataFile << "MinFreq:";
	writeParameter(dataFile, idata.MinFreq, idata.Trades);

	//Writing initial fleet
	dataFile << "IP:";
	writeParameter(dataFile, idata.InitialPool, idata.Ships);

	//Writing previously ordered new builds
	dataFile << "YNB:";
	writeParameter(dataFile, idata.InitialNB, idata.Ships);

	//Writing revenue
	dataFile << "RO:";
	writeParameter(dataFile, idata.UnitRevenue, idata.Trades);

	//Writing capacities
	dataFile << "Cap:";
	writeParameter(dataFile, idata.Capacity, idata.Ships, idata.Product);

	//Writing total capacity
	dataFile << "CapTot:";
	writeParameter(dataFile, idata.TotalCapacity, idata.Ships);

	//Writing duration
	dataFile << "Dur:";
	writeParameter(dataFile, idata.Duration, idata.Ships, idata.Routes);

	//Writing loop cost
	dataFile << "CTR:";
	writeParameterLoop(dataFile, idata.LoopCost, idata.Ships, idata.Routes);

	//Writing operating cost
	dataFile << "COP:";
	writeParameter(dataFile, idata.OperatingCost, idata.Ships);

	// Writing charter in limit
	dataFile << "CIL:";
	writeParameter(dataFile, idata.CharterInLimit, idata.Ships);

	// Writing charter out limit
	dataFile << "COL:";
	writeParameter(dataFile, idata.CharterOutLimit, idata.Ships);

	// Writing selling limit
	dataFile << "SEL:";
	writeParameter(dataFile, idata.sellingLimit, idata.Ships);

	// Writing buying limit
	dataFile << "SHL:";
	writeParameter(dataFile, idata.buyingLimit, idata.Ships);

	//Writing total charter in limit
	dataFile << "CILT:";
	writeInteger(dataFile, idata.CharterInLimitTot);

	//Writing total charter out limit
	dataFile << "COLT:";
	writeInteger(dataFile, idata.CharterOutLimitTot);

	//Writing total selling limit
	dataFile << "SELT:";
	writeInteger(dataFile, idata.sellingLimitTot);

	//Writing total buying limit
	dataFile << "SHLT:";
	writeInteger(dataFile, idata.buyingLimitTot);

	//Writing scenario probability
	dataFile << "P:";
	writeParameter(dataFile, idata.Prob);

	//Writing the robustness factor
	if (idata.Lt.empty()) {

	}
	else {
		dataFile << "Lt:";
		writeParameter(dataFile, idata.Lt);
	}
	



	dataFile.close();
}

void CreateData::writeSet(ofstream& file,const map<string, int>& Set)
{
	map<string, int>::const_iterator iter;
	file << "[";

	for (iter = Set.begin(); iter != Set.end(); iter++) {
		file << "'" + iter->first + "'";
	}
	file << "]";
	file << endl;
}

void CreateData::writeSet(ofstream & file, const vector<map<string, int>>& Set)
{
	file << "[";
	for (int i = 0; i < Set.size(); i++) {
		file << "(";
		file << i;
		file << ")";
		writeSet(file, Set[i]);
	}
}

void CreateData::writeSet(ofstream & file, const vector<vector<map<string, int>>>& Set, const map<string, int> & stringIndex)
{
	
	file << "[";
	for (int i = 0; i < Set.size(); i++) {
		map<string, int>::const_iterator iterCheck;	//Used to find the string of the first index
		string text;
		bool flag = false;
		for (iterCheck = stringIndex.begin(); iterCheck != stringIndex.end(); iterCheck++) {
			if (iterCheck->second == i) {
				text = iterCheck->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Set[i].size(); j++) {
				file << "(";
				file << "'" + text + "',";
				file << j;
				file << ")";
				writeSet(file, Set[i][j]);
			}
		}
	}
	file << "]";
	file << endl;

}

void CreateData::writeSet(ofstream & file, const vector<vector<vector<map<string, int>>>>& Set, const map<string, int>& stringIndex1, const map<string, int>& stringIndex2)
{
	file << "[";
	for (int i = 0; i < Set.size(); i++) {
		map<string, int>::const_iterator iterCheck;	//Used to find the string of the first index
		string text;
		bool flag = false;
		for (iterCheck = stringIndex1.begin(); iterCheck != stringIndex1.end(); iterCheck++) {
			if (iterCheck->second == i) {
				text = iterCheck->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Set[i].size(); j++) {
				string text2;
				bool flag2 = false;
				map<string, int>::const_iterator iterCheck2;
				for (iterCheck2 = stringIndex2.begin(); iterCheck2 != stringIndex2.end(); iterCheck2++) {
					if (iterCheck2->second == j) {
						text2 = iterCheck2->first;
						flag2 = true;
						break;
					}
				}
					if (flag2) {
						for (int k = 0; k < Set[i][j].size(); k++) {
							file << "(";
							file << "'" + text + "',";
							file << "'" + text2 + "',";
							file << k;
							file << ")";
							writeSet(file, Set[i][j][k]);

						}
					}
				}
			}
		}
	file << "]";
	file << endl;
}

void CreateData::writeInteger(ofstream & file, const int & Integer)
{
	file << Integer;
	file << endl;
}

void CreateData::writeParameter(ofstream& file, const vector<int>& Parameter, const map<string, int>& stringIndex1) {
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				file << "('" + iter->first + "')";
				file << Parameter[i];
				file << endl;
				break;
			}
		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream & file, const vector<vector<double>>& Parameter)
{
	file << "[";
	file << endl;
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			file << "(";
			file << i;
			file << ", ";
			file << j;
			file << ") ";
			file << "[";
			file << Parameter[i][j];
			file << "]";
			file << endl;
		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream& file, const vector<double> & Parameter, const map<string, int>& stringIndex1) {

	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			file << "('" + text + "')";
			file << Parameter[i];
			file << endl;
		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameterDemand(ofstream & file, const vector<vector<vector<double>>>& Parameter, const map<string, int>& stringIndex1, const map<string, int>& stringIndex2)
{
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Parameter[i].size(); j++) {
				map<string, int>::const_iterator iter2;
					string text2;
					bool flag2 = false;
					for (iter2 = stringIndex2.begin(); iter2 != stringIndex2.end(); iter2++) {
						if (iter2->second == j) {
							text2 = iter2->first;
							flag2 = true;
							break;
						}
					}
					if (flag2) {
						for (int k = 0; k < Parameter[i][j].size(); k++) {
					
						file << "('" + text2 + "','" + text  + "',";
						file << k;
						file << ")";
						file << Parameter[i][j][k];
						file << endl;
					}
				}
				
			}
		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream & file, const vector<vector<vector<double>>>& Parameter, const map<string, int>& stringIndex1, const map<string, int>& stringIndex3)
{
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Parameter[i].size(); j++) {
				map<string, int>::const_iterator iter3;
				string text3;
				bool flag3 = false;
				for (iter3 = stringIndex3.begin(); iter3 != stringIndex3.end(); iter3++) {
					if (iter3->second == j) {
						text3 = iter3->first;
						flag3 = true;
						break;
					}
				}
			if (flag3) {
				for (int k = 0; k < Parameter[i][j].size(); k++) {
					
					
					
								file << "('" + text + "',";
								file << k;
								file << "," + text3 + ")";
								file << Parameter[i][j][k];
						}
				file << endl;
					}
				}
				
			}
		}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream & file, const vector<vector<double>>& Parameter, const map<string, int>& stringIndex1)
{
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Parameter[i].size(); j++) {
				
						file << "('" + text + "',";
						file << j;
						file << ")";
						file << std::defaultfloat << std::setprecision(std::numeric_limits<long double>::digits10) << Parameter[i][j];
						file << endl;
					}
				
			}
		}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream& file, const vector<vector<int>>& Parameter, const map<string, int>& stringIndex1) {
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Parameter[i].size(); j++) {

				file << "('" + text + "',";
				file << j;
				file << ")";
				file << Parameter[i][j];
				file << endl;
			}

		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream& file, const vector<vector<int>>& Parameter, const map<string, int>& stringIndex1, const map<string, int>& stringIndex2) {
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Parameter[i].size(); j++) {
				map<string, int>::const_iterator iter2;
				string text2;
				bool flag2 = false;
				for (iter2 = stringIndex2.begin(); iter2 != stringIndex2.end(); iter2++) {
					if (iter2->second == j) {
						text2 = iter2->first;
						flag2 = true;
						break;
					}
				}
				if (flag2) {
				

						file << "('" + text2 + "','" + text  +"')";
						file << Parameter[i][j];
						file << endl;
				}

			}
		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameterLoop(ofstream & file, const vector<vector<vector<double>>>& Parameter, const map<string, int>& stringIndex1, const map<string, int>& stringIndex2)
{
	file << "[";
	for (int i = 0; i < Parameter.size(); i++) {
		map<string, int>::const_iterator iter;
		string text;
		bool flag = false;
		for (iter = stringIndex1.begin(); iter != stringIndex1.end(); iter++) {
			if (iter->second == i) {
				text = iter->first;
				flag = true;
				break;
			}
		}
		if (flag) {
			for (int j = 0; j < Parameter[i].size(); j++) {
				map<string, int>::const_iterator iter2;
				string text2;
				bool flag2 = false;
				for (iter2 = stringIndex2.begin(); iter2 != stringIndex2.end(); iter2++) {
					if (iter2->second == j) {
						text2 = iter2->first;
						flag2 = true;
						break;
					}
				}
				if (flag2) {
					for (int k = 0; k < Parameter[i][j].size(); k++) {

						file << "('" + text + "','" + text2 + "',";
						file << k;
						file << ")";
						file << Parameter[i][j][k];
						file << endl;
					}
				}

			}
		}
	}
	file << "]";
	file << endl;
}

void CreateData::writeParameter(ofstream& file, const vector<double>& Parameter) {
	file << "[";
	file << endl;

	for (int i = 0; i < Parameter.size(); i++) {
		file << "(";
		file << i;
		file << ")";
		file << Parameter[i];
		file << endl;
	}
	file << "]";
	file << endl;
}
