#include "createTree.h"


void createTree::create(const ImportData& idata, const string & filePath, const int& numberOfStages, const int& numberOfScenarios)
{
	const string distType = "tx";
	const double range = 0.5;
	int numberOfRandoms = printMargins(idata, distType, range);			//Prints distribution to "tg_margins.txt" and returns number of random variables

	printCorrelation(idata, numberOfRandoms);			//Prints a zero-correlation matrix with 1 on the diagonal

	int nScenarios = numberOfScenarios;
	string stages = to_string((_Longlong)numberOfStages);
	string scenarios = to_string((_Longlong)numberOfScenarios);

	switch (numberOfStages) {

	case 2: {
		for (int t = 1; t < idata.Time; t++) {
			string time = to_string((_Longlong)t);
			/*if (t == 3) {
			nScenarios *= 2;
			scenarios = to_string((_ULonglong)numberOfScenarios);
			}*/
			string name = "T" + time + "Stage" + stages + "Node" + scenarios;

			string fileType = ".txt";
			string fileName = name + fileType;

			scenarioGen(filePath, fileName, nScenarios);
			Sleep(1000); // Required to get different scenarios from time period to time period


		}
		break;
	}
	case 3: {
		for (int t = branchPeriods[0]; t < idata.Time; t++) {
			string time = to_string((_Longlong)t);
			string name;
			if (t == branchPeriods[1]) {
				
				for (int i = 0; i < numberOfScenarios; i++) {
					string nodeTime = to_string((_Longlong)i);
					name = "T" + time + "Stage" + stages + "Node" + nodeTime + "Scenario" + scenarios;

					string fileType = ".txt";
					string fileName = name + fileType;

					scenarioGen(filePath, fileName, nScenarios);
					Sleep(1000); // Required to get different scenarios from time period to time period
				}
			}
			else {
				
				name = "T" + time + "Stage" + stages + "Node" + scenarios;

				string fileType = ".txt";
				string fileName = name + fileType;

				scenarioGen(filePath, fileName, nScenarios);
				Sleep(1000); // Required to get different scenarios from time period to time period
			}

		}

		break;
	}
	default: {
		throw "Failed generating scenarios. Select 2 or 3 stages.";
	}
	}

}

void createTree::scenarioGen(const string & filepath, const string & fileName, const int & numberOfScenarios)
{
	int S = numberOfScenarios;
	DWORD exitcode;


	stringstream strout;
	strout << "scengen.exe " << S;
	LPSTR cstr = _strdup(strout.str().c_str());
	STARTUPINFO info = { sizeof(info) };
	PROCESS_INFORMATION processInfo;
	if (CreateProcess("scengen.exe", cstr, NULL, NULL, TRUE, 0, NULL, NULL, &info, &processInfo))
	{
		::WaitForSingleObject(processInfo.hProcess, INFINITE);

		::GetExitCodeProcess(processInfo.hProcess, &exitcode);
		CloseHandle(processInfo.hProcess);
		CloseHandle(processInfo.hThread);
		if (exitcode != 0) {
			throw "Scenario generation algorithm failed. Try choosing a higher number of scenarios!";
		}
	}


	string dest = filepath + fileName;
	int count = 0;

	ifstream infile;
	do {
		
		infile.open("C:\\Users\\jorge\\OneDrive\\Dokumenter\\Visual Studio 2017\\Projects\\MFRP\\MFRP\\out_scen.txt", ios::binary);
		count++;
	} while (!infile.is_open() && count < 10);	//Tries to open the file 10 times before reporting an error

	ofstream destination(dest, ios::binary);

	destination << infile.rdbuf();

	infile.close();
	destination.close();
}

int createTree::printMargins(const ImportData & idata, const string& distType, const double& range)
{
	int counter = 0;			//Counting number of random variables
	ofstream output("tg_margins.txt");

	output << endl << endl;

	//Creating one random variable for each product and each trade

	for (int i = 0; i < idata.nTrades; i++) {
		for (int k = 0; k < idata.nProduct; k++) {
			output << distType << "\t" << "-" << range << "\t" << "0" << "\t" << range << endl;
			counter++;
		}
	}


	//Adding random variable new build price condition
	output << distType << "\t" << "-" << range << "\t" << "0" << "\t" << range << endl;
	counter++;

	//Adding random variable fuel price
	output << distType << "\t" << "-" << range << "\t" << "0" << "\t" << range << endl;
	counter++;

	//Adding random variable steel price
	output << distType << "\t" << "-" << range << "\t" << "0" << "\t" << range << endl;
	counter++;


	output.seekp(ios::beg);
	output << counter;

	output.close();

	return counter;

}

void createTree::printCorrelation(const ImportData & idata, const int & numberOfRandoms)
{



	ofstream output("tg_corrs.txt");

	output << numberOfRandoms << endl;
	output << numberOfRandoms << endl;

	int row = 0;
	int marketRow = numberOfRandoms - 3;
	int marketCol = numberOfRandoms - 3;

	for (int i = 0; i < numberOfRandoms; i++) {
		int col = 0;
		for (int j = 0; j < numberOfRandoms; j++) {
			if (row == col) {		//Filling the diagonal with 1
				output << "1 \t";
			}
			else if (col == marketCol && row <= marketRow) {		//Correlating the market condition to demand
				output << "0.7\t";
			}
			else if (row == marketRow && col <= marketCol) {		//Correlating the market condition to demand
				output << "0.7\t";
			}
			else if (row > marketRow || col > marketCol) {
				output << "0.2 \t";
			}
			else {
				output << "0.7 \t";
			}
			col++;

		}
		row++;
		output << endl;
	}

	output.close();

}