#include "importData.h"

void ImportData::importData(string fileID) {

	int deltaWidth = 3;											//The second dimension in the parameter DeltasOM
	Time = Horizon + 1;									//Used as argument to create the t-index in some functions
	regexParName = "[A-Z]+[a-z]*[A-Z]*[0-9]*[a-z]*:";		//Pattern of a parameter name
	regex regParName(regexParName);
	trimCond = '\'';
	trimCond2 = ',';
	trimCond3 = '\\)';
	trimCond4 = '\\(';
	trimCond5 = '\\[';

	regexColon = ":";
	regex regColon(regexColon);
										//Used to save trimmed strings, e.g to remove "'"
	
	inStream.open(fileID);
	
	if (inStream.fail())
	{
		cout << "Input file opening failed!\n";
		exit(1);
	}


	int switchCondition = 1;

	while (getline(inStream, row)) {

			
		if (regex_search(row, regColon))
		{
		if (regex_search(row, match, regParName)) {		//Requires a lot of computational time to check for long strings
			switch (switchCondition) {				//The data have to be read in order, i.e. following the order of the data file!!!
				
			case 1:
				//Reading ships
				if (match.str() == "Ships:") {
					regexPattern = "[A-Z]+[0-9]+";
					readSet(Ships, nShips, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 2:
				//Reading Trades
				if (match.str() == "Trades:") {
					regexPattern = "'[A-Z]+";
					readSet(Trades, nTrades, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 3:
				//Reading TradesC
				if (match.str() == "TradesC:") {
					regexPattern = "'[A-Z]+'";
					readSubSet(TradesC, Trades, nTradesC, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 4:
				//Reading TradesO
				if (match.str() == "TradesO:") {
					regexPattern = "'[A-Z]+'";
					readSubSet(TradesO, Trades, nTradesO, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 5:
				//Reading Product
				if (match.str() == "Cargo:") {
					regexPattern = "'[A-Z]*[a-z]*'";
					readSet(Product, nProduct, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 6:
				//Reading Scenario (int)
				if (match.str() == "Scenario:") {
					regexPattern = "[0-9]+";
					readInt(Scenario, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 7:
				//Reading DeltasOM
				if (match.str() == "DeltasOM:") {
					regexPattern = "-*0.[0-9]+";							//Defining the pattern to search
					readParameter(DeltasOM, Scenario, deltaWidth);
					switchCondition++;
					continue;
				}
				break;

			case 8:
				//Reading Routes
				if (match.str() == "Routes:") {
					regexPattern = "[0-9]+";
					readSet(Routes, nRoutes, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 9:
				// Reading VoF
				if (match.str() == "Vof:") {
					regexPattern = "\\[(.*?)\\]";
					regexPattern2 = "'[A-Z]+[0-9]+'";
					readSubSet(VoF, Ships, nVoF, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 10:
				//Reading VN
				if (match.str() == "VN:") {
					regexPattern = "\\[(.*?)\\]";
					regexPattern2 = "'[A-Z]+[0-9]+'";
					readSubSet(VN, Ships, nVN, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 11:
				// Reading RofVT
				if (match.str() == "RofVT:") {
					regexPattern = "\\((.*?)\\]";						//String with both indexes and values [\\[\\((.*?)\\[(.*?)\\]]+"
					regexPattern2 = ",[0-9]+\\)";						//Reads time index
					regexPattern3 = "'[A-Z]+[0-9]+'";					//Reads ship index
					regexPattern4 = "[0-9]+";							//Reads loop value
					regexPattern5 = "\\[(.*?)\\]";
					readSubSet(RofVT, Routes, nRofVT, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 12:
				// Reading RofIVT
				if (match.str() == "RofIVT:") {
					regexPattern = "\\((.*?)\\]";						//String with both indexes and values [\\[\\((.*?)\\[(.*?)\\]]+"
					regexPattern2 = ",[0-9]+\\)";						//Reads time index
					regexPattern3 = "'[A-Z]+[0-9]+'";					//Reads ship index
					regexPattern4 = "'[A-Z]+'";							//Reads trade
					regexPattern5 = "[0-9]+";							//Reads loop value
					regexPattern6 = "\\[(.*?)\\]";
					readSubSet(RofIVT, RofVT, nRofIVT, nTrades, nShips, Horizon, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 13:
				// Reading TradesOf
				if (match.str() == "TradesOf:") {
					regexPattern = "\\[(.*?)\\]";
					regexPattern2 = "'[A-Z]+'";
					//readSubSet(TradesOf, Trades, nTradesOf, regColon);
					createTimeSubSet(Trades, TradesOf, nTradesOf, Time);
					switchCondition++;
					continue;
				}
				break;

			case 14:
				// Reading TradesCOf
				if (match.str() == "TradesCOf:") {
					regexPattern = "\\[(.*?)\\]";
					regexPattern2 = "'[A-Z]+'";
					//readSubSet(TradesCOf, Trades, nTradesCOf, regColon);
					createTimeSubSet(TradesC, TradesCOf, nTradesCOf, Time);
					switchCondition++;
					continue;
				}
				break;

			case 15:
				// Reading TradesOOf
				if (match.str() == "TradesOOf:") {
					regexPattern = "\\[(.*?)\\]";
					regexPattern2 = "'[A-Z]+'";
					//readSubSet(TradesOOf, Trades, nTradesOOf, regColon);
					createTimeSubSet(TradesO, TradesOOf, nTradesOOf, Time);
					switchCondition++;
					continue;
				}
				break;

			case 16:
				//Reading CspaceC
				if (match.str() == "CspaceC:") {
					regexPattern = "'[A-Z]+',[0-9]+,[A-Z]+[a-z]*\\)[0-9]+.[0-9]+";
					regexPattern2 = "[,[0-9]+,";		//Reads time
					regexPattern3 = "'[A-Z]+'";			//Reads Trades
					regexPattern4 = "[A-Z]+[a-z]*\\)";		//Reads product
					regexPattern5 = "[0-9]+\\.[0-9]+";
					readParameter(CspaceC, nTrades, nProduct, Time, Trades, Product, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 17:
				//Reading CNB
				if (match.str() == "CNB:") {
					regexPattern = "\\((.*?)\\)[0-9]+\\.*[0-9]*";
					regexPattern2 = ",[0-9]+";
					regexPattern3 = "[A-Z]+[0-9]+";
					regexPattern4 = "\\)[0-9]+\\.*[0-9]*";

					readParameter(CNB, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;

				}
				break;

			case 18:
				//Reading CSH (=RSE)
				if (match.str() == "CSH:") {
					regexPattern = "\\((.*?)\\)[0-9]+\\.*[0-9]*";
					regexPattern2 = ",[0-9]+";
					regexPattern3 = "[A-Z]+[0-9]+";
					regexPattern4 = "\\)[0-9]+\\.*[0-9]*";

					readParameter(secondHand, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;

				}
				break;

			case 19:
				//Reading CCI
				if (match.str() == "CCI:") {
					regexPattern = "\\((.*?)\\.[0-9]+";
					regexPattern2 = ",[0-9]+\\)";
					regexPattern3 = "[A-Z]+[0-9]+";
					regexPattern4 = "[0-9]+\\.[0-9]+";

					readParameter(CCI, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 20:
				//Reading RLU
				if (match.str() == "RLU:") {
					regexPattern = "\\((.*?)\\.[0-9]+";
					regexPattern2 = ",[0-9]+\\)";
					regexPattern3 = "[A-Z]+[0-9]+";
					regexPattern4 = "[0-9]+\\.[0-9]+";

					readParameter(RLU, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 21:
				//Reading RSC
				if (match.str() == "RSC:") {
					regexPattern = "\\((.*?)\\)[0-9]+.*[0-9]*";
					regexPattern2 = ",[0-9]+";
					regexPattern3 = "[A-Z]+[0-9]+";
					regexPattern4 = "\\)[0-9]+\\.*[0-9]*";

					readParameter(RSC, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 22:
				//Reading RSV
				if (match.str() == "RSV:") {
					regexPattern = "\\((.*?)\\.[0-9]+";
					regexPattern2 = "[A-Z]+[0-9]+";
					regexPattern3 = "[0-9]+\\.[0-9]+";

					readParameter(RSV, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 23:
				//Reading Demand
				if (match.str() == "D:") {
					regexPattern = "\\((.*?)\\)[0-9]+\\.*[0-9]*";
					regexPattern2 = "[0-9]+\\)";		//Reads time
					regexPattern3 = ",'[A-Z]+'";			//Reads Trades
					regexPattern4 = "\\('[A-Z]+[a-z]*'";		//Reads product
					regexPattern5 = "[0-9]+\\.*[0-9]*";

					readParameter(Demand, nTrades, nProduct, Time, Trades, Product, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 24:
				//Reading MinFreq
				if (match.str() == "MinFreq:") {
					regexPattern = "\\((.*?)\\)[0-9]+\\.*[0-9]*";
					regexPattern2 = ",[0-9]+";
					regexPattern3 = "'[A-Z]+'";
					regexPattern4 = "\\)[0-9]+\\.*[0-9]*";

					readParameter(MinFreq, nTrades, Time, Trades, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 25:
				//Reading InitialPool
				if (match.str() == "IP:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "'[A-Z]+[0-9]+'";
					regexPattern3 = "\\)[0-9]+";

					readParameter(InitialPool, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 26:
				//Reading InitialNB
				if (match.str() == "YNB:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = ",[0-9]+";
					regexPattern3 = "'[A-Z]+[0-9]+'";
					regexPattern4 = "\\)[0-9]+";

					readParameter(InitialNB, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 27:
				//Reading UnitRevenue
				if (match.str() == "RO:") {
					regexPattern = "\\((.*?)\\)[0-9]+\\.*[0-9]*E*-*[0-9]*";
					regexPattern2 = ",[0-9]+";
					regexPattern3 = "'[A-Z]+'";
					regexPattern4 = "\\)[0-9]+\\.*[0-9]*E*-*[0-9]*";

					readParameter(UnitRevenue, nTrades, Time, Trades, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 28:
				//Reading Capacity
				if (match.str() == "Cap:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "'[A-Z]+[0-9]+'";
					regexPattern3 = "\\('[A-Z]+[a-z]*'";
					regexPattern4 = "\\)[0-9]+";

					readParameter(Capacity, nShips, nProduct, Ships, Product, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 29:
				//Reading TotalCapacity
				if (match.str() == "CapTot:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "'[A-Z]+[0-9]+'";
					regexPattern3 = "\\)[0-9]+";

					readParameter(TotalCapacity, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 30:
				//Reading Duration
				if (match.str() == "Dur:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "'[A-Z]+[0-9]+'";
					regexPattern3 = "\\('*[0-9]+'*";
					regexPattern4 = "\\)[0-9]+";

					readParameter(Duration, nShips, nRoutes, Ships, Routes, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 31:
				//Reading LoopCost
				if (match.str() == "CTR:") {
					regexPattern = "\\((.*?)\\)[0-9]+\\.*[0-9]*";
					regexPattern2 = "[[0-9]+\\)";		//Reads time
					regexPattern3 = "'[A-Z]+[0-9]+'";	//Reads Set1
					regexPattern4 = ",'*[0-9]+'*,";		//Reads Set2
					regexPattern5 = "[0-9]+\\.*[0-9]*";

					readParameter(LoopCost, nShips, nRoutes, Time, Ships, Routes, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 32:
				//Reading OperatingCost
				if (match.str() == "COP:") {
					regexPattern = "\\((.*?)\\.[0-9]+";
					regexPattern2 = ",[0-9]+";				//Reading time
					regexPattern3 = "'[A-Z]+[0-9]+'";		//Reading Ships
					regexPattern4 = "\\)[0-9]+\\.[0-9]+";	//Reading value

					readParameter(OperatingCost, nShips, Time, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 33:
				//Reading CIL, charter in limit
				if (match.str() == "CIL:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "[A-Z]+[0-9]+";
					regexPattern3 = "\\)[0-9]+";

					readParameter(CharterInLimit, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 34:
				//Reading COL, charter out limit
				if (match.str() == "COL:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "[A-Z]+[0-9]+";
					regexPattern3 = "\\)[0-9]+";

					readParameter(CharterOutLimit, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 35:
				//Reading SEL, Selling limit
				if (match.str() == "SEL:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "[A-Z]+[0-9]+";
					regexPattern3 = "\\)[0-9]+";

					readParameter(sellingLimit, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 36:
				//Reading SHL, buying limit
				if (match.str() == "SHL:") {
					regexPattern = "\\((.*?)\\)[0-9]+";
					regexPattern2 = "[A-Z]+[0-9]+";
					regexPattern3 = "\\)[0-9]+";

					readParameter(buyingLimit, nShips, Ships, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 37:
				//Reading CILT, Total charter in limit
				if (match.str() == "CILT:") {
					regexPattern = "[0-9]+";
					readInt(CharterInLimitTot, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 38:
				//Reading COLT, Total charter out limit
				if (match.str() == "COLT:") {
					regexPattern = "[0-9]+";
					readInt(CharterOutLimitTot, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 39:
				//Reading SELT, Total selling limit
				if (match.str() == "SELT:") {
					regexPattern = "[0-9]+";
					readInt(sellingLimitTot, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 40:
				//Reading SHLT, Total buying limit
				if (match.str() == "SHLT:") {
					regexPattern = "[0-9]+";
					readInt(buyingLimitTot, regColon);
					switchCondition++;
					continue;
				}
				break;

			case 41:
				// Reading Prob
				if (match.str() == "P:") {
					regexPattern = "0\\.[0-9]+";							//Defining the pattern to search
					readParameter(Prob, Scenario);
					switchCondition++;
					continue;
				}
				break;

			case 42:
				//Reading robustness factor Lt
				if (match.str() == "Lt:") {
					regexPattern = "\\)[0-9]+\\.*[0-9]*";
					readParameter(Lt, Time);
					initializeParameter(Lt, L);
					switchCondition++;
					continue;
				}
				break;

			default:
				//Here goes additonal data that isn't implemented in one of the cases
				continue;

				}
			}

		}
	}
	
	inStream.close();


	// Writing sets

/*	
	writeSet(Ships, nShips);
	writeSet(Trades, nTrades);
	writeSet(TradesC, nTradesC);
	writeSet(TradesO, nTradesO);
	writeSet(Product, nProduct);
	cout << "Scenario: " << Scenario << endl;
	writeParameter(DeltasOM);
	writeSet(Routes, nRoutes);
	writeSet(VoF, nVoF);
	writeSet(VN, nVN);
	writeSet(RofVT, nRofVT);
	writeSet(RofIVT, nRofIVT);
	writeSet(TradesOf, nTradesOf);
	writeSet(TradesCOf, nTradesCOf);
	writeSet(TradesOOf, nTradesOOf);
*/
	writeSetToFile(RofVT, nRofVT);
	writeSetToFile(RofIVT, nRofIVT);


	//compareSets(RofVT, nRofVT, RofIVT, nRofIVT, VoF, Time);
	/*
	//Writing Parameters
	writeParameter(CspaceC);
	writeParameter(CNB);
	writeParameter(secondHand);
	writeParameter(CCI);
	writeParameter(RLU);
	writeParameter(RSC);
	writeParameter(RSV);
	writeParameter(Demand);
	writeParameter(MinFreq);
	writeParameter(InitialPool);
	writeParameter(InitialNB);
	writeParameter(UnitRevenue);
	writeParameter(Capacity);
	writeParameter(TotalCapacity);
	writeParameter(Duration);
	writeParameter(LoopCost);
	writeParameter(OperatingCost);
	writeParameter(CharterInLimit);
	cout << "CILT: " << CharterInLimitTot << endl;
	writeParameter(Prob);
	*/	
	

}



void ImportData::readSet(map<string, int>& Set, int& elementCounter,regex& regColon) {
	cout << "Found specific parameter" << endl;
		regex reg1(regexPattern);							//Declaring the regex object

		
		string trim;
		int i = 0;
		do {												//Reading data into the parameter until a ':' is discovered
			
			sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
			sregex_iterator it_end;								//Creating the end condition/stop condition
			
			while (it != it_end) {
				trim = it->str();

				trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());

				Set[trim] = i;

				//cout << trim << " - " << i << endl;
				i++;
				it++;
			}
			elementCounter = i;
			linePos = inStream.tellg();
			getline(inStream, row);
		} while (!regex_search(row, regColon));
		inStream.seekg(linePos, ios_base::beg);
	}

void ImportData::readSubSet(map<string, int>& SubSet, map<string, int>& Set, int & elementCounter, regex & regColon)
{
	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object

	
	string trim;
	int i = 0;
	do {												//Reading data into the parameter until a ':' is discovered
		
		sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
		sregex_iterator it_end;								//Creating the end condition/stop condition
		
		while (it != it_end) {
			trim = it->str();

			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());
			map<string, int>::const_iterator iterSet;
			for (iterSet = Set.begin(); iterSet != Set.end(); iterSet++) {
				if (iterSet->first == trim) {
				SubSet[trim] = Set[trim];
				i++;
				}
				
			}
			

			//cout << trim << " - " << i << endl;
			
			it++;
		}
		elementCounter = i;
		linePos = inStream.tellg();
		getline(inStream, row);
	} while (!regex_search(row, regColon));
	inStream.seekg(linePos, ios_base::beg);
}

void ImportData::readSubSet(vector<map<string, int>>& SubSet, map<string, int>& Set, vector<int> & elementCounter, regex & regColon)
{

	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);

	string trim;
	int i;
	int k = 0;
	
	
	SubSet.resize(Horizon + 1);
	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
		sregex_iterator it_end;								//Creating the end condition/stop condition
		
		while (it != it_end) {
			i = 0;
			temp = it->str();
			sregex_iterator innerIt(temp.begin(), temp.end(), reg2);
			sregex_iterator innerIt_end;

			while (innerIt != innerIt_end) {
				trim = innerIt->str();
				trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());

				map<string, int>::const_iterator iterSet;
				for (iterSet = Set.begin(); iterSet != Set.end(); iterSet++) {
					if (iterSet->first == trim) {
						SubSet[k][trim] = Set[trim];
						i++;
					}
				}
				
				
				innerIt++;
			}
			elementCounter.push_back(i);
			//cout << trim << " - " << i << endl;
			k++;
			it++;
		}
		linePos = inStream.tellg();
		getline(inStream, row);
	} while (!regex_search(row, regColon));
	inStream.seekg(linePos, ios_base::beg);
}

void ImportData::readSubSet(vector<vector<map<string, int>>>& SubSet, map<string, int>& Set, vector<vector<int>>& elementCounter, regex& regColon) {


	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);
	regex reg4(regexPattern4);
	regex reg5(regexPattern5);

	bool flag;
	string trim;
	string temp;
	int i;
	int k = 0;
	int m = 0;

	SubSet.resize(nShips);
	elementCounter.resize(nShips);
	for (int j = 0; j < nShips; j++) {
		SubSet[j].resize(Horizon + 1);
		elementCounter[j].resize(Horizon+1);
		}

	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
		sregex_iterator it_end;								//Creating the end condition/stop condition
		
		
		while (it != it_end) {
			flag = true;
			i = 0;
			temp = it->str();

			sregex_iterator secondIt(temp.begin(), temp.end(), reg2);
			sregex_iterator secondIt_end;
			
			trim = secondIt->str();
					trim.erase(remove(trim.begin(), trim.end(), trimCond2), trim.end());
					trim.erase(remove(trim.begin(), trim.end(), trimCond3), trim.end());
					stringstream convert(trim);
					convert >> m;

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg3);
			sregex_iterator thirdIt_end;
			
			trim = thirdIt->str();
					trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());

					map<string, int>::const_iterator iterSet;
					for (iterSet = Ships.begin(); iterSet != Ships.end(); iterSet++) {
						if (iterSet->first == trim) {											//Checking for match in superset
							k = Ships[trim];
							flag = false;
							break;
						}
					}
					if (flag) {
						it++;
						continue;
					}
					
					smatch tempMatch;
					regex_search(temp, tempMatch, reg5);			//Extracts the string between to square brackets
					string routes = tempMatch.str();

							sregex_iterator innerIt(routes.begin(), routes.end(), reg4);
							sregex_iterator innerIt_end;

							string tempRoute;
							while (innerIt != innerIt_end) {
								

								tempRoute = innerIt->str();
								tempRoute.erase(remove(tempRoute.begin(), tempRoute.end(), trimCond5), tempRoute.end());
								for (iterSet = Set.begin(); iterSet != Set.end(); iterSet++) {
									if (iterSet->first == tempRoute) {
										SubSet[k][m][tempRoute] = Set[tempRoute];
										i++;
									}
								}

								innerIt++;

							}
			elementCounter[k][m] = i;

			it++;
			

		}
	
		linePos = inStream.tellg();
		getline(inStream, row);
	} while (!regex_search(row, regColon));
	inStream.seekg(linePos, ios_base::beg);
}

void ImportData::readSubSet(vector<vector<vector<map<string, int>>>>& SubSet, vector<vector<map<string, int>>>& Set, vector<vector<vector<int>>>& elementCounter, int& index1, int& index2, int& index3, regex& regColon) {
	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);
	regex reg4(regexPattern4);
	regex reg5(regexPattern5);
	regex reg6(regexPattern6);

	string trim;
	
	bool flag;

	int k;
	int i = 0;
	int v = 0;
	int t = 0;

	SubSet.resize(index1);
	elementCounter.resize(index1);
	for (int i = 0; i < index1; i++) {
		SubSet[i].resize(index2);
		elementCounter[i].resize(index2);
		for (int v = 0; v < index2; v++) {
			SubSet[i][v].resize(index3 + 1);
			elementCounter[i][v].resize(index3 + 1);
		}
	}

	
	
	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			k = 0;

			temp = it->str();
				
			sregex_iterator secondIt(temp.begin(), temp.end(), reg2);		// Finds the period
			sregex_iterator secondIt_end;

			trim = secondIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond2), trim.end());
			trim.erase(remove(trim.begin(), trim.end(), trimCond3), trim.end());
			stringstream convert(trim);
			convert >> t;

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg3);		//Finds the ship
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());
			map<string, int>::const_iterator iterSet;
			for (iterSet = Ships.begin(); iterSet != Ships.end(); iterSet++) {		//Checking for match in superset
				if (iterSet->first == trim) {
					v = Ships[trim];
					flag = false;
					break;
				}
			}

			if (flag) { 
				it++;
				continue; }

			flag = true;

			sregex_iterator fourthIt(temp.begin(), temp.end(), reg4);
			sregex_iterator fourthIt_end;

			trim = fourthIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());			//Finds the trades
			for (iterSet = Trades.begin(); iterSet != Trades.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					i = Trades[trim];
					flag = false;
					break;
				}
			}
			if (flag) { 
				it++;
				continue; }

			smatch tempMatch;
			regex_search(temp, tempMatch, reg6);			//Extracts the string between to square brackets
			string routes = tempMatch.str();

			sregex_iterator innerIt(routes.begin(), routes.end(), reg5);
			sregex_iterator innerIt_end;

			string tempRoute;
			while (innerIt != innerIt_end) {


				tempRoute = innerIt->str();
				tempRoute.erase(remove(tempRoute.begin(), tempRoute.end(), trimCond5), tempRoute.end());
					for (iterSet = Set[v][t].begin(); iterSet != Set[v][t].end(); iterSet++) {
					if (iterSet->first == tempRoute) {
						SubSet[i][v][t][tempRoute] = Set[v][t][tempRoute];							//Checking fot match in set
						k++;
						break;
					}
				}
				
				innerIt++;
				
			}
			elementCounter[i][v][t] = k;
			it++;
			

		}

		linePos = inStream.tellg();
		getline(inStream, row);
	} while (!regex_search(row, regColon));
	inStream.seekg(linePos, ios_base::beg);
}

void ImportData::createTimeSubSet(map<string, int>& Set, vector<map<string, int>>& SubSet, vector<int>& elementCounter, int & index1)
{
	SubSet.resize(index1);
	elementCounter.resize(index1);
	elementCounter[0] = 0;


	for (int t = 1; t < index1; t++) {
		int k = 0;
		map<string, int>::const_iterator iterSet;
		for (iterSet = Set.begin(); iterSet != Set.end(); iterSet++) {
			string str = iterSet->first;
			SubSet[t][str] = Set[str];
			k++;
		}
		elementCounter[t] = k;
	}
}


void ImportData::readInt(int & integer, regex& regColon)
{
		regex reg1(regexPattern);							//Declaring the regex object
		
		do {												//Reading data into the parameter until a ':' is discovered

			sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
			sregex_iterator it_end;								//Creating the end condition/stop condition

			while (it != it_end) {
				trim = (it->str());
				stringstream convert(trim);
				convert >> integer;
				++it;
			}
			linePos = inStream.tellg();						//Saving the line position
			getline(inStream, row);							//Reading a new line
		} while (!regex_search(row, regColon));				//If colon found, exit loop
		inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
	

}

void ImportData::readParameter(vector<vector<double>>& Parameter, int & index1, int &index2)
{
		regex reg1(regexPattern);							//Declaring the regex object


		Parameter.resize(index1);							//Creating the matrix

		for (int i = 0; i < index1; i++)
		{
			Parameter[i].resize(index2);
		}

		for (int i = 0; i < index1; i++) {
			for (int j = 0; j < index2; j++) {

				//do {												//Reading data into the parameter until a ':' is discovered

				sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
				sregex_iterator it_end;								//Creating the end condition/stop condition
				int test = 0;
				while (it != it_end) {

					trim = it->str();
					stringstream convert(trim);
					convert >> Parameter[i][j];
					++it;
					test++;
				}
				if (test == 0) {								//If the string doesn't contain the pattern, the iterator j is decremented, because the the loop never activated
					j--;
				}

				//} while (!regex_search(row, regColon));			//If colon found, exit loop
				linePos = inStream.tellg();						//Saving the line position
				getline(inStream, row);							//Reading a new line
			}
		}
		inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<vector<vector<double>>>& Parameter, int &index1, int& index2, int& index3, map<string,int> Set1, map<string,int> Set2, regex & regColon)
{
	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);
	regex reg4(regexPattern4);
	regex reg5(regexPattern5);

	bool flag;
	int i = 0;
	int v = 0;
	int t = 0;
	double parVal; //The value of the parameter

	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
		for (int v = 0; v < index2; v++) {
			Parameter[i][v].resize(index3);
		}
	}

	//initializing Parameter in order to avoid strange storage

	for (int i = 0; i < index1; i++) {
		for (int v = 0; v < index2; v++) {
			for (int t = 0; t < index3; t++) {
				Parameter[i][v][t] = 0;
			}
		}
	}


	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			temp = it->str();

			sregex_iterator secondIt(temp.begin(), temp.end(), reg2);		//Finds reg2
			sregex_iterator secondIt_end;

			trim = secondIt->str();
			trimFunction(trim);
			stringstream convert(trim);
			convert >> t;

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg3);		//Finds reg3
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trimFunction(trim);
			map<string, int>::const_iterator iterSet;

			for (iterSet = Set1.begin(); iterSet != Set1.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					i = Set1[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}
			
			flag = true;

			sregex_iterator fourthIt(temp.begin(), temp.end(), reg4);					//Finds reg 4
			sregex_iterator fourthIt_end;

			trim = fourthIt->str();
			trimFunction(trim);

			for (iterSet = Set2.begin(); iterSet != Set2.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					v = Set2[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}


			sregex_iterator innerIt(temp.begin(), temp.end(), reg5);
			sregex_iterator innerIt_end;

			string tempSpace;

			while (innerIt != innerIt_end) {
				tempSpace = innerIt->str();
				string::size_type sz;				//Needed to use the stod()-function
				parVal = stod(tempSpace, &sz);			//Converts string to double
				
				Parameter[i][v][t] = parVal;
				innerIt++;
			}

			it++;

		}
			
			linePos = inStream.tellg();						//Saving the line position
			getline(inStream, row);							//Reading a new line
		} while (!regex_search(row, regColon));			//If colon found, exit loop
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<vector<double>>& Parameter, int & index1, int & index2, map<string, int>& Set1, regex& regColon)
{
	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);
	regex reg4(regexPattern4);

	bool flag;
	int v = 0;
	int t = 0;
	double parVal = 0; //The value of the parameter

	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
	}


	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			temp = it->str();

			sregex_iterator secondIt(temp.begin(), temp.end(), reg2);		//Finds reg2
			sregex_iterator secondIt_end;

			trim = secondIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond2), trim.end());
			trim.erase(remove(trim.begin(), trim.end(), trimCond3), trim.end());
			stringstream convert(trim);
			convert >> t;

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg3);		//Finds reg3
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());
			map<string, int>::const_iterator iterSet;

			for (iterSet = Set1.begin(); iterSet != Set1.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					v = Set1[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}


			sregex_iterator innerIt(temp.begin(), temp.end(), reg4);
			sregex_iterator innerIt_end;

			while (innerIt != innerIt_end) {
				temp = innerIt->str();
				trimFunction(temp);
				string::size_type sz;				//Needed to use the stod()-function
				parVal = stod(temp, &sz);			//Converts string to double

				Parameter[v][t] = parVal;
				innerIt++;
			}

			it++;

		}

		linePos = inStream.tellg();						//Saving the line position
		getline(inStream, row);							//Reading a new line
	} while (!regex_search(row, regColon));			//If colon found, exit loop
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<double>& Parameter, int & index1, map<string, int> Set1, regex & regColon)
{

	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);

	bool flag;
	int v = 0;
	double parVal = 0; //The value of the parameter

	Parameter.resize(index1);
	
	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			temp = it->str();

		

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg2);		//Finds reg2
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());
			map<string, int>::const_iterator iterSet;

			for (iterSet = Set1.begin(); iterSet != Set1.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					v = Set1[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}



			sregex_iterator innerIt(temp.begin(), temp.end(), reg3);		//Finds reg3
			sregex_iterator innerIt_end;

			while (innerIt != innerIt_end) {
				temp = innerIt->str();
				temp.erase(remove(temp.begin(), temp.end(), trimCond4), trim.end());
				string::size_type sz;				//Needed to use the stod()-function
				parVal = stod(temp, &sz);			//Converts string to double

				Parameter[v] = parVal;
				innerIt++;
			}

			it++;

		}

		linePos = inStream.tellg();						//Saving the line position
		getline(inStream, row);							//Reading a new line
	} while (!regex_search(row, regColon));			//If colon found, exit loop
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<int>& Parameter, int&index1, map<string, int> Set1, regex& regColon) 
{

	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);

	bool flag;
	int v = 0;
	int parVal = 0; //The value of the parameter

	Parameter.resize(index1);

	//Initialize Parameter

	for (int i = 0; i < Parameter.size(); i++) {
		Parameter[i] = 0;
	}


	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			temp = it->str();



			sregex_iterator thirdIt(temp.begin(), temp.end(), reg2);		//Finds reg2
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());
			map<string, int>::const_iterator iterSet;

			for (iterSet = Set1.begin(); iterSet != Set1.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					v = Set1[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}


			sregex_iterator innerIt(temp.begin(), temp.end(), reg3);		//Finds reg3
			sregex_iterator innerIt_end;

			while (innerIt != innerIt_end) {
				temp = innerIt->str();
				temp.erase(remove(temp.begin(), temp.end(), trimCond3), trim.end());
				stringstream convert(temp);
				convert >> parVal;			//Converts string to int

				Parameter[v] = parVal;
				innerIt++;
			}

			it++;

		}

		linePos = inStream.tellg();						//Saving the line position
		getline(inStream, row);							//Reading a new line
	} while (!regex_search(row, regColon));			//If colon found, exit loop
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<vector<int>>& Parameter, int& index1, int& index2, map<string, int>& Set1, regex& regColon) 
{
	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);
	regex reg4(regexPattern4);

	bool flag;
	int v = 0;
	int t = 0;
	int parVal = 0; //The value of the parameter

	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
	}


	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			temp = it->str();

			sregex_iterator secondIt(temp.begin(), temp.end(), reg2);		//Finds reg2
			sregex_iterator secondIt_end;

			trim = secondIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond2), trim.end());
			trim.erase(remove(trim.begin(), trim.end(), trimCond3), trim.end());
			stringstream convert(trim);
			convert >> t;

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg3);		//Finds reg3
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trim.erase(remove(trim.begin(), trim.end(), trimCond), trim.end());
			map<string, int>::const_iterator iterSet;

			for (iterSet = Set1.begin(); iterSet != Set1.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					v = Set1[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}

				
			sregex_iterator innerIt(temp.begin(), temp.end(), reg4);
			sregex_iterator innerIt_end;

			while (innerIt != innerIt_end) {
				temp = innerIt->str();
				temp.erase(remove(temp.begin(), temp.end(), trimCond3), trim.end());
				temp.erase(remove(temp.begin(), temp.end(), trimCond4), trim.end());
				stringstream convert(temp);
				convert >> parVal;

				Parameter[v][t] = parVal;
				innerIt++;
			}

			it++;

		}

		linePos = inStream.tellg();						//Saving the line position
		getline(inStream, row);							//Reading a new line
	} while (!regex_search(row, regColon));			//If colon found, exit loop
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<vector<int>>& Parameter, int & index1, int & index2, map<string, int>& Set1, map<string, int>& Set2, regex & regColon)
{

	cout << "Found specific parameter" << endl;
	regex reg1(regexPattern);							//Declaring the regex object
	regex reg2(regexPattern2);
	regex reg3(regexPattern3);
	regex reg4(regexPattern4);

	bool flag;
	int v = 0;
	int k = 0;
	int parVal = 0; //The value of the parameter

	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
	}


	do {												//Reading data into the parameter until a ':' is discovered

		sregex_iterator it(row.begin(), row.end(), reg1);	//Splits the row into strings of index+value
		sregex_iterator it_end;								//Creating the end condition/stop condition


		while (it != it_end) {
			flag = true;
			temp = it->str();

			sregex_iterator secondIt(temp.begin(), temp.end(), reg2);		//Finds reg2
			sregex_iterator secondIt_end;

			trim = secondIt->str();
			trimFunction(trim);
			map<string, int>::const_iterator iterSet;

			for (iterSet = Set1.begin(); iterSet != Set1.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					v = Set1[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}

			flag = true;

			sregex_iterator thirdIt(temp.begin(), temp.end(), reg3);		//Finds reg3
			sregex_iterator thirdIt_end;

			trim = thirdIt->str();
			trimFunction(trim);
			for (iterSet = Set2.begin(); iterSet != Set2.end(); iterSet++) {
				if (iterSet->first == trim) {											//Checking for match in superset
					k = Set2[trim];
					flag = false;
					break;
				}
			}
			if (flag) {
				it++;
				continue;
			}
			
			sregex_iterator innerIt(temp.begin(), temp.end(), reg4);
			sregex_iterator innerIt_end;

			while (innerIt != innerIt_end) {
				temp = innerIt->str();
				trimFunction(temp);
				stringstream convert(temp);
				convert >> parVal;

				Parameter[v][k] = parVal;
				innerIt++;
			}

			it++;

		}

		linePos = inStream.tellg();						//Saving the line position
		getline(inStream, row);							//Reading a new line
	} while (!regex_search(row, regColon));			//If colon found, exit loop
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::readParameter(vector<double>& Parameter, int & index1)
{

	regex reg1(regexPattern);							//Declaring the regex object


	Parameter.resize(index1);							//Creating the matrix

	for (int i = 0; i < index1; i++) {
		
			//do {												//Reading data into the parameter until a ':' is discovered

			sregex_iterator it(row.begin(), row.end(), reg1);	//Creating the iterator
			sregex_iterator it_end;								//Creating the end condition/stop condition
			int test = 0;
			while (it != it_end) {

				trim = it->str();
				trimFunction(trim);
				stringstream convert(trim);
				convert >> Parameter[i];
				++it;
				test++;
			}
			if (test == 0) {								//If the string doesn't contain the pattern, the iterator j is decremented, because the the loop never activated
				i--;
			}

			//} while (!regex_search(row, regColon));			//If colon found, exit loop
			linePos = inStream.tellg();						//Saving the line position
			getline(inStream, row);							//Reading a new line
	}
	inStream.seekg(linePos, ios_base::beg);				//Sets the read position to the previous line
}

void ImportData::initializeParameter(const vector<double>& Parameter, double & ParameterInit)
{
	ParameterInit = Parameter[1];	//Not including year 0, because we might get infinite results. 
	for (int t = 1; t < Parameter.size(); t++) {
		if (ParameterInit > Parameter[t]) {
			ParameterInit = Parameter[t];
		}
	}
}




void ImportData::trimFunction(string& trimmingString)
{
	trimmingString.erase(remove(trimmingString.begin(), trimmingString.end(), trimCond), trimmingString.end());
	trimmingString.erase(remove(trimmingString.begin(), trimmingString.end(), trimCond2), trimmingString.end());
	trimmingString.erase(remove(trimmingString.begin(), trimmingString.end(), trimCond3), trimmingString.end());
	trimmingString.erase(remove(trimmingString.begin(), trimmingString.end(), trimCond4), trimmingString.end());
}




void ImportData::writeSet(map<string, int>& Set, int& elementCounter) {
	map<string, int>::const_iterator iter;
	cout << "Number of Set elements: " << elementCounter << endl;
	for (iter = Set.begin(); iter != Set.end(); iter++) {
		cout << iter->first << " - " << iter->second << endl;
	}

}

void ImportData::writeSet(vector<map<string, int>>& Set, vector<int> & elementCounter)
{
	for (int i = 0; i < Set.size(); i++) {
		cout << "Number of Set elements: " << elementCounter[i] << endl;;
		map<string, int>::const_iterator iter;
		for (iter = Set[i].begin(); iter != Set[i].end(); iter++) {
			cout << iter->first << " - " << iter->second << "\t";
		}
		cout << endl;
	}
}

void ImportData::writeSet(vector<vector<map<string, int>>>& Set, vector<vector<int>> elementCounter)
{
	for (int i = 0; i < Set.size(); i++) {
		for (int j = 0; j < Set[i].size(); j++){
		cout << "Number of Set elements: " << elementCounter[i][j] << endl;
		cout << "Period: " << j << endl;

		map<string, int>::const_iterator iter;
		for (iter = Ships.begin(); iter != Ships.end(); iter++) {
			if (iter->second == i) {
				cout << "Ship: " << iter->first << endl;
				break;
			}
		}

		for (iter = Set[i][j].begin(); iter != Set[i][j].end(); iter++) {
			cout << iter->first << " - " << iter->second << endl;
			}
		cout << endl;
		}
		cout << endl;
	}
}

void ImportData::writeSet(vector<vector<vector<map<string, int>>>>& Set, vector<vector<vector<int>>> elementCounter)
{
	for (int i = 0; i < Set.size(); i++) {
		for (int v = 0; v < Set[i].size(); v++) {
			for (int t = 0; t < Set[i][v].size(); t++) {
				cout << "Number of Set elements: " << elementCounter[i][v][t] << endl;

				map<string, int>::const_iterator iter;
				for (iter = Trades.begin(); iter != Trades.end(); iter++) {
					if (iter->second == i) {
						cout << "Trades: " << iter->first << endl;
						break;
					}
				}

				for (iter = Ships.begin(); iter != Ships.end(); iter++) {
					if (iter->second == v) {
						cout << "Ships: " << iter->first << endl;
						break;
					}
				}
				cout << "Period: " << t << endl;
				
				map<string, int>::const_iterator iterSet;
				for (iterSet = Set[i][v][t].begin(); iterSet != Set[i][v][t].end(); iterSet++) {
					cout << iterSet->first << " - " << iterSet->second << endl;
				}
				cout << endl;
			}
		}
		cout << endl;
	}
}

void ImportData::writeParameter(vector<vector<double>>& Parameter)
{
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			cout << "(" << i << ", " << j << ") " << Parameter[i][j] << endl;
		}
	}
}

void ImportData::writeParameter(vector<vector<vector<double>>>& Parameter) {
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			for (int k = 0; k < Parameter[i][j].size(); k++) {
				cout << "(" << i << ", " << j << ", "<< k << ") - " << Parameter[i][j][k] << endl;
			}
		}
	}
}

void ImportData::writeParameter(vector<double>& Parameter) {
	
	for (int i = 0; i < Parameter.size(); i++) {
		cout << i << " - " << Parameter[i] << endl;
	}
}

void ImportData::writeParameter(vector<int>& Parameter) {

	for (int i = 0; i < Parameter.size(); i++) {
		cout << i << " - " << Parameter[i] << endl;
	}
}

void ImportData::writeParameter(vector<vector<int>>& Parameter)
{
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			cout << "(" << i << ", " << j << ") " << Parameter[i][j] << endl;
		}
	}
}

void ImportData::compareSets(const vector<vector<map<string, int>>>& Set, const vector<vector<int>>& elementCounter, const vector<vector<vector<map<string, int>>>>& SubSet, const vector<vector<vector<int>>>& elementCounterSub, const vector<map<string, int>>& index1, const int & index2)
{
	cout << "Routes that doesn't exist in the superset: " << endl;
	bool flag;
	for (int t = 0; t < index2; t++) {
		map<string, int>::const_iterator iter;
		for (iter = index1[t].begin(); iter != index1[t].end(); iter++) {
			int v = iter->second;
			string index1Name = iter->first;
			
			map<string, int>::const_iterator iterTrade;
			for (iterTrade = TradesOf[t].begin(); iterTrade != TradesOf[t].end(); iterTrade++) {
				int i = iterTrade->second;

				map<string, int>::const_iterator iter3;
				for (iter3 = SubSet[i][v][t].begin(); iter3 != SubSet[i][v][t].end(); iter3++) {
					flag = true;
					//cout << iter3->first << " - " << iter3->second << endl;
					map<string, int>::const_iterator iter2;
					for (iter2 = Set[v][t].begin(); iter2 != Set[v][t].end(); iter2++) {
					//cout << iter2->first << " - " << iter2->second << endl;
						if (iter2->second == iter3->second) {
							flag = false;
							break;
						}
					}
					if (flag) {
						cout << "(" << iterTrade->first << ", " << index1Name << ", " << t << ")" << " - " << iter3->first << endl;
					}

				}
			}

		}
	}
}

void ImportData::writeSetToFile(vector<vector<map<string, int>>>& Set, vector<vector<int>> elementCounter)
{
	ofstream output;
	output.open("Routes.txt");
	for (int i = 0; i < Set.size(); i++) {
		for (int j = 0; j < Set[i].size(); j++) {
			output << "Number of Set elements: " << elementCounter[i][j] << endl;
			output << "Period: " << j << endl;

			map<string, int>::const_iterator iter;
			for (iter = Ships.begin(); iter != Ships.end(); iter++) {
				if (iter->second == i) {
					output << "Ship: " << iter->first << endl;
					break;
				}
			}

			for (iter = Set[i][j].begin(); iter != Set[i][j].end(); iter++) {
				output << iter->first << " - " << iter->second << endl;
			}
			output << endl;
		}
		output << endl;
	}

	output.close();
}

void ImportData::writeSetToFile(vector<vector<vector<map<string, int>>>>& Set, vector<vector<vector<int>>> elementCounter) 
{
	ofstream output;
	output.open("Routes.txt", ios::app);

	output << "RofIVT: " << endl;

	for (int i = 0; i < Set.size(); i++) {
		for (int v = 0; v < Set[i].size(); v++) {
			for (int t = 0; t < Set[i][v].size(); t++) {
				output << "Number of Set elements: " << elementCounter[i][v][t] << endl;

				map<string, int>::const_iterator iter;
				for (iter = Trades.begin(); iter != Trades.end(); iter++) {
					if (iter->second == i) {
						output << "Trades: " << iter->first << endl;
						break;
					}
				}

				for (iter = Ships.begin(); iter != Ships.end(); iter++) {
					if (iter->second == v) {
						output << "Ships: " << iter->first << endl;
						break;
					}
				}
				output << "Period: " << t << endl;

				map<string, int>::const_iterator iterSet;
				for (iterSet = Set[i][v][t].begin(); iterSet != Set[i][v][t].end(); iterSet++) {
					output << iterSet->first << " - " << iterSet->second << endl;
				}
				output << endl;
			}
		}
		output << endl;
	}
}