#pragma once

#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include <map>
#include <ios>
#include <vector>
#include <regex>
#include <limits>

using namespace std;

class ImportData {
public:
	ImportData() : Horizon(5), fileID("dataSmall6_1.dat") { importData("dataSmall6_1.dat"); }
	ImportData(string fileID, int timeHorizon) : Horizon(timeHorizon), fileID(fileID) {	importData(fileID); }


void importData(string fileID);
void writeSet(map<string, int>& Set, int& elementCounter);
void writeSet(vector<map<string, int>>& Set, vector<int>& elementCounter);
void writeSet(vector<vector<map<string, int>>>& Set, vector<vector<int>> elementCounter);
void writeSet(vector<vector<vector<map<string, int>>>>& Set, vector<vector<vector<int>>> elementCounter);
void writeParameter(vector<vector<double>>& Parameter);
void writeParameter(vector<vector<vector<double>>>& Parameter);
void writeParameter(vector<double>& Parameter);
void writeParameter(vector<int>& Parameter);
void writeParameter(vector<vector<int>>& Parameter);

void compareSets(const vector<vector<map<string, int>>>& Set, const vector<vector<int>>& elementCounter,const vector<vector<vector<map<string, int>>>>& SubSet, const vector<vector<vector<int>>>& elementCounterSub, const vector<map<string,int>>& index1, const int& index2);	//The two last indexes have to be of same type for each set
void writeSetToFile(vector<vector<map<string, int>>>& Set, vector<vector<int>> elementCounter);
void writeSetToFile(vector<vector<vector<map<string, int>>>>& Set, vector<vector<vector<int>>> elementCounter);

//nSets
int Horizon;		//Number of time periods
int Time;			//Including period 0
int nShips;
int nTrades;
int nTradesC;
int nTradesO;
int nProduct;

int nRoutes;
vector<int> nVoF;					//Number of ships at time t
vector<int> nVN;					//Number of ships at time t
vector<vector<int>> nRofVT;				//Number of different loops that can be serviced by ship v in time t
vector<vector<vector<int>>> nRofIVT;	//Number of different loops that can be made on trade i by ship v in time t
vector<int> nTradesOf;					//Number of trades available in period t
vector <int> nTradesCOf;				//Number of contracted trades available in period t
vector<int> nTradesOOf;					//Number of optional trades available in period t


//Sets
map<string,int> Ships;				//V, indexed by v
map<string,int> Trades;				//N, indexed by i
map<string,int> TradesC;			// N^C, indexed by i
map<string,int> TradesO;			// N^O, indexed by i
map<string,int> Product;			// K, indexed by k

map<string,int> Routes;						// R, loops indexed by r 
vector<map<string, int>> VoF;				//V_t, ships available in period t, indexed by v
vector<map<string, int>> VN;				//V_t^N, new ships available in the market at time t, indexed by v
vector<vector<map<string, int>>> RofVT;				//R_vt, Loops that can be made by ship v in time t, indexed by r
vector<vector<vector<map<string, int>>>> RofIVT;	//R_ivt, Loops that can be made on trade i by ship v in time t, indexed by r
vector<map<string, int>> TradesOf;					//N_t, trades available in period t, indexed by i
vector<map<string, int>> TradesCOf;					//N_t^C, contracted trades available in period t, indexed by i
vector<map<string, int>> TradesOOf;					//N_t^O, Optional trades...


//Parameters
int Scenario;							//Number of scenarios
vector<vector<double>> DeltasOM;		//The scenario modifier
vector<vector<vector<double>>> CspaceC; //Unit space charter cost
vector<vector<double>> CNB;			//New build cost
vector<vector<double>> CCI;				//Charter in cost
vector<vector<double>> RLU;				//R_{vts}^[LU], Lay-up savings
vector<vector<double>> RSC;				//Scrapping value
vector<double> RSV;						//Sunset value
vector<vector<vector<double>>> Demand;			//Demand
vector<vector<double>> MinFreq;				//Minumum frequency
vector<int> InitialPool;				//Y_v^IP, initial fleet/pool of ships of type v
vector<vector<int>> InitialNB;					//Y_vt^NB, New ships of type v ordered in the previous planning period delivered at time t 
vector<vector<double>> UnitRevenue;		//R_{it}^D, The revenue of transporting 1 unit of goods on trade i, in period t 
vector<vector<int>> Capacity;			//Q_{vk}, The capacity of product k on ship pf type v
vector<int> TotalCapacity;				//Q_{v], total capacity of goods on ship v
vector<vector<int>> Duration;			//Z_{vr}, Time it takes for a ship v to sail a loop r
vector<vector<vector<double>>> LoopCost;	//C_{vrt}^TR, the cost for ship v to perform loop r in period t
vector<vector<double>> OperatingCost;		//C_{vt}^OP, The operating cost of ship v in period t
vector<int> CharterInLimit;					//CI_{v}, charter in limit for each ship of type v
vector<int> CharterOutLimit;				//CO_{v}, charter out limit for each ship of type v
vector<int> sellingLimit;					//SE_{v}, second hand selling limit for each ship of type v
vector<int> buyingLimit;					//SH_{v}, second hand buyung limit for each ship of type v
vector<vector<double>> secondHand;					//R_{vt}^{SE} = C_{vt}^{SH}, the second hand price for each ship of type v at period t
int CharterInLimitTot;						//CI, Total charter in limit
int CharterOutLimitTot;						//CO, Total charter out limit
int sellingLimitTot;						//SE, Total selling limit
int buyingLimitTot;							//SH, Total buying limit
vector<double> Prob;						//P_s, probability of scenario s occuring


string fileID;


//Robust formulation
vector<double> Lt;			//The robustness factor, multiplied with revenue to control negative cashflow
double L;					// The maximum of the vector Lt;

private:
	void readSet(map<string, int>& Set, int& elementCounter, regex& regColon);
	void readSubSet(map<string, int>& SubSet, map<string, int>& Set, int& elementCounter, regex& regColon);
	void readSubSet(vector<map<string, int>>& SubSet, map<string, int>& Set, vector<int>& elementCounter, regex& regColon);
	void readSubSet(vector<vector<map<string, int>>>& SubSet, map<string, int>& Set, vector<vector<int>>& elementCounter, regex& regColon);
	void readSubSet(vector<vector<vector<map<string, int>>>>& SubSet, vector<vector<map<string, int>>>& Set, vector<vector<vector<int>>>& elementCounter, int& index1, int& index2, int& index3, regex& regColon);

	void createTimeSubSet(map<string, int> & Set, vector<map<string, int>> & SubSet, vector<int>& elementCounter, int& index1);

	void readInt(int& integer, regex& regColon);
	void readParameter(vector<vector<double>>& Parameter, int& index1, int& index2);
	void readParameter(vector<vector<vector<double>>>& Parameter, int &index1, int& index2, int& index3, map<string, int> Set1, map<string, int> Set2, regex & regColon);
	void readParameter(vector<vector<double>>& Parameter, int& index1, int& index2, map<string,int>& Set1, regex& regColon);
	void readParameter(vector<double>& Parameter, int&index1, map<string, int> Set1, regex& regColon);
	void readParameter(vector<int>& Parameter, int&index1, map<string, int> Set1, regex& regColon);
	void readParameter(vector<vector<int>>& Parameter, int& index1, int& index2, map<string, int>& Set1, regex& regColon);
	void readParameter(vector<vector<int>>& Parameter, int& index1, int& index2, map<string, int>& Set1, map<string,int>& Set2, regex& regColon);
	void readParameter(vector<double>& Parameter, int& index1);

	void initializeParameter(const vector<double>& Parameter1, double & Parameter2);


	void trimFunction(string& trimmingString);


	ifstream inStream;				//Input stream
	string row;						//Each line is saved to this string
	int linePos;					//Used to record line position
	string regexParName;
	smatch match;					
	char trimCond;					//Trimming condition
	char trimCond2;
	char trimCond3;
	char trimCond4;
	char trimCond5;
	string regexColon;				//Used to search for ':'
	string regexPattern;			//Used to define a pattern to search for
	string regexPattern2;			//Used to define a pattern to search for
	string regexPattern3;
	string regexPattern4;
	string regexPattern5;
	string regexPattern6;
	string trim;					//Used to save trimmed strings, e.g to remove "'"
	string temp;					//Used for temporary storage

};

