#include "nodeData.h"



void nodeData::calcInstalment(const ImportData& idata)
{
	double inf = (numeric_limits<double>::max)();

	Instalment.resize(idata.nShips);
	for (int v = 0; v < idata.nShips; v++) {
		Instalment[v].resize(Nodes.size());
		for (int a = 0; a < Nodes.size(); a++) {
			Instalment[v][a].resize(Nodes.size() + NodesTime[idata.Horizon].size()*M);
			for (int n = 0; n < (Nodes.size() + NodesTime[idata.Horizon].size()*M); n++) {
				Instalment[v][a][n] = inf;
			}
		}
	}

	
	//Instalment nodes - same as NodesTime, but for a horizon going longer than the planning period
	InstalmentNodes.resize(idata.Time + M);
	InstNodes.resize(Nodes.size()+NodesTime[idata.Horizon].size()*M);

	for (int i = 0; i < Nodes.size(); i++) {
		InstNodes[i] = Nodes[i];
	}
	ReverseInstalmentNodes.resize(idata.Time + M);

	for (int t = 0; t < idata.Time; t++) {
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
			int n = iterNodes->second;
			InstalmentNodes[t][iterNodes->first] = n;
			ReverseInstalmentNodes[t][iterNodes->second] = iterNodes->first;
		}
	}
	int numberOfNodes = Nodes.size();
	for (int t = idata.Time; t < (idata.Time + M); t++) {
		for (int i = 0; i < NodesTime[idata.Horizon].size(); i++) {
			InstalmentNodes[t][i] = numberOfNodes;
			ReverseInstalmentNodes[t][numberOfNodes] = i;
			InstNodes[numberOfNodes] = numberOfNodes;
			numberOfNodes++;
		}
	}
	//AncestorInstalmentNodes
	AncestorInstalmentNodes.resize(idata.Horizon + M);
	for (int t = 0; t < idata.Horizon + M; t++) {
		AncestorInstalmentNodes[t].resize(idata.Time + M);
	}
	for (int i = 0; i < idata.Horizon+M; i++) {
		int branchSize = 0;
		
	
		for (int t = 1; t < idata.Time+M; t++) {
			int iterator = 0;
			int position = 1;
			map<int, int> node;
			if (t > i) {
				map<int, int>::const_iterator iterPast;
				for (iterPast = InstalmentNodes[i].begin(); iterPast != InstalmentNodes[i].end(); iterPast++) {



					bool flag = false;
					map<int, int>::const_iterator iterNow;
					for (iterNow = InstalmentNodes[t].begin(); iterNow != InstalmentNodes[t].end(); iterNow++) {
						if (InstalmentNodes[t].size() == InstalmentNodes[i].size()) {

							if (iterNow->first == iterPast->first) {

								node[iterNow->second] = iterPast->second;
								flag = true;
							}

							if (flag) {
								break;
							}
						}
						else {
							branchSize = InstalmentNodes[t].size() / InstalmentNodes[i].size();
							if (InstalmentNodes[t].size() % InstalmentNodes[i].size() != 0) {
								cout << "Error: The branches has to increase by the same factor in every branch" << endl;
							}
							else if (iterator < branchSize*position && iterator <= iterNow->first) {
								node[iterNow->second] = iterPast->second;
								iterator++;
							}


						}
					}
					position++;

				}
				AncestorInstalmentNodes[i][t] = node;
			}

		}
	}
	
	/*
	writeSet(InstalmentNodes);
	cout << "Ancestor nodes: " << endl;
	writeSet(AncestorInstalmentNodes);
	*/

	for (int tmarked = 0; tmarked < (idata.Time - buildTime); tmarked++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VN[tmarked + buildTime].begin(); iterShips != idata.VN[tmarked + buildTime].end(); iterShips++) {
			int v = iterShips->second;
			map<int, int>::const_iterator iterNodes;
			for (int t = tmarked+1; t < (tmarked + M); t++) {
				for (iterNodes = AncestorInstalmentNodes[tmarked][t].begin(); iterNodes != AncestorInstalmentNodes[tmarked][t].end(); iterNodes++) {
					int a = iterNodes->second;
					int n = iterNodes->first;
					if (CNB[v][a] > 0) {
						Instalment[v][a][a] = CNB[v][a] *interest *pow(1+interest,M)/(pow(1+interest,M)-1);			//Annualized loan instalments
						Instalment[v][a][n] = CNB[v][a] * interest *pow(1 + interest, M) / (pow(1 + interest, M) - 1) * (1/pow(1+discountFactor,t-tmarked));	//Multiplying the instalment with the proper discount factor
					}
				}
			}
		}
	}
}

void nodeData::writeSet(vector<map<int, int>> Set)
{
	for (int i = 0; i < Set.size(); i++) {
		cout << "Period: " << i << endl;

		map<int, int>::const_iterator iter;
		for (iter = Set[i].begin(); iter != Set[i].end(); iter++) {
			cout << iter->first << "	-	" << iter->second << endl;
		}
		cout << endl;
	}
}

void nodeData::writeSet(vector<vector<map<int,int>>> Set)
{
	for (int i = 0; i < Set.size(); i++) {
		

		for (int j = 0; j < Set[i].size(); j++) {
			if (j > i) {
				cout << "(Ancestor Period: " << i << ", Node period: " << j << ")" << endl;

				map<int, int>::const_iterator iter;
				for (iter = Set[i][j].begin(); iter != Set[i][j].end(); iter++) {
					cout << iter->first << "	-	" << iter->second << endl;
				}
			}
		}
		}

	
		cout << endl;
	}

void nodeData::registerNodes(const int & numberOfStages,const int& numberOfScenarios, const ImportData& idata,const string& path, const int& branchPeriod)
{
	
	
	
	switch (numberOfStages) {
	case 2: {

		string stages = to_string((_Longlong)numberOfStages);
		string scenarios = to_string((_Longlong)numberOfScenarios);
		string fileType = ".txt";


		//Setting the first stage to 0, no realisation of the random variables yet
		for (int t = 0; t < branchPeriod; t++) {
			vector<double> n;
			n.push_back(0);
			vector<vector<double>> s;
			s.push_back(n);
			randomVariable.push_back(s);
		}


		for (int t = branchPeriod; t < idata.Time; t++) {
			string time = to_string((_Longlong)t);
			string fileName = "T" + time + "Stage" + stages + "Node" + scenarios;
			string fileID = path + fileName + fileType;

			vector<vector<double>> random;					//Random variable, (node, variable type)
			readParameter(fileID, random);

			randomVariable.push_back(random);

		}

		for (int i = branchPeriod; i < randomVariable.size(); i++) {
			for (int j = 0; j < randomVariable[i].size(); j++) {
				randomVariable[i][j].push_back(0);					//Adding a column with 0
			}
		}

		string fileID = path + "FullTree" + fileType;
		writeParameter(randomVariable, fileID);

		break;

	}

	case 3: {
		string stages = to_string((_Longlong)numberOfStages);
		string scenarios = to_string((_Longlong)numberOfScenarios);
		string fileType = ".txt";


		//Setting the first stage to 0, no realisation of the random variables yet
		vector<double> n;
		n.push_back(0);
		vector<vector<double>> s;
		s.push_back(n);
		randomVariable.push_back(s);


		for (int t = 1; t < idata.Time; t++) {
			string time = to_string((_Longlong)t);
			if (t == 3) {
				scenarios = to_string((_Longlong)numberOfScenarios * 2);
			}
			string fileName = "T" + time + "Stage" + stages + "Node" + scenarios;
			string fileID = path + fileName + fileType;

			vector<vector<double>> random;					//Random variable, (node, variable type)
			readParameter(fileID, random);

			randomVariable.push_back(random);

		}

		for (int i = 0; i < randomVariable.size(); i++) {
			for (int j = 0; j < randomVariable[i].size(); j++) {
				randomVariable[i][j].push_back(0);					//Adding a column with 0
			}
		}

		string fileID = path + "FullTree" + fileType;
		writeParameter(randomVariable, fileID);
		break;

	}
	default: {
		cout << "No implementation exist for " << numberOfStages << " stages" << endl;
	}
	}
			
	
	
		int numberOfNodes = 0;
		for (int i = 0; i < randomVariable.size(); i++) {

			map<int, int> node;
			for (int j = 0; j < randomVariable[i].size(); j++) {
				Nodes.push_back(numberOfNodes);
				node[j] = numberOfNodes;
				numberOfNodes++;
				
			}
			NodesTime.push_back(node);
		}


		//Ancestor nodes
		AncestorNodes.resize(idata.Horizon);
		for (int i = 0; i < idata.Horizon; i++) {
			int branchSize = 0;							//Used to determine the size if each branch
			map<int, int> node;
			for (int t = 1; t < idata.Time; t++) {
				int iterator = 0;
				int position = 1;						//

				if (t > i) {
					map<int, int>::const_iterator iterPast;
					for (iterPast = NodesTime[i].begin(); iterPast != NodesTime[i].end(); iterPast++) {
					
						
						
						bool flag = false;
						map<int, int>::const_iterator iterNow;
						for (iterNow = NodesTime[t].begin(); iterNow != NodesTime[t].end(); iterNow++) {
							if (NodesTime[t].size() == NodesTime[i].size()) {

								if (iterNow->first == iterPast->first) {

									node[iterNow->second] = iterPast->second;
									flag = true;
								}

								if (flag) {
									break;
								}
							}
							else {
								branchSize = NodesTime[t].size() / NodesTime[i].size();
								if (NodesTime[t].size() % NodesTime[i].size() != 0) {
									throw "Error: The branches has to increase by the same factor in every branch";
								}
								else if (iterator < branchSize*position && iterator <= iterNow->first) {
										node[iterNow->second] = iterPast->second;
										iterator++;
								}
								

							}
						}
						position++;
						
					}
					AncestorNodes[i] = node;
				}

			}
		}

		
/*
		writeSet(NodesTime);
		writeSet(AncestorNodes);
		*/
		

		convertDataToNode(idata);

		

		
	
	}

	void nodeData::registerNodes(const int & numberOfStages, const int & numberOfScenarios, const ImportData & idata, const string & path)
	{
		switch (numberOfStages) {
		case 2: {

			string stages = to_string((_Longlong)numberOfStages);
			string scenarios = to_string((_Longlong)numberOfScenarios);
			string fileType = ".txt";


			//Setting the first node to 0, no realisation of the random variables yet
			for (int t = 0; t < branchPeriods[0]; t++) {
				vector<double> n;
				n.push_back(0);
				vector<vector<double>> s;
				s.push_back(n);
				randomVariable.push_back(s);
			}


			for (int t = branchPeriods[0]; t < idata.Time; t++) {
				string time = to_string((_Longlong)t);
				string fileName = "T" + time + "Stage" + stages + "Node" + scenarios;
				string fileID = path + fileName + fileType;

				vector<vector<double>> random;					//Random variable, (node, variable type)
				readParameter(fileID, random);

				randomVariable.push_back(random);

			}

			for (int i = branchPeriods[0]; i < randomVariable.size(); i++) {
				for (int j = 0; j < randomVariable[i].size(); j++) {
					randomVariable[i][j].push_back(0);					//Adding a column with 0
				}
			}

			string fileID = path + "FullTree" + fileType;
			writeParameter(randomVariable, fileID);

			break;

		}

		case 3: {
			string stages = to_string((_Longlong)numberOfStages);
			string scenarios = to_string((_Longlong)numberOfScenarios);
			string fileType = ".txt";


			//Setting the first node to 0, no realisation of the random variables yet
			for (int i = 0; i < branchPeriods[0]; i++) {
				vector<double> n;
				n.push_back(0);
				vector<vector<double>> s;
				s.push_back(n);
				randomVariable.push_back(s);
			}

			string fileName;
			for (int t = branchPeriods[0]; t < idata.Time; t++) {
				string time = to_string((_Longlong)t);
				if (t == branchPeriods[1]) {
					string nodeTime = to_string((_Longlong)0);
					fileName = "T" + time + "Stage" + stages + "Node" + nodeTime + "Scenario" + scenarios;
					string fileID = path + fileName + fileType;
					vector<vector<double>> random;					//Random variable, (node, variable type)
					readParameter(fileID, random);

					for (int i = 1; i < numberOfScenarios; i++) {
						string nodeTime = to_string((_Longlong)i);
						fileName = "T" + time + "Stage" + stages + "Node" + nodeTime + "Scenario" + scenarios;
						string fileID = path + fileName + fileType;

						appendParameter(fileID, random);
						
					}
					randomVariable.push_back(random);
				}
				else {
					fileName = "T" + time + "Stage" + stages + "Node" + scenarios;
					string fileID = path + fileName + fileType;

					vector<vector<double>> random;					//Random variable, (node, variable type)
					readParameter(fileID, random);

					randomVariable.push_back(random);
				}
				

			}

			for (int i = branchPeriods[0]; i < randomVariable.size(); i++) {
				for (int j = 0; j < randomVariable[i].size(); j++) {
					randomVariable[i][j].push_back(0);					//Adding a column with 0
				}
			}

			string fileID = path + "FullTree" + fileType;
			writeParameter(randomVariable, fileID);
			break;

		}
		default: {
			cout << "No implementation exist for " << numberOfStages << " stages" << endl;
		}
		}



		int numberOfNodes = 0;
		for (int i = 0; i < randomVariable.size(); i++) {

			map<int, int> node;
			for (int j = 0; j < randomVariable[i].size(); j++) {
				Nodes.push_back(numberOfNodes);
				node[j] = numberOfNodes;
				numberOfNodes++;

			}
			NodesTime.push_back(node);
		}


		//Ancestor nodes
		AncestorNodes.resize(idata.Horizon);
		for (int i = 0; i < idata.Horizon; i++) {
			int branchSize = 0;							//Used to determine the size if each branch
			map<int, int> node;
			for (int t = 1; t < idata.Time; t++) {
				int iterator = 0;
				int position = 1;						//

				if (t > i) {
					map<int, int>::const_iterator iterPast;
					for (iterPast = NodesTime[i].begin(); iterPast != NodesTime[i].end(); iterPast++) {



						bool flag = false;
						map<int, int>::const_iterator iterNow;
						for (iterNow = NodesTime[t].begin(); iterNow != NodesTime[t].end(); iterNow++) {
							if (NodesTime[t].size() == NodesTime[i].size()) {

								if (iterNow->first == iterPast->first) {

									node[iterNow->second] = iterPast->second;
									flag = true;
								}

								if (flag) {
									break;
								}
							}
							else {
								branchSize = NodesTime[t].size() / NodesTime[i].size();
								if (NodesTime[t].size() % NodesTime[i].size() != 0) {
									throw "Error: The branches has to increase by the same factor in every branch";
								}
								else if (iterator < branchSize*position && iterator <= iterNow->first) {
									node[iterNow->second] = iterPast->second;
									iterator++;
								}


							}
						}
						position++;

					}
					AncestorNodes[i] = node;
				}

			}
		}


		/*
		writeSet(NodesTime);
		writeSet(AncestorNodes);
		*/


		convertDataToNode(idata);
	}

	void nodeData::defaultRegister()
	{
		string path = "C:\\Users\\jorge\\OneDrive\\Dokumenter\\Visual Studio 2017\\Projects\\MFRP\\MFRP\\ScenarioTrees\\Project-scenario\\";
			string fileName = "T1Stage2Node6";
		string fileType = ".txt";

		string fileID = path + fileName + fileType;

		readParameter(fileID, randomTest);

		writeParameter(randomTest);
	}



template<class T>
void nodeData::readParameter(string & fileID, vector<vector<T>>& Parameter)
{
	ifstream input;

	input.open(fileID);

	vector<vector<string>> matrix;
	string row;
	string value;

	while (getline(input, row)) {		//Reading one row of the data file
		istringstream tmp(row);			//making the row an input stream
		vector<string> record;			//Temporary storage

		while (tmp.good()) {
			getline(tmp, value, '\t');	//Reads word by word
			record.push_back(value);
		}
		matrix.push_back(record);
	}

	Parameter.resize(matrix.size()-1);
	for (int i = 1; i < matrix.size(); i++) {		//Initialize the parameter
		Parameter[i-1].resize(matrix[i].size());
	}

	for (int i = 1; i < matrix.size(); i++) {
		for (int j = 0; j < matrix[i].size(); j++) {
			stringstream convert(matrix[i][j]);		//Converts the data to correct type
			int k = i - 1;
			convert >> Parameter[k][j];
		}
	}



	input.close();

}

template<class T>
void nodeData::appendParameter(string & fileID, vector<vector<T>>& Parameter)
{
	ifstream input;

	input.open(fileID);

	vector<vector<string>> matrix;
	string row;
	string value;

	while (getline(input, row)) {		//Reading one row of the data file
		istringstream tmp(row);			//making the row an input stream
		vector<string> record;			//Temporary storage

		while (tmp.good()) {
			getline(tmp, value, '\t');	//Reads word by word
			record.push_back(value);
		}
		matrix.push_back(record);
	}

	for (int i = 1; i < matrix.size(); i++) {
		vector<T> tempPar;
		for (int j = 0; j < matrix[i].size(); j++) {
			tempPar.push_back(0);
			stringstream convert(matrix[i][j]);		//Converts the data to correct type

			convert >> tempPar[j];
		}
		Parameter.push_back(tempPar);
	}


	input.close();


}

template<class T>
void nodeData::writeParameter(vector<vector<T>>& Parameter, const string& fileID)
{
	ofstream output;

	output.open(fileID);

	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			output << Parameter[i][j];
			output << "\t";
		}
		output << endl;
	}

	output.close();
}

template<class T>
void nodeData::writeParameter(vector<vector<vector<T>>>& Parameter,const string& fileID)
{
	ofstream output;

	output.open(fileID);


	for (int i = 0; i < Parameter.size(); i++) {
		output << "Period: " << i << endl;
		for (int j = 0; j < Parameter[i].size(); j++) {
			for (int k = 0; k < Parameter[i][j].size(); k++) {

				output << Parameter[i][j][k];
				output << "\t";
			}
			output << endl;
		}
		output << endl;
		output << endl;
		
	}

	output.close();

}

void nodeData::convertDataToNode(const ImportData & idata)
{

	//Space charter cost
	convertParameterIncrement(CspaceC, idata.CspaceC, idata.nTrades, idata.nProduct, idata.Time , 0, 4, false, true);
	//nodeConvert(CspaceC, idata.CspaceC, idata.nTrades, idata.nProduct, idata.Time, 3); 

	

	//New build cost
	convertParameterIncrement(CNB, idata.CNB, idata.nShips, idata.Time, 1, 4, false);
	//nodeConvert(CNB, idata.CNB, idata.nShips, idata.Time, 0);

	//Second hand price
	convertParameterIncrement(secondHand, idata.secondHand, idata.nShips, idata.Horizon, 0, 1, false);
	
	//Charter in cost
	convertParameterIncrement(CCI, idata.CCI, idata.nShips, idata.Time, 1, 4, true);
	//nodeConvert(CCI, idata.CCI, idata.nShips, idata.Time, 0);

	

	//Lay-up savings
	convertParameterIncrement(RLU, idata.RLU, idata.nShips, idata.Time, 0, 1, false);
	//nodeConvert(RLU, idata.RLU, idata.nShips, idata.Time, 3);


	//Scrapping value
	convertParameterIncrement(RSC, idata.RSC, idata.nShips, idata.Horizon, 1, 2, true);

	//nodeConvert(RSC, idata.RSC, idata.nShips, idata.Horizon, 2);


	//Sunset value
	convertParameterIncrement(RSV, idata.RSV, idata.nShips, idata.Horizon, 1);

	//nodeConvert(RSV, idata.RSV, idata.nShips, idata.Horizon, 0);
	

	//Demand
	convertParameterIncrement(Demand, idata.Demand, idata.nTrades, idata.nProduct, idata.Time, 1, 0, true, false);
	//nodeConvert(Demand, idata.Demand, idata.nTrades, idata.nProduct, idata.Time, 0);

	//Minimum frequency
	convertParameterIncrement(MinFreq, idata.MinFreq, idata.nTrades, idata.Time, 0, 1, false);
	//nodeConvert(MinFreq, idata.MinFreq, idata.nTrades, idata.Time, 3);	

	//Initital new builds
	convertParameterIncrement(InitialNB, idata.InitialNB, idata.nShips, idata.Time, 0, 1, false);
	//nodeConvert(InitialNB, idata.InitialNB, idata.nShips, idata.Time, 3);

	//Unit revenue
	convertParameterIncrement(UnitRevenue, idata.UnitRevenue, idata.nTrades, idata.Time, 0, 1, false);
	//nodeConvert(UnitRevenue, idata.UnitRevenue, idata.nTrades, idata.Time, 0);

	//Loop cost
	convertParameterIncrement(LoopCost, idata.LoopCost, idata.nShips, idata.nRoutes, idata.Time, 1, 3, false, false);
	//nodeConvert(LoopCost, idata.LoopCost, idata.nShips, idata.nRoutes, idata.Time, 1);
	
	//Operating cost
	convertParameterIncrement(OperatingCost, idata.OperatingCost, idata.nShips, idata.Time, 0, 1, false);
	//nodeConvert(OperatingCost, idata.OperatingCost, idata.nShips, idata.Time, 3);

	//Calculating instalments
	calcInstalment(idata);

	//Probability
	calcProb(Prob, InstalmentNodes, idata.Time);

	//Total time available for a ship of type v
	calcAvailableTime(availableTime, idata.nShips);

	//Calculating second hand prices and sunet value
	calcSecondHand(idata, 1);


	

	//compareParameter(idata.Duration, idata.Ships, idata.Routes);
	//compareParameter(LoopCost, idata.Time, idata.VoF, idata.Routes);
	//Writing the new parameters
	/*
	writeDataParameter(CspaceC);
	cout << "CNB: " << endl;
	writeParameter(CNB);
	cout << "Instalmetns: " << endl;
	writeDataParameter(Instalment);
	cout << "Second hand price: " << endl;
	writeParameter(secondHand);
	cout << "Charter in cost: " << endl;
	writeParameter(CCI);
	cout << "Lay-up savings: " << endl;
	writeParameter(RLU);
	cout << "Scrapping value: " << endl;
	writeParameter(RSC);
	cout << "Sunset value: " << endl;
	writeParameter(RSV);
	cout << "Demand: " << endl;
	writeDataParameter(Demand);
	cout << "Minimum frequency: " << endl;
	writeParameter(MinFreq);
	cout << "Initial NB: " << endl;
	writeParameter(InitialNB);
	cout << "Unit revenue: " << endl;
	writeParameter(UnitRevenue);
	cout << "Loop cost: " << endl;
	writeDataParameter(LoopCost);
	cout << "Operating cost: " << endl;
	writeParameter(OperatingCost);
	cout << "Probability: " << endl;
	writeParameter(Prob);
	*/
	
}

template<class T>
void nodeData::nodeConvert(vector<vector<vector<T>>>& Parameter, const vector<vector<vector<T>>> & Data, const int & index1, const int & index2, const int& index3, const int& random)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
		for (int k = 0; k < index2; k++) {
			Parameter[i][k].resize(Nodes.size());
		}
	}

	//Assigning the parameter the correct values
	for (int i = 0; i < index1; i++) {
		for (int k = 0; k < index2; k++) {
			for (int t = 0; t < index3; t++) {
				map<int, int>::const_iterator iter;
				for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
					int n = iter->second;

					Parameter[i][k][n] = (1+randomVariable[t][iter->first][random]) * Data[i][k][t];
				}
			}
		}
	}
}

template<class T>
void nodeData::nodeConvert(vector<vector<T>>& Parameter, const vector<vector<T>>& Data, const int& index1, const int &index2, const int & random)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(Nodes.size());
	}

	//Assignment
	for (int i = 0; i < index1; i++) {
		for (int t = 0; t < index2; t++) {
			map<int, int>::const_iterator iter;
			for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
				int n = iter->second;
				if (t == 0) {
					Parameter[i][n] = Data[i][t];
				}
				else{
					Parameter[i][n] = (1 + randomVariable[t][iter->first][random]) * Data[i][t];
				}
			}
		}
	}

}

template<class T>
void nodeData::nodeConvert(vector<vector<T>>& Parameter, const vector<T>& Data, const int & index1, const int & horizon, const int & random)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(Nodes.size());
	}

	//Assignment
	for (int i = 0; i < index1; i++) {
		map<int, int>::const_iterator iter;
		for (iter = NodesTime[horizon].begin(); iter != NodesTime[horizon].end(); iter++) {
			int n = iter->second;

			Parameter[i][n] = (1 + randomVariable[horizon][iter->first][random])*Data[i];
		}
	}

}

template<class T>
void nodeData::nodeConvert(T & Parameter, const T & forecastParameter, const int & index1, const int & index2, const int & index3)
{
	Parameter = forecastParameter*(1 + randomVariable[index1][index2][index3]);
}

template<class T>
void nodeData::nodeConvertDiscount(T & Parameter, const T & forecastParameter, const int & index1, const int & index2, const int & index3)
{
	Parameter = forecastParameter*(1 + randomVariable[index1][index2][index3])*(1/pow(1+discountFactor, index1-1));
}

template<class T>
void nodeData::convertParameter(vector<vector<vector<T>>>& Parameter, const vector<vector<vector<T>>>& Data, const int & index1, const int & index2, const int & index3, const int & random)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
		for (int k = 0; k < index2; k++) {
			Parameter[i][k].resize(Nodes.size());
		}
	}

	//Assigning the parameter the correct values

	int counter = 0;		//Used to loop through the random variables
	for (int i = 0; i < index1; i++) {
		for (int k = 0; k < index2; k++) {
			for (int t = 0; t < index3; t++) {
				map<int, int>::const_iterator iter;
				for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
					int n = iter->second;
					int s = iter->first;

					switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
					case 0: {
						nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
						break;

					}

					case 1: {
						nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, counter);
						break;
					}

					default: {
						throw "Error converting paramter to node formulation";
					}

					}




				}
			}
			counter++;
		}
	}
}

template<class T>
void nodeData::convertParameterIncrement(vector<vector<vector<T>>>& Parameter, const vector<vector<vector<T>>>& Data, const int & index1, const int & index2, const int & index3, const int & random, const int& variableChoice, const bool& timeDependent, const bool& discount)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(index2);
		for (int k = 0; k < index2; k++) {
			Parameter[i][k].resize(Nodes.size());
		}
	}

	//Initializaton
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			for (int k = 0; k < Parameter[i][j].size(); k++) {
				Parameter[i][j][k] = 0;
			}
		}
	}


	//Assigning the parameter the correct values

	if (variableChoice != 0) {
		for (int i = 0; i < index1; i++) {
			for (int k = 0; k < index2; k++) {
				for (int t = 0; t < branchPeriod; t++) {
					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;

						switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
						case 0: {
							if (discount) {
								nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							else {
								nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							break;

						}

						case 1: {
							if (discount) {
								nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							else {
								nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, randomVariable[t][s].size() - 1);
							}
							break;
						}

						default: {
							throw "Error converting parameter to node formulation";
						}

						}




					}
				}

			}
		}

		// Incrementing paramters from previous periods parameteter

		for (int i = 0; i < index1; i++) {
			for (int k = 0; k < index2; k++) {


				switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
				case 0: {

					for (int t = branchPeriod; t < index3; t++) {
						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;

							if (discount) {
								nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							else {
								nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}

						}
					}
					break;
				}

				case 1: {
					if (timeDependent) {
						for (int t = branchPeriod; t < index3; t++) {

							if (Data[i][k][t - 1] > 0) {

								map<int, int>::const_iterator iter;
								for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
									int n = iter->second;
									int s = iter->first;

									map<int, int>  temp = AncestorNodes[t - 1];
									int a = temp[n];
									if (discount) {
										nodeConvertDiscount(Parameter[i][k][n], Parameter[i][k][a], t, s, (randomVariable[t][s].size() - variableChoice));
									}
									else {
										nodeConvert(Parameter[i][k][n], Parameter[i][k][a], t, s, randomVariable[t][s].size() - variableChoice);
									}
								}
							}


							else {


								map<int, int>::const_iterator iter;
								for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
									int n = iter->second;
									int s = iter->first;

									if (discount) {
										nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - variableChoice));
									}
									else {
										nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, randomVariable[t][s].size() - variableChoice);
									}
								}
							}
						}
					}

					else {
						for (int t = branchPeriod; t < index3; t++) {
							map<int, int>::const_iterator iter;
							for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
								int n = iter->second;
								int s = iter->first;

								if (discount) {
									nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - variableChoice));
								}
								else {
									nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, randomVariable[t][s].size() - variableChoice);
								}
							}
						}
					}
					break;
				}

				default: {
					throw "Error converting parameter to node formulation";
				}

				}
			}
		}
	}


	else {

		
		for (int i = 0; i < index1; i++) {
			for (int k = 0; k < index2; k++) {
				for (int t = 0; t < branchPeriod; t++) {
					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;

						switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
						case 0: {
							if (discount) {
								nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							else {
								nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							break;

						}

						case 1: {
							if (discount) {
								nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							else {
								nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, randomVariable[t][s].size() - 1);
							}
							break;
						}

						default: {
							throw "Error converting parameter to node formulation";
						}

						}




					}
				}
				
			}
		}

		// Incrementing paramters from previous periods parameteter
		int counter = 0;

		for (int i = 0; i < index1; i++) {
			for (int k = 0; k < index2; k++) {


				switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
				case 0: {
					for (int t = branchPeriod; t < index3; t++) {
						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;
							if (discount) {
								nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
							else {
								nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, (randomVariable[t][s].size() - 1));
							}
						}
					}

					break;
				}


				case 1: {
					if (timeDependent) {

						for (int t = branchPeriod; t < index3; t++) {

							if (Data[i][k][t - 1] > 0) {

								map<int, int>::const_iterator iter;
								for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
									int n = iter->second;
									int s = iter->first;

									map<int, int>  temp = AncestorNodes[t - 1];
									int a = temp[n];
									if (discount) {
										nodeConvertDiscount(Parameter[i][k][n], Parameter[i][k][a], t, s, counter);
									}
									else {
										nodeConvert(Parameter[i][k][n], Parameter[i][k][a], t, s, counter);
									}
								}
							}

							else {

								map<int, int>::const_iterator iter;
								for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
									int n = iter->second;
									int s = iter->first;

									if (discount) {
										nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, counter);
									}
									else {
										nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, counter);
									}
								}
							}
						}
					}
					else {
						for (int t = branchPeriod; t < index3; t++) {
							map<int, int>::const_iterator iter;
							for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
								int n = iter->second;
								int s = iter->first;

								if (discount) {
									nodeConvertDiscount(Parameter[i][k][n], Data[i][k][t], t, s, counter);
								}
								else {
									nodeConvert(Parameter[i][k][n], Data[i][k][t], t, s, counter);
								}
							}
						}
					}
					break;
				}

				default: {
					throw "Error converting parameter to node formulation";
				}
				}
			}
		}
	}
}


template<class T>
void nodeData::convertParameter(vector<vector<T>>& Parameter, const vector<vector<T>>& Data, const int & index1, const int & index2, const int & random, const int & variableChoice)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(Nodes.size());
	}

	//Assigning the parameter the correct values


	if (variableChoice != 0) {	//Variable choice chooses which one of the random variables market contidion (3), steel price (2) or fuel price(1) to select. 0 = trades-and-product random variable

		switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
		case 0: {

			for (int i = 0; i < index1; i++) {
				for (int t = 0; t < index2; t++) {

					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;

						nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - 1));
					}
				}
			}
			break;

		}

		case 1: {

			for (int i = 0; i < index1; i++) {
				bool flag = false;
				for (int t = 0; t < index2; t++) {

					if (!flag) {
						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;

							nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - variableChoice));

						}
					}


					if (flag) {
						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;
							map<int, int> temp = AncestorNodes[t - 1];
							int a = temp[n];

							nodeConvert(Parameter[i][n], Parameter[i][a], t, s, (randomVariable[t][s].size() - variableChoice));
						}

					}
					if (Data[i][t] != 0) {
						flag = true;
					}
				}

			}
			break;
		}

		default: {
			throw "Error converting parameter to node formulation";
		}
		}
	}

	else {

		switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
		case 0: {

			for (int i = 0; i < index1; i++) {
				for (int t = 0; t < index2; t++) {
					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;


						nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - 1));


					}
				}
			}
			break;
		}

		case 1: {
			int counter = 0;
			for (int i = 0; i < index1; i++) {
				bool flag = false;
				for (int t = 0; t < index2; t++) {

					if (!flag) {
						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;

							nodeConvert(Parameter[i][n], Data[i][t], t, s, counter);

						}
					}

					if (flag) {
						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;
							map<int, int> temp = AncestorNodes[t - 1];
							int a = temp[n];

							nodeConvert(Parameter[i][n], Parameter[i][a], t, s, counter);
						}
					}
					if (Data[i][t] != 0) {
						flag = true;
					}
				}
				counter++;
			}
			break;
		}

		default: {
			throw "Error converting parameter to node formulation";
		}
		}
	}
	
}

template<class T>
void nodeData::convertParameterIncrement(vector<vector<T>>& Parameter, const vector<vector<T>> & Data, const int & index1, const int & index2, const int& random, const int& variableChoice, const bool& timeDependent)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(Nodes.size());
	}

	//Assigning the parameter the correct values


	if (variableChoice != 0) {	//Variable choice chooses which one of the random variables market contidion (4), steel price (3) or fuel price(2) to select. 0 = trades-and-product random variable

		switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
		case 0: {

			for (int i = 0; i < index1; i++) {
				for (int t = 0; t < index2; t++) {

					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;

						nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - 1));
					}
				}
			}
			break;

		}

		case 1: {

			for (int i = 0; i < index1; i++) {
				bool flag = false;
				for (int t = 0; t < branchPeriod; t++) {

					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;

						nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - 1));

					}


				}
				if (timeDependent) {
					for (int t = branchPeriod; t < index2; t++) {

						if (Data[i][t - 1] > 0) {
							map<int, int>::const_iterator iter;
							for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
								int n = iter->second;
								int s = iter->first;
								map<int, int> temp = AncestorNodes[t - 1];
								int a = temp[n];

								nodeConvert(Parameter[i][n], Parameter[i][a], t, s, (randomVariable[t][s].size() - variableChoice));
							}
						}
						else {
							map<int, int>::const_iterator iter;
							for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
								int n = iter->second;
								int s = iter->first;


								nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - variableChoice));
							}
						}
					}
				}
				else {
					for (int t = branchPeriod; t < index2; t++) {

						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;
							

							nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - variableChoice));
						}
					}
				}
			}

			break;
		}

		default: {
			throw "Error converting parameter to node formulation";
		}
		}
	}

	else {

		switch (random) {	//If the parameter is deterministic or random, random = 0 ->deterministic, random = 1 ->random
		case 0: {

			for (int i = 0; i < index1; i++) {
				for (int t = 0; t < index2; t++) {
					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;


						nodeConvert(Parameter[i][n], Data[i][t], t, s, (randomVariable[t][s].size() - 1));


					}
				}
			}
			break;
		}

		case 1: {
			int counter = 0;
			for (int i = 0; i < index1; i++) {
				bool flag = false;
				for (int t = 0; t < branchPeriod; t++) {


					map<int, int>::const_iterator iter;
					for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
						int n = iter->second;
						int s = iter->first;

						nodeConvert(Parameter[i][n], Data[i][t], t, s, randomVariable[t][s].size()-1);

					}
				}

				if (timeDependent) {
					for (int t = branchPeriod; t < index2; t++) {

						if (Data[i][t - 1] > 0) {
							map<int, int>::const_iterator iter;
							for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
								int n = iter->second;
								int s = iter->first;
								map<int, int> temp = AncestorNodes[t - 1];
								int a = temp[n];

								nodeConvert(Parameter[i][n], Parameter[i][a], t, s, counter);
							}
						}
						else {
							map<int, int>::const_iterator iter;
							for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
								int n = iter->second;
								int s = iter->first;


								nodeConvert(Parameter[i][n], Data[i][t], t, s, counter);
							}
						}
					}
				}
				else {
					for (int t = branchPeriod; t < index2; t++) {

						map<int, int>::const_iterator iter;
						for (iter = NodesTime[t].begin(); iter != NodesTime[t].end(); iter++) {
							int n = iter->second;
							int s = iter->first;

							nodeConvert(Parameter[i][n], Data[i][t], t, s, counter);
						}
					}
				}
				counter++;
			}
			break;
		}

		default: {
			throw "Error converting parameter to node formulation";
		}
		}
	}

}

template<class T>
void nodeData::convertParameterIncrement(vector<vector<T>>& Parameter, const vector<T> & Data, const int & index1, const int & index2, const int& variableChoice)
{
	//Allocating memory
	Parameter.resize(index1);
	for (int i = 0; i < index1; i++) {
		Parameter[i].resize(Nodes.size());
	}

	//Initialising
	for (int j = 0; j < Parameter.size(); j++) {
		for (int k = 0; k < Parameter[j].size(); k++) {
			Parameter[j][k] = 0;
		}
	}

	//Assigning the parameter the correct values

	for (int i = 0; i < index1; i++) {

		map<int, int>::const_iterator iter;
		for (iter = NodesTime[index2].begin(); iter != NodesTime[index2].end(); iter++) {
			int n = iter->second;
			int s = iter->first;

			nodeConvert(Parameter[i][n], Data[i], index2, s, (randomVariable[index2][s].size() - variableChoice));
		}
	}
}



void nodeData::calcProb(vector<double>& Probability, const vector<map<int, int>>& Node, const int& time)
{
	Probability.resize(InstNodes.size());
	Probability[0] = 1.0;
	for (int i = 0; i < time-buildTime+M; i++) {
		double prob = 1.0 / Node[i].size();
		map<int, int>::const_iterator iter;
		for (iter = Node[i].begin(); iter != Node[i].end(); iter++) {
			Probability[iter->second] = prob;
		}
	}
}

void nodeData::calcSecondHand(const ImportData & idata, const int& random)
{
	//Allocating memory
	secondHand.resize(idata.nShips);
	for (int v = 0; v < idata.nShips; v++) {
		secondHand[v].resize(Nodes.size());
	}

	//initializing
	for (int v = 0; v < idata.nShips; v++) {
		for (int node = 0; node < Nodes.size(); node++) {
			secondHand[v][node] = 0;
		}
	}

	setRemainingLife(idata);	//Setting the remaining lifetime of each ship in the first period

	//Reading the data from Mrch and adjusting for different realisastions
	for (int t = 0; t < idata.Time; t++) {

		if (t >= branchPeriod && t < idata.Horizon) {

			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
				int v = iterShips->second;

				map<int, int>::const_iterator iterNodes;
				for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					int s = iterNodes->first;
					secondHand[v][node] = idata.secondHand[v][t] * (1+randomVariable[branchPeriod][s][randomVariable[branchPeriod][s].size()-random]);
				}

			}
		}

		else if (t == idata.Horizon) {
/*
			double totDetDemand = 0;
			//Calculating total deterministic demand
			for (int i = 0; i < idata.nTrades; i++) {
				for (int k = 0; k < idata.nProduct; k++) {
					totDetDemand += idata.Demand[i][k][1];
				}
			}
			*/
			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
				int v = iterShips->second;

				map<int, int>::const_iterator iterNodes;
				for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					int s = iterNodes->first;

					/*double totRealDemand = 0;
					//Calculating realised total demand
					for (int i = 0; i < idata.nTrades; i++) {
						for (int k = 0; k < idata.nProduct; k++) {
							totRealDemand += Demand[i][k][node];
						}
					}

					//Deviation from determinisitc
					double devDemand = (totRealDemand - totDetDemand) / totDetDemand;

					if (devDemand < 0) {

						RSV[v][node] = idata.RSV[v] * (1 + devDemand);  //randomVariable[branchPeriod][s][randomVariable[branchPeriod][s].size() - random]);
					}
					else {
						RSV[v][node] = idata.RSV[v];
					}
					*/

					RSV[v][node] = idata.RSV[v] * (1 + randomVariable[t][s][randomVariable[t][s].size() - random]);
				}
				
			}

		}
		else {
			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
				int v = iterShips->second;

				map<int, int>::const_iterator iterNodes;
				for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					int s = iterNodes->first;
					secondHand[v][node] = idata.secondHand[v][t];
				}
			}
		}
	}


	//Updating second hand prices given the new build cost

	for (int t = 0; t < idata.Time - buildTime; t++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VN[t + buildTime].begin(); iterShips != idata.VN[t + buildTime].end(); iterShips++) {
			int v = iterShips->second;

				map<int, int>::const_iterator iterNodes;
				for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					int s = iterNodes->first;
					double d = CNB[v][node] / 30;		//Annual depreciation
					int time = t;
					if (t < branchPeriod) {
						double price = idata.CNB[v][t];
						updateSecondHand(v, idata, price, d, time, s, random, false);		//Uses the first part of the updateSecondHand definition
					}
					else {
						double price = CNB[v][node];
						updateSecondHand(v, idata, price, d, time, s, random, false);		//Uses the last part of the updateSecondHand definition
					}
				}
			}
		}


	//Updating second hand prices given the second hand ships in period 1
	int t = 1;
		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
			int v = iterShips->second;


			map<int, int>::const_iterator iterNodes;
			for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				int s = iterNodes->first;
				double d = idata.secondHand[v][t]/remainingLife[v];		//Annual depreciation
				int time = t;
				if (t < branchPeriod) {
					double price = idata.secondHand[v][t];
					updateSecondHand(v, idata, price, d, time, s, random, false);		//Uses the first part of the updateSecondHand definition
				}
			}
		}
	}

void nodeData::updateSecondHand(const int& ship, const ImportData& idata, double& price, double& depreciation, int& time, const int& s, const int& random, bool beforeBranch)
{
	if (time < branchPeriod) {	//If the ship can be bought before branchPeriod, enter this scope
		int node = NodesTime[time][s];
		secondHand[ship][node] = price;

		//calculating next price
		price = (price - depreciation)*(1 / (1 + discountFactor));
		depreciation = depreciation*(1 / (1 + discountFactor));
		time++;
		
		beforeBranch = true;			//beforeBranch set to true to signal that the respective ship can be bought before the branch period

		updateSecondHand(ship, idata, price, depreciation, time, s, random, beforeBranch);
		
	}

	else if (beforeBranch && time < idata.Horizon) { //If the ship can be bought before the branch indtroduce uncertainty to the second hand value

		map<int, int>::const_iterator iterNodes;
		for (iterNodes = NodesTime[time].begin(); iterNodes != NodesTime[time].end(); iterNodes++) {
			int t = time + 1;
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			secondHand[ship][node] = price*(1 + randomVariable[time][scenario][randomVariable[time][scenario].size() - random]);
		
			double price2 = (secondHand[ship][node] - depreciation)*(1 / (1 + discountFactor));
			double depreciation2 = depreciation*(1 / (1 + discountFactor));
			bool flag = false;		//Makes sure the iteration stays on the same scenario
			updateSecondHand(ship, idata, price2, depreciation2, t, scenario, random, flag);
		}

		
	}

	else if (beforeBranch && time == idata.Horizon) {
/*
		double totDetDemand = 0;
		//Calculating total deterministic demand
		for (int i = 0; i < idata.nTrades; i++) {
			for (int k = 0; k < idata.nProduct; k++) {
				totDetDemand += idata.Demand[i][k][1];
			}
		}
		*/
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = NodesTime[time].begin(); iterNodes != NodesTime[time].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;

		/*	double totRealDemand = 0;
			//Calculating realised total demand
			for (int i = 0; i < idata.nTrades; i++) {
				for (int k = 0; k < idata.nProduct; k++) {
					totRealDemand += Demand[i][k][node];
				}
			}

			//Deviation from determinisitc
			double devDemand = (totRealDemand - totDetDemand) / totDetDemand;

			if (devDemand < 0) {

				RSV[ship][node] = price*(1 + devDemand);//randomVariable[time][scenario][randomVariable[time][scenario].size() - random]);
			}
			else {
				RSV[ship][node] = price;
			}
			*/
			RSV[ship][node] = price*(1 + randomVariable[time][scenario][randomVariable[time][scenario].size() - random]);
		}

	}

	else if (time >= branchPeriod && time < idata.Horizon) {
		int node = NodesTime[time][s];
		secondHand[ship][node] = price;

		price = (price - depreciation)*(1 / (1 + discountFactor));
		depreciation = depreciation*(1 / (1 + discountFactor));
		time++;

		updateSecondHand(ship, idata, price, depreciation, time, s, random, beforeBranch);

	}
	else if (time == idata.Horizon) {
		int node = NodesTime[time][s];

	/*	double totDetDemand = 0;
		//Calculating total deterministic demand
		for (int i = 0; i < idata.nTrades; i++) {
			for (int k = 0; k < idata.nProduct; k++) {
				totDetDemand += idata.Demand[i][k][time-buildTime];
			}
		}

		double totRealDemand = 0;
		//Calculating realised total demand
		for (int i = 0; i < idata.nTrades; i++) {
			for (int k = 0; k < idata.nProduct; k++) {
				totRealDemand += Demand[i][k][node];
			}
		}

		//Deviation from determinisitc
		double devDemand = (totRealDemand - totDetDemand) / totDetDemand;

		//Only allowing the downside of sunset value
		if (devDemand < 0) {
			RSV[ship][node] = price*(1 + devDemand);		//Adjusting the sunset value according to demand
		}
		else {
			RSV[ship][node] = price;
		}
		*/
		RSV[ship][node] = price;
	}

}

void nodeData::writeParameter(vector<vector<double>>& Parameter)
{
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			cout << "(" << i << ", " << j << ") " << Parameter[i][j] << endl;
		}
	}
}

void nodeData::writeDataParameter(vector<vector<vector<double>>>& Parameter) {
	double inf = numeric_limits<double>::infinity();
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			for (int k = 0; k < Parameter[i][j].size(); k++) {
				if (!(isDoubleEqual(Parameter[i][j][k], (numeric_limits<double>::max)(),roundingError))) {
					cout << "(" << i << ", " << j << ", " << k << ") - " << Parameter[i][j][k] << endl;
				}
			}
		}
	}
}

void nodeData::writeParameter(vector<double>& Parameter) {

	for (int i = 0; i < Parameter.size(); i++) {
		cout << i << " - " << Parameter[i] << endl;
	}
}

void nodeData::writeParameter(vector<int>& Parameter) {

	for (int i = 0; i < Parameter.size(); i++) {
		cout << i << " - " << Parameter[i] << endl;
	}
}

void nodeData::writeParameter(vector<vector<int>>& Parameter)
{
	for (int i = 0; i < Parameter.size(); i++) {
		for (int j = 0; j < Parameter[i].size(); j++) {
			cout << "(" << i << ", " << j << ") " << Parameter[i][j] << endl;
		}
	}
}

bool nodeData::isDoubleEqual(double x, double y, double RoundingError)
{
		return (abs(x - y) < RoundingError);
}

void nodeData::compareParameters(const vector<vector<double>>& Param1, const vector<vector<double>>& Param2)
{
	for (int i = 0; i < Param1.size(); i++) {
for (int j = 0; j < Param1[i].size(); j++) {
	cout << "(" << i << ", " << j << ")\t" << Param1[i][j] << "\t" << Param2[i][j] << endl;
}
	}
}

void nodeData::compareParameter(const vector<vector<int>>& Parameter, const map<string, int>& index1, const map<string, int>& index2)
{
	ofstream output("CompareDuration.txt");

	map<string, int>::const_iterator iterIndex1;
	for (iterIndex1 = index1.begin(); iterIndex1 != index1.end(); iterIndex1++) {
		string ship = iterIndex1->first;
		output << "\t" << ship;
	}
	output << endl;



	map<string, int>::const_iterator iterIndex2;
	for (iterIndex2 = index2.begin(); iterIndex2 != index2.end(); iterIndex2++) {
		int r = iterIndex2->second;
		string route = iterIndex2->first;
		output << "(" << route << ")" << "\t";

		map<string, int>::const_iterator iterIndex1;
		for (iterIndex1 = index1.begin(); iterIndex1 != index1.end(); iterIndex1++) {
			int v = iterIndex1->second;

			output << Parameter[v][r] << "\t";

		}
		output << endl;

	}

	output.close();
}

void nodeData::compareParameter(const vector<vector<vector<double>>>& Parameter, const int& time, const vector<map<string, int>>& index1, const map<string, int>& index2)
{
	ofstream output("CompareLoopCost.txt");

	for (int t = 1; t < time; t++) {

		output << "Period: " << t << endl;

		map<int, int>::const_iterator iterNodes;
		for (iterNodes = NodesTime[t].begin(); iterNodes != NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;

			map<string, int>::const_iterator iterIndex1;
			for (iterIndex1 = index1[t].begin(); iterIndex1 != index1[t].end(); iterIndex1++) {
				string ship = iterIndex1->first;
				output << "\t" << ship;
			}
			output << endl;



			map<string, int>::const_iterator iterIndex2;
			for (iterIndex2 = index2.begin(); iterIndex2 != index2.end(); iterIndex2++) {
				int r = iterIndex2->second;
				string route = iterIndex2->first;
				output << "(" << route << ")" << "\t";

				map<string, int>::const_iterator iterIndex1;
				for (iterIndex1 = index1[t].begin(); iterIndex1 != index1[t].end(); iterIndex1++) {
					int v = iterIndex1->second;

					output << Parameter[v][r][node] << "\t";

				}
				output << endl;

			}
		}
	}

	output.close();
}





void nodeData::calcAvailableTime(vector<double>& Par, const int& index1)
{
	Par.resize(index1);

	for (int v = 0; v < index1; v++) {
		Par[v] = 350.0;							//Assumed that the available time for each ship is 350 days each year
	}
}

void nodeData::setRemainingLife(const ImportData & idata)
{
	remainingLife.resize(idata.nShips);

	map<string, int>::const_iterator iterShips;
	for (iterShips = idata.Ships.begin(); iterShips != idata.Ships.end(); iterShips++) {
		string ship = iterShips->first;
		int v = iterShips->second;

		if (ship == "PCC87") {
			remainingLife[v] = 3;
		}
		else if (ship == "PCC00") {
			remainingLife[v] = 17;
		}
		else if (ship == "PCTC2002") {
			remainingLife[v] = 20;
		}
		else if (ship == "LCTC07") {
			remainingLife[v] = 25;
		}
		else if (ship == "PCTC2007") {
			remainingLife[v] = 25;
		}
		else if (ship == "PCTC97") {
			remainingLife[v] = 15;
		}
		else if (ship == "RORO83") {
			remainingLife[v] = 1;
		}
		else if (ship == "RORO2012") {
			remainingLife[v] = 30;
		}
		else if (ship == "LCTC11") {
			remainingLife[v] = 29;
		}
		else if (ship == "LCTC12") {
			remainingLife[v] = 30;
		}
		else {
			remainingLife[v] = 30;
		}
	}
}
