#pragma once
#include "importData.h"
#include "createData.h"
#include "Problem.h"

#include<cstring>
#include<Windows.h>
#include<WinBase.h>
#include<process.h>

#include<string>
#include<limits>

#include<math.h>




class nodeData {
public:
	
	nodeData() : interest(0.03) { defaultRegister(); };
	nodeData(const int& numberOfStages, const int& numberOfScenarios, const ImportData& idata,const int& Instalments,const int& buildTime, const string & filePath, 
		const int& branchPeriod, const double& discountFactor, const double& interest): M(Instalments), buildTime(buildTime), roundingError(0.002), branchPeriod(branchPeriod), 
		discountFactor(discountFactor), interest(interest) { registerNodes(numberOfStages, numberOfScenarios, idata, filePath, branchPeriod); }

	nodeData(const int& numberOfStages, const int& numberOfScenarios, const ImportData& idata, const int& Instalments, const int& buildTime, const string & filePath,
		const vector<int>& branchPeriods, const double& discountFactor, const double& interest) : M(Instalments), buildTime(buildTime), roundingError(0.002), branchPeriods(branchPeriods),
		discountFactor(discountFactor), interest(interest), branchPeriod(branchPeriods[0]) { registerNodes(numberOfStages, numberOfScenarios, idata, filePath); }
	

	//Sets
	vector<int> Nodes;						//Set of all nodes E
	vector<map<int, int>> NodesTime;		//Set of all nodes in time period t, Et
	vector<map<int, int>> AncestorNodes;	//The set of ancestor nodes in time t preceeding node n, map<node,ancestor node>

	vector<int> InstNodes;					//Set of all nodes, including future nodes
	vector<map<int, int>> InstalmentNodes;	//The set of all nodes in time period t, Et. max(t) = (t+M)
	vector<map<int, int>> ReverseInstalmentNodes;	//which node points to which scenario in time t
	vector<vector<map<int, int>>> AncestorInstalmentNodes;	////The set of ancestor nodes in time tmarked preceeding node n in time t, map<node,ancestor node>

	vector<vector<double>> randomTest;	//Random variable (node,type)
	vector<vector<vector<double>>> randomVariable; //Random variable each time period, (time, scenario, variable type)


	vector<double> availableTime;	//Z_v
	
	//From ImportData
	
	//Parameters
	vector<vector<vector<double>>> CspaceC; //Unit space charter cost, i,k,n
	vector<vector<double>> CNB;				//New build cost, v,n
	vector<vector<double>> secondHand;		//Second hand price, v,n
	vector<vector<double>> CCI;				//Charter in cost, v,n
	vector<vector<double>> RLU;				//R_{vts}^[LU], Lay-up savings, v,n
	vector<vector<double>> RSC;				//Scrapping value,v,n
	vector<vector<double>> RSV;						//Sunset value, v, n
	vector<vector<vector<double>>> Demand;			//Demand, i,k,n
	vector<vector<double>> MinFreq;				//Minumum frequency, i,n
	vector<vector<int>> InitialNB;					//Y_vt^NB, New ships of type v ordered in the previous planning period delivered at node n 
	vector<vector<double>> UnitRevenue;		//R_{it}^D, The revenue of transporting 1 unit of goods on trade i, at node n 
	vector<vector<vector<double>>> LoopCost;	//C_{vrt}^TR, the cost for ship v to perform loop r at node n
	vector<vector<double>> OperatingCost;		//C_{vt}^OP, The operating cost of ship v at node n
	vector<double> Prob;						//P_n, probability of node n occuring

	vector<vector<vector<double>>> Instalment;
	vector<map<int, int>> FutureNodes;

	

	void calcInstalment(const ImportData& idata);

	
	template<class T>
	void readParameter(string& fileID, vector<vector<T>>& Parameter);

	template<class T>
	void appendParameter(string& fileID, vector<vector<T>>& Parameter);

	template<class T>
	void writeParameter(vector<vector<T>>& Parameter, const string& fileID);
	template<class T>
	void writeParameter(vector<vector<vector<T>>>& Parameter, const string& fileID);

	void writeSet(vector<map<int, int>> Set);
	void writeSet(vector<vector<map<int, int>>> Set);
	
	void registerNodes(const int & numberOfStages, const int& numberOfScenarios,const ImportData& idata, const string & filePath, const int& branchPeriod);
	void registerNodes(const int & numberOfStages, const int& numberOfScenarios, const ImportData& idata, const string & filePath);
	void defaultRegister();

	void convertDataToNode(const ImportData& idata);

	template<class T>
	void nodeConvert(vector<vector<vector<T>>>& Parameter,const vector<vector<vector<T>>> & Data,const int & index1,const int & index2,const int& index3,const int& random);

	template<class T>
	void nodeConvert(vector<vector<T>>& Parameter, const vector<vector<T>> & Data, const int & index1, const int & index2, const int & random);

	template<class T>
	void nodeConvert(vector<vector<T>>& Parameter, const vector<T> & Data, const int& index1, const int& horizon, const int & random);

	template<class T>
	void nodeConvert(T& Parameter, const T& forecastParameter, const int & index1, const int & index2, const int& index3);

	template<class T>
	void nodeConvertDiscount(T& Parameter, const T& forecastParameter, const int & index1, const int & index2, const int& index3);

	template<class T>
	void convertParameter(vector<vector<vector<T>>>& Parameter, const vector<vector<vector<T>>> & Data, const int & index1, const int & index2, const int& index3, const int& random);

	template<class T>
	void convertParameterIncrement(vector<vector<vector<T>>>& Parameter, const vector<vector<vector<T>>> & Data, const int & index1, const int & index2, const int& index3, const int& random, const int& variableChoice, const bool& timeDependent, const bool& discount);


	template<class T>
	void convertParameter(vector<vector<T>>& Parameter, const vector<vector<T>> & Data, const int & index1, const int & index2, const int& random, const int& variableChoice);

	template<class T>
	void convertParameterIncrement(vector<vector<T>>& Parameter, const vector<vector<T>> & Data, const int & index1, const int & index2, const int& random, const int& variableChoice, const bool& timeDependent);
									//(Parameter, forecast, first index, second index-usually time, deterministic/stochastic (0/1), selecting random variable (0-looping through trade dependent random variable/ 1-deterministic/ any other number-selecting the given random variable), true if the parameter is dependent to the previous period/false if not)

	template<class T>
	void convertParameterIncrement(vector<vector<T>>& Parameter, const vector<T> & Data, const int & index1, const int & index2, const int& variableChoice);
	//(Parameter, forecast, first index, second index-usually time, deterministic/stochastic (0/1), selecting random variable (0-looping through trade dependent random variable/ 1-deterministic/ any other number-selecting the given random variable), true if the parameter is dependent to the previous period/false if not)

	void calcProb(vector<double>& Probability, const vector<map<int,int>>& Node, const int& time);

	void calcSecondHand(const ImportData& idata, const int& random);	//"random" selects which random variable to use, 4 = market condition
	void updateSecondHand(const int& ship, const ImportData& idata, double& price, double& depreciation, int& time, const int& s, const int& random, bool beforeBranch);		//beforeBranch set to false first time called

	void writeParameter(vector<vector<double>>& Parameter);
	void writeDataParameter(vector<vector<vector<double>>>& Parameter);
	void writeParameter(vector<double>& Parameter);
	void writeParameter(vector<int>& Parameter);
	void writeParameter(vector<vector<int>>& Parameter);



	double M;		//Number of instalments
	int buildTime;	//Buildtime
	int branchPeriod;	//The first stage/ the first period where the nodes branch
	vector<int> branchPeriods;	// A vector containing the branchPeriods of a multi-stage problem
	double discountFactor;	//The factor used to discount all the input parameters
	vector<int> remainingLife;	//Remaining economic lifetime of the second hand ships in the first period


private:
	void compareParameters(const vector<vector<double>> & Param1, const vector<vector<double>>& Param2);
	void compareParameter(const vector<vector<int>>& Parameter, const map<string,int>& index1, const map<string,int>& index2);
	void compareParameter(const vector<vector<vector<double>>>& Parameter, const int& time, const vector<map<string, int>>& index1, const map<string, int>& index2);
	void calcAvailableTime(vector<double>& Par,const int& index1);
	void setRemainingLife(const ImportData & idata);
	bool isDoubleEqual(double x, double y, double RoundingError);
	double roundingError;
	const double& interest;

};