#include "nodeModel.h"

nodeModel::nodeModel()
{

}



void nodeModel::model(const nodeData& nodeData, const ImportData& idata)
{
	
	int nodes = nodeData.Nodes.size();
	int days = 350;		//Used to multiplie with charter in cost, CCI is given as rate pr. day.
	double spaceFactor = 1;	//A factor to adjust the space charter cost to see the possible effects
	double revenueFactor = 1.1;	//A factor to adjust the unit revenue
	double layUpFactor = 0.5;		//A factor to adjust the lay Up savings
	double operatingFactor = 4;	//A factor to adjust the operating cost
	double sunsetFactor = 0.7;	//A factor to adjust the sunset value
	double investmentFactor = 0.05; //Percentage of the contracted revenue in period 0 that is available for investments

	//Allocating memory results parameters
	revenueOResults.resize(idata.nTrades);
	revenueCResults.resize(idata.nTrades);
	spaceCharterCostResults.resize(idata.nTrades);
	for (int i = 0; i < idata.nTrades; i++) {
		revenueOResults[i].resize(idata.nProduct);
		revenueCResults[i].resize(idata.nProduct);
		spaceCharterCostResults[i].resize(idata.nProduct);
		for (int k = 0; k < idata.nProduct; k++) {
			revenueOResults[i][k].resize(nodes);
			revenueCResults[i][k].resize(nodes);
			spaceCharterCostResults[i][k].resize(nodes);
		}
	}

	operatingCostResults.resize(idata.nShips);
	charterCostResults.resize(idata.nShips);
	charterOutResults.resize(idata.nShips);
	loopCostResults.resize(idata.nShips);
	layUpSavingsResults.resize(idata.nShips);
	for (int v = 0; v < idata.nShips; v++) {
		operatingCostResults[v].resize(nodes);
		charterCostResults[v].resize(nodes);
		charterOutResults[v].resize(nodes);
		layUpSavingsResults[v].resize(nodes);

		loopCostResults[v].resize(idata.nRoutes);
		for (int r = 0; r < idata.nRoutes; r++) {
			loopCostResults[v][r].resize(nodes);
		}
	}

	instalmentCostResults.resize(idata.nShips);
	for (int v = 0; v < idata.nShips; v++) {
		instalmentCostResults[v].resize(nodeData.Nodes.size());
		for (int a = 0; a < nodes; a++) {
			instalmentCostResults[v][a].resize(nodeData.InstNodes.size());
		}
	}

	sunsetValueResults.resize(idata.nShips);
	for (int v = 0; v < idata.nShips; v++) {
			sunsetValueResults[v].resize(nodes);
		}

	scrappingValueResults.resize(idata.nShips);
	buyingResults.resize(idata.nShips);
	sellingResults.resize(idata.nShips);
	for (int v = 0; v < idata.nShips; v++){
			scrappingValueResults[v].resize(nodeData.Nodes.size());
			buyingResults[v].resize(nodes);
			sellingResults[v].resize(nodes);
		}

	ySEResults.resize(idata.nShips);
	ySHResults.resize(idata.nShips);
	yNBResults.resize(idata.nShips);
	ySCResults.resize(idata.nShips);
	yPResults.resize(idata.nShips);
	layUpResults.resize(idata.nShips);
	hIResults.resize(idata.nShips);
	hOResults.resize(idata.nShips);
	xResults.resize(idata.nShips);
	nResults.resize(idata.nTrades);
	dResults.resize(idata.nTrades);

	for (int i = 0; i < idata.nTrades; i++) {
		nResults[i].resize(idata.nProduct);
		dResults[i].resize(nodeData.Nodes.size());
		for (int k = 0; k < idata.nProduct; k++) {
			nResults[i][k].resize(nodeData.Nodes.size());
		}
	}

	for (int v = 0; v < idata.nShips; v++) {
		ySEResults[v].resize(nodes);
		ySHResults[v].resize(nodes);
		yNBResults[v].resize(nodeData.Nodes.size());
		ySCResults[v].resize(nodeData.Nodes.size());
		yPResults[v].resize(nodeData.Nodes.size());
		layUpResults[v].resize(nodeData.Nodes.size());
		hIResults[v].resize(nodeData.Nodes.size());
		hOResults[v].resize(nodes);
		xResults[v].resize(idata.nRoutes);
		
		for (int r = 0; r < idata.nRoutes; r++) {
			xResults[v][r].resize(nodeData.Nodes.size());
		}
	}


	//Initialize
	for (int i = 0; i < idata.nTrades; i++) {
		for (int k = 0; k < idata.nProduct; k++) {
			for (int node = 0; node < nodeData.Nodes.size(); node++) {
				revenueOResults[i][k][node] = 0;
				revenueCResults[i][k][node] = 0;
				spaceCharterCostResults[i][k][node] = 0;
			}
		}
	}

	for (int v = 0; v < idata.nShips; v++) {
		for (int node = 0; node < nodeData.Nodes.size(); node++) {
			operatingCostResults[v][node] = 0;
			charterCostResults[v][node] = 0;
			charterOutResults[v][node] = 0;
			layUpSavingsResults[v][node] = 0;

			loopCostResults[v].resize(idata.nRoutes);
			for (int r = 0; r < idata.nRoutes; r++) {
				loopCostResults[v][r][node] = 0;
			}
		}
	}

	
	for (int v = 0; v < idata.nShips; v++) {
		for (int a = 0; a < nodes; a++) {
			for (int node = 0; node < nodeData.InstNodes.size(); node++) {
				instalmentCostResults[v][a][node] = 0;
			}
		}
	}

	
	for (int v = 0; v < idata.nShips; v++){
		for (int node = 0; node < nodeData.Nodes.size(); node++) {
			sunsetValueResults[v][node] = 0;
		}
	}

	
	for (int v = 0; v < idata.nShips; v++){
			for (int node = 0; node < nodeData.Nodes.size(); node++) {
				buyingResults[v][node] = 0;
				sellingResults[v][node] = 0;
				scrappingValueResults[v][node] = 0;
			}
		}

	//Result Variables
	for (int i = 0; i < idata.nTrades; i++) {
		for (int node = 0; node < nodeData.Nodes.size(); node++) {
			dResults[i][node] = 0;
			for (int k = 0; k < idata.nProduct; k++) {
				nResults[i][k][node] = 0;
			}
		}
	}

	for (int v = 0; v < idata.nShips; v++) {
		for (int node = 0; node < nodeData.Nodes.size(); node++) {
			ySEResults[v][node] = 0;
			ySHResults[v][node] = 0;
			yNBResults[v][node] = 0;
			ySCResults[v][node] = 0;
			yPResults[v][node] = 0;
			layUpResults[v][node] = 0;
			hIResults[v][node] = 0;
			hOResults[v][node] = 0;

			for (int r = 0; r < idata.nRoutes; r++) {
				xResults[v][r][node] = 0;
			}
		}
	}


	IloEnv env;

	try {
		feasible = true;
		IloModel basicModel(env);

	

		//Declaring variables
		vector<vector<IloNumVar>> yNB(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//The number of new buildings ordered of ship of type v, at node n
		vector<vector<IloNumVar>> ySC(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//The number of ships of type v scrapped at node n
		vector<vector<IloNumVar>> yP(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//The number of ships of type v in the pool, at node n
		
		vector<vector<IloNumVar>> layUp(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));	//The number of ships of type v on lay - up, at node n

		vector<vector<vector<IloNumVar>>> x(idata.nShips, vector<vector<IloNumVar>>(idata.nRoutes, vector<IloNumVar>(nodeData.Nodes.size())));		//The number of loops r performed of ship of type v, at node n
		vector<vector<vector<IloNumVar>>> n(idata.nTrades, vector<vector<IloNumVar>>(idata.nProduct, vector<IloNumVar>(nodeData.Nodes.size())));	//The amount of product k delivered at node n, by space charters on trade i 

		vector<vector<IloNumVar>> d(idata.nTrades, vector<IloNumVar>(nodeData.Nodes.size()));		//Set to 1 if the company services optional trade i at node n, 0 otherwise.

		//addOption = 1
		vector<vector<IloNumVar>> ySE(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//Number of ships sold in the second hand market, y_{vn}^{SE}
		vector<vector<IloNumVar>> ySH(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//Number of ships bought in the second hand market, y_{vn}^{SH}
		vector<vector<IloNumVar>> hI(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//The number of ships of type v chartered in, at node n
		vector<vector<IloNumVar>> hO(idata.nShips, vector<IloNumVar>(nodeData.Nodes.size()));		//The number of ships of type v chartered out, at node n



		//Factors
		int buildTime = 2;
		IloNum var = 0;

		//Initializing variables
	

		for (int t = 0; t < idata.Time - buildTime; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VN[t + buildTime].begin(); iter != idata.VN[t + buildTime].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					int s = iter2->first;
					stringstream strout;
					strout << "yNB" << "(" << v << "," << t << "," << s << ")";
					const string name = strout.str();
					const char *cstr = name.c_str();
					yNB[v][node] = IloNumVar(env, 0, 100, ILOINT, cstr);	//int
				}
			}
		}

		for (int t = 0; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					int s = iter2->first;
					stringstream strout;
					strout << "yP" << "(" << v << "," << t << "," << s << ")";
					const string name = strout.str();
					const char *cstr = name.c_str();
					yP[v][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr);	//float
				}
			}
		}

		for (int t = 0; t < idata.Horizon; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					int s = iter2->first;
					stringstream strout;
					strout << "ySC" << "(" << v << "," << t << "," << s << ")";
					const string name = strout.str();
					const char *cstr = name.c_str();
					ySC[v][node] = IloNumVar(env, 0, IloInfinity, ILOINT, cstr);	//int
				}

			}
		}

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					int s = iter2->first;
					stringstream strout2;
					strout2 << "layUp" << "(" << v << "," << t << "," << s << ")";
					const string name2 = strout2.str();
					const char *cstr2 = name2.c_str();
					layUp[v][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr2);	//float
				}

			}
		}

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<string, int>::const_iterator iterRoutes;
				for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
					int r = iterRoutes->second;
					map<int, int>::const_iterator iterNodes;
					for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
						int node = iterNodes->second;
						int s = iterNodes->first;
						stringstream strout;
						strout << "x" << "(" << v << "," << iterRoutes->first << "," << t << "," << s << ")";
						const string name = strout.str();
						const char *cstr = name.c_str();
						x[v][r][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr);	//Float
					}
				}
			}
		}

		for (int t = 1; t < idata.Time; t++) {
			for (int k = 0; k < idata.nProduct; k++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.TradesCOf[t].begin(); iter != idata.TradesCOf[t].end(); iter++) {
					int i = iter->second;
					map<int, int>::const_iterator iterNodes;
					for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
						int node = iterNodes->second;
						int s = iterNodes->first;
						stringstream strout;
						strout << "n" << "(" << i << "," << k << "," << t << "," << s << ")";
						const string name = strout.str();
						const char *cstr = name.c_str();
						n[i][k][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr);
					}
				}
			}
		}

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.TradesOOf[t].begin(); iter != idata.TradesOOf[t].end(); iter++) {
				int i = iter->second;
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					int s = iterNodes->first;
					stringstream strout;
					strout << "d" << "(" << i << "," << t << "," << s << ")";
					const string name = strout.str();
					const char *cstr = name.c_str();
					d[i][node] = IloNumVar(env, 0, 1, ILOBOOL, cstr);	//binary
				}
			}
		}


		//Used for CVaR-constraints
		IloNumVar zeta(env, -IloInfinity, IloInfinity, "zeta");
		vector<IloNumVar> flow(nodeData.Nodes.size());

		for (int node = 0; node < nodeData.Nodes.size(); node++) {
			stringstream strout;
			strout << "flow" << "(" << node << ")";
			const string name = strout.str();
			const char *cstr = name.c_str();
			flow[node] = IloNumVar(env, 0, IloInfinity, cstr);
		}



		//Creating objective function


		//Declaring expressions
		vector<vector<IloNumExpr>> charterCostMatrix;				//Charter in cost, v, n
		vector<vector<IloNumExpr>> charterOutMatrix;
		vector<vector<IloNumExpr>> buyingMatrix;
		vector<vector<IloNumExpr>> sellingMatrix;
		

		
		vector<vector<vector<IloNumExpr>>> revenueOMatrix;					//i,k,n
		vector<vector<vector<double>>> revenueCMatrix;					//i,n
		vector<vector<vector<IloNumExpr>>> spaceCharterCostMatrix;	//i,k,n
		vector<vector<vector<IloNumExpr>>> loopCostMatrix;					//v,r,n
		vector<vector<IloNumExpr>> operatingCostMatrix;				//v,n
		vector<vector<IloNumExpr>> layUpSavingsMatrix;				//v,n
		vector<vector<IloNumExpr>> scrappingValueMatrix;			//v,n
		vector<vector<vector<IloNumExpr>>> instalmentCostMatrix;	//v,a,n
		vector<vector<IloNumExpr>> sunsetValueMatrix;				//v,n


		vector<IloNumExpr> charterCostObjective;
		vector<IloNumExpr> charterOutObjective;
		vector<IloNumExpr> buyingObjective;
		vector<IloNumExpr> sellingObjective;

		vector<IloNumExpr> nodeObjective;	//Storing the objective function for every node
		vector<IloNumExpr> revenueOObjective;
		vector<double> revenueCObjective;
		vector<IloNumExpr> spaceCharterCostObjective;
		vector<IloNumExpr> operatingCostObjective;
		vector<IloNumExpr> loopCostObjective;
		vector<IloNumExpr> layUpSavingsObjective;
		vector<IloNumExpr> instalmentObjective;
		vector<IloNumExpr> sunsetObjective;
		vector<IloNumExpr> scrappingObjective;
		IloNumExpr objfunc(env, var);			//Objective function

	//Allocating memory
		revenueOMatrix.resize(idata.nTrades);
		revenueCMatrix.resize(idata.nTrades);
		spaceCharterCostMatrix.resize(idata.nTrades);
		for (int i = 0; i < idata.nTrades; i++) {
			revenueOMatrix[i].resize(idata.nProduct);
			revenueCMatrix[i].resize(idata.nProduct);
			spaceCharterCostMatrix[i].resize(idata.nProduct);
			for (int k = 0; k < idata.nProduct; k++) {
				revenueOMatrix[i][k].resize(nodes);
				revenueCMatrix[i][k].resize(nodes);
				spaceCharterCostMatrix[i][k].resize(nodes);
			}
		}

		operatingCostMatrix.resize(idata.nShips);
		
		loopCostMatrix.resize(idata.nShips);
		layUpSavingsMatrix.resize(idata.nShips);
		for (int v = 0; v < idata.nShips; v++) {
			operatingCostMatrix[v].resize(nodes);
			
			layUpSavingsMatrix[v].resize(nodes);

			loopCostMatrix[v].resize(idata.nRoutes);
			for (int r = 0; r < idata.nRoutes; r++) {
				loopCostMatrix[v][r].resize(nodes);
			}
		}

		instalmentCostMatrix.resize(idata.nShips);
		for (int v = 0; v < idata.nShips; v++) {
			instalmentCostMatrix[v].resize(nodes);
			for (int a = 0; a < nodes; a++) {
				instalmentCostMatrix[v][a].resize(nodeData.InstNodes.size());
			}
		}

		sunsetValueMatrix.resize(idata.nShips);
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[idata.Horizon].begin(); iter != idata.VoF[idata.Horizon].end(); iter++) {
			int v = iter->second;
			sunsetValueMatrix[v].resize(nodes);
		}

		scrappingValueMatrix.resize(idata.nShips);
		for (int t = 0; t < idata.Horizon; t++) {		//Not possible to scrap in last period
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				scrappingValueMatrix[v].resize(nodeData.Nodes.size());
			}
		}



		revenueOObjective.resize(nodes);
		revenueCObjective.resize(nodes);
		spaceCharterCostObjective.resize(nodes);
		operatingCostObjective.resize(nodes);
		loopCostObjective.resize(nodes);
		layUpSavingsObjective.resize(nodes);
		instalmentObjective.resize(nodes + nodeData.NodesTime[idata.Horizon].size()*nodeData.M);
		sunsetObjective.resize(nodes);
		scrappingObjective.resize(nodes);

		CashFlowIn.resize(nodes);
		CashFlowOut.resize(nodes);

		//Initialising expressions
		for (int t = 0; t < idata.Time; t++) {
			map<int, int>::const_iterator iterNodes;

			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				IloNumExpr initCashIn(env, var);
				IloNumExpr initCashOut(env, var);

				CashFlowIn[node] = initCashIn;
				CashFlowOut[node] = initCashOut;
			}
		}


		for (int t = 1; t < idata.Time; t++) {
			map<int, int>::const_iterator iterNodes;

			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				IloNumExpr initObjective(env, var);
				IloNumExpr initObjective2(env, var);
				IloNumExpr initObjective3(env, var);
				IloNumExpr initObjective4(env, var);
				IloNumExpr initObjective5(env, var);
				IloNumExpr initObjective6(env, var);
				IloNumExpr initObjective7(env, var);
				revenueOObjective[node] = initObjective;
				revenueCObjective[node] = 0;
				spaceCharterCostObjective[node] = initObjective2;
				operatingCostObjective[node] = initObjective3;
				
				loopCostObjective[node] = initObjective5;
				layUpSavingsObjective[node] = initObjective6;
				for (int k = 0; k < idata.nProduct; k++) {
					map<string, int>::const_iterator iterTrades;
					for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
						int i = iterTrades->second;
						IloNumExpr init(env, var);
						revenueOMatrix[i][k][node] = init;
					}
					for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
						int i = iterTrades->second;
						IloNumExpr init2(env, var);
						revenueCMatrix[i][k][node] = 0;
						spaceCharterCostMatrix[i][k][node] = init2;
					}
				}
				map<string, int>::const_iterator iterShips;
				for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
					int v = iterShips->second;
					IloNumExpr init3(env, var);
					operatingCostMatrix[v][node] = init3;
					IloNumExpr init4(env, var);
					
					IloNumExpr init5(env, var);
					layUpSavingsMatrix[v][node] = init5;

					map<string, int>::const_iterator iterRoutes;
					for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
						IloNumExpr init6(env, var);
						int r = iterRoutes->second;
						loopCostMatrix[v][r][node] = init6;
					}

				}
			}
		}


		for (int tmarked = 0; tmarked < idata.Time - nodeData.buildTime; tmarked++) {

			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VN[tmarked + nodeData.buildTime].begin(); iterShips != idata.VN[tmarked + nodeData.buildTime].end(); iterShips++) {
				int v = iterShips->second;

				map<int, int>::const_iterator iterAncestor;
				for (iterAncestor = nodeData.InstalmentNodes[tmarked].begin(); iterAncestor != nodeData.InstalmentNodes[tmarked].end(); iterAncestor++) {
					int a = iterAncestor->second;

					for (int t = tmarked; t < tmarked + nodeData.M; t++) {
						map<int, int>::const_iterator iterNodes;
						for (iterNodes = nodeData.InstalmentNodes[t].begin(); iterNodes != nodeData.InstalmentNodes[t].end(); iterNodes++) {
							int node = iterNodes->second;

							IloNumExpr init7(env, var);
							instalmentCostMatrix[v][a][node] = init7;
						}
					}
				}
			}
		}


		for (int t = 0; t < idata.Horizon - nodeData.buildTime + nodeData.M; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.InstalmentNodes[t].begin(); iterNodes != nodeData.InstalmentNodes[t].end(); iterNodes++) {
				int node = iterNodes->second;
				IloNumExpr init9(env, var);
				instalmentObjective[node] = init9;
			}
		}

		for (iter = idata.VoF[idata.Horizon].begin(); iter != idata.VoF[idata.Horizon].end(); iter++) {
			int v = iter->second;
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[idata.Horizon].begin(); iterNodes != nodeData.NodesTime[idata.Horizon].end(); iterNodes++) {
				int node = iterNodes->second;
				IloNumExpr init10(env, var);
				sunsetValueMatrix[v][node] = init10;		//Initializing

			}
		}
		map<int, int>::const_iterator iterNodesSunset;
		for (iterNodesSunset = nodeData.NodesTime[idata.Horizon].begin(); iterNodesSunset != nodeData.NodesTime[idata.Horizon].end(); iterNodesSunset++) {
			int node = iterNodesSunset->second;
			IloNumExpr init11(env, var);
			sunsetObjective[node] = init11;
		}

		for (int t = 0; t < idata.Horizon; t++) {		//Not possible to scrap in last period
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr init12(env, var);
					scrappingValueMatrix[v][node] = init12;

				}
			}
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				IloNumExpr init13(env, var);
				scrappingObjective[node] = init13;
			}
		}


		//Objective function

		for (int t = 1; t < idata.Time; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				for (int k = 0; k < idata.nProduct; k++) {
					map<string, int>::const_iterator iterTrades;
					for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
						int i = iterTrades->second;
						revenueOMatrix[i][k][node] = revenueFactor * nodeData.UnitRevenue[i][node] * nodeData.Demand[i][k][node] * d[i][node];
						revenueOObjective[node] += revenueOMatrix[i][k][node];
					}
					for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
						int i = iterTrades->second;
						revenueCMatrix[i][k][node] = revenueFactor * nodeData.UnitRevenue[i][node] * nodeData.Demand[i][k][node];
						revenueCObjective[node] += revenueCMatrix[i][k][node];
						spaceCharterCostMatrix[i][k][node] = spaceFactor*nodeData.CspaceC[i][k][node] * n[i][k][node];
						spaceCharterCostObjective[node] += spaceCharterCostMatrix[i][k][node];
					}
				}
				map<string, int>::const_iterator iterShips;
				for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
					int v = iterShips->second;
					operatingCostMatrix[v][node] = operatingFactor * nodeData.OperatingCost[v][node] * yP[v][node];
					operatingCostObjective[node] += operatingCostMatrix[v][node];


					layUpSavingsMatrix[v][node] = layUpFactor * nodeData.RLU[v][node] * layUp[v][node];
					layUpSavingsObjective[node] += layUpSavingsMatrix[v][node];

					map<string, int>::const_iterator iterRoutes;
					for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
						int r = iterRoutes->second;
						loopCostMatrix[v][r][node] = nodeData.LoopCost[v][r][node] * x[v][r][node];
						loopCostObjective[node] += loopCostMatrix[v][r][node];
					}
				}
			}
		}

		//Instalment-part of objective function

		vector<vector<bool>> flag;			//Used to check if instalmentMatrix[v][a][a] has been added already. True if it hasn't been added yet, and false if it has
		flag.resize(idata.nShips);
		for (int v = 0; v < idata.nShips; v++) {
			flag[v].resize(nodeData.Nodes.size());
			for (int i = 0; i < nodeData.Nodes.size(); i++) {
				flag[v][i] = true;
			}
		}

		for (int tmarked = 0; tmarked < idata.Time - nodeData.buildTime; tmarked++) {

			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VN[tmarked + nodeData.buildTime].begin(); iterShips != idata.VN[tmarked + nodeData.buildTime].end(); iterShips++) {
				int v = iterShips->second;

				
				

				map<int, int>::const_iterator iterNodes;
				for (int t = tmarked + 1; t < (tmarked + nodeData.M); t++) {
					for (iterNodes = nodeData.AncestorInstalmentNodes[tmarked][t].begin(); iterNodes != nodeData.AncestorInstalmentNodes[tmarked][t].end(); iterNodes++) {
						int a = iterNodes->second;
						
						int node = iterNodes->first;
						instalmentCostMatrix[v][a][node] = nodeData.Instalment[v][a][node] * yNB[v][a];
						instalmentObjective[node] += instalmentCostMatrix[v][a][node];

						if (flag[v][a]) {
							instalmentCostMatrix[v][a][a] = nodeData.Instalment[v][a][a] * yNB[v][a];
							instalmentObjective[a] += instalmentCostMatrix[v][a][a];
							flag[v][a] = false;
						}
					}
				}
			}
		}


		//Sunset value
		map<int, int>::const_iterator iterN;
		for (iterN = nodeData.NodesTime[idata.Horizon].begin(); iterN != nodeData.NodesTime[idata.Horizon].end(); iterN++) {
				int node = iterN->second;
				map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VoF[idata.Horizon].begin(); iterShips != idata.VoF[idata.Horizon].end(); iterShips++) {
			int v = iterShips->second;

				sunsetValueMatrix[v][node] = sunsetFactor * nodeData.RSV[v][node] * yP[v][node];
				sunsetObjective[node] += sunsetValueMatrix[v][node];
			}
		}

		//Scrapping

		for (int t = 0; t < idata.Horizon; t++) {		//Not possible to scrap in last period
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					scrappingValueMatrix[v][node] = nodeData.RSC[v][node] * ySC[v][node];
					scrappingObjective[node] += scrappingValueMatrix[v][node];
				}
			}
		}

		for (int j = 1; j < nodes; j++) {
			objfunc += nodeData.Prob[j] * (revenueCObjective[j] + revenueOObjective[j]);
		}

		for (int j = 1; j < nodes; j++) {
			objfunc += nodeData.Prob[j] * layUpSavingsObjective[j];
		}

		for (int j = 1; j < nodes; j++) {
			objfunc -= nodeData.Prob[j] * (spaceCharterCostObjective[j]  + operatingCostObjective[j] + loopCostObjective[j]);	
		}


		for (int t = 0; t < idata.Horizon - nodeData.buildTime + nodeData.M; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.InstalmentNodes[t].begin(); iterNodes != nodeData.InstalmentNodes[t].end(); iterNodes++) {
				int node = iterNodes->second;
				objfunc -= nodeData.Prob[node] * instalmentObjective[node];
			}
		}


		map<int, int>::const_iterator iter2;
		for (iter2 = nodeData.NodesTime[idata.Horizon].begin(); iter2 != nodeData.NodesTime[idata.Horizon].end(); iter2++) {
			int node = iter2->second;
			objfunc += nodeData.Prob[node] * sunsetObjective[node];
		}

		for (int t = 0; t < idata.Horizon; t++) {
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				objfunc += nodeData.Prob[node] * scrappingObjective[node];
			}
		}

		switch (addOption) {
		case 1: {
		
			//Initialising variables
			for (int t = 0; t < idata.Horizon; t++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					map<int, int>::const_iterator iter2;
					for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
						int node = iter2->second;
						int s = iter2->first;
						stringstream strout;
						strout << "ySE" << "(" << v << "," << t << "," << s << ")";
						const string name = strout.str();
						const char *cstr = name.c_str();
						ySE[v][node] = IloNumVar(env, 0, IloInfinity, ILOINT, cstr);	//int

						stringstream strout2;
						strout2 << "ySH" << "(" << v << "," << t << "," << s << ")";
						const string name2 = strout2.str();
						const char *cstr2 = name2.c_str();
						ySH[v][node] = IloNumVar(env, 0, IloInfinity, ILOINT, cstr2);	//int
					}

				}
			}




			for (int t = 1; t < idata.Time; t++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					map<int, int>::const_iterator iter2;
					for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
						int node = iter2->second;
						int s = iter2->first;
						stringstream strout;
						strout << "hI" << "(" << v << "," << t << "," << s << ")";
						const string name = strout.str();
						const char *cstr = name.c_str();
						hI[v][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr);	//float
						stringstream strout2;
						strout2 << "hO" << "(" << v << "," << t << "," << s << ")";
						const string name2 = strout2.str();
						const char *cstr2 = name2.c_str();
						hO[v][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr2);	//float
					}
				}
			}

		


			//Allocating memory
			charterCostMatrix.resize(idata.nShips);
			charterOutMatrix.resize(idata.nShips);
			buyingMatrix.resize(idata.nShips);
			sellingMatrix.resize(idata.nShips);

			for (int v = 0; v < idata.nShips; v++) {
				charterCostMatrix[v].resize(nodes);
				charterOutMatrix[v].resize(nodes);
				buyingMatrix[v].resize(nodes);
				sellingMatrix[v].resize(nodes);
			}

			charterCostObjective.resize(nodes);
			charterOutObjective.resize(nodes);
			buyingObjective.resize(nodes);
			sellingObjective.resize(nodes);

			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;

				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr initObjective(env, var);
					IloNumExpr initObjective2(env, var);

					charterCostObjective[node] = initObjective;
					charterOutObjective[node] = initObjective2;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						IloNumExpr init(env, var);
						IloNumExpr init2;
						charterCostMatrix[v][node] = init;
						charterOutMatrix[v][node] = init2;
					}

				}
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;

				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr initObjective(env, var);
					IloNumExpr initObjective2(env, var);

					buyingObjective[node] = initObjective;
					sellingObjective[node] = initObjective2;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						IloNumExpr init(env, var);
						IloNumExpr init2;
						buyingMatrix[v][node] = init;
						sellingMatrix[v][node] = init2;
					}
				}
			}


			//Adding elements to the objective function
			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						charterCostMatrix[v][node] = days*nodeData.CCI[v][node] * hI[v][node];
						charterCostObjective[node] += charterCostMatrix[v][node];

						charterOutMatrix[v][node] = days*nodeData.CCI[v][node] * hO[v][node];
						charterOutObjective[node] += charterOutMatrix[v][node];
					}
				}
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						buyingMatrix[v][node] = nodeData.secondHand[v][node] * ySH[v][node];
						buyingObjective[node] += buyingMatrix[v][node];

						sellingMatrix[v][node] = nodeData.secondHand[v][node] * ySE[v][node];
						sellingObjective[node] += sellingMatrix[v][node];

					}
				}
			}

			for (int j = 1; j < nodes; j++) {
				objfunc -= nodeData.Prob[j] * charterCostObjective[j];
				objfunc += nodeData.Prob[j] * charterOutObjective[j];
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					objfunc -= nodeData.Prob[node] * buyingObjective[node];
					objfunc += nodeData.Prob[node] * sellingObjective[node];
				}
			}


			//------------------Finished objective function-----------------------------------


			//Creating cashflow expressions

			for (int j = 1; j < nodes; j++) {
				CashFlowIn[j] = revenueCObjective[j] + revenueOObjective[j] + layUpSavingsObjective[j] + charterOutObjective[j];
				CashFlowOut[j] = spaceCharterCostObjective[j] + operatingCostObjective[j] + charterCostObjective[j] +  loopCostObjective[j];
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowIn[node] += (scrappingObjective[node] + sellingObjective[node]);
				}
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowOut[node] += buyingObjective[node];
				}
			}

			for (int t = 0; t < idata.Time; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowOut[node] += instalmentObjective[node];
				}
			}




			cout << "Running add option = 1" << endl;
			break;
		}

		case 2: {
			//Initialising variables
			for (int t = 0; t < idata.Horizon; t++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					map<int, int>::const_iterator iter2;
					for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
						int node = iter2->second;
						int s = iter2->first;
						stringstream strout;
						strout << "ySE" << "(" << v << "," << t << "," << s << ")";
						const string name = strout.str();
						const char *cstr = name.c_str();
						ySE[v][node] = IloNumVar(env, 0, IloInfinity, ILOINT, cstr);	//int

						stringstream strout2;
						strout2 << "ySH" << "(" << v << "," << t << "," << s << ")";
						const string name2 = strout2.str();
						const char *cstr2 = name2.c_str();
						ySH[v][node] = IloNumVar(env, 0, IloInfinity, ILOINT, cstr2);	//int
					}

				}
			}

			//Allocating memory
			
			buyingMatrix.resize(idata.nShips);
			sellingMatrix.resize(idata.nShips);

			for (int v = 0; v < idata.nShips; v++) {
				buyingMatrix[v].resize(nodes);
				sellingMatrix[v].resize(nodes);
			}

			
			buyingObjective.resize(nodes);
			sellingObjective.resize(nodes);

			

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;

				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr initObjective(env, var);
					IloNumExpr initObjective2(env, var);

					buyingObjective[node] = initObjective;
					sellingObjective[node] = initObjective2;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						IloNumExpr init(env, var);
						IloNumExpr init2;
						buyingMatrix[v][node] = init;
						sellingMatrix[v][node] = init2;
					}
				}
			}


			//Adding elements to the objective function
		
			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						buyingMatrix[v][node] = nodeData.secondHand[v][node] * ySH[v][node];
						buyingObjective[node] += buyingMatrix[v][node];

						sellingMatrix[v][node] = nodeData.secondHand[v][node] * ySE[v][node];
						sellingObjective[node] += sellingMatrix[v][node];

					}
				}
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					objfunc -= nodeData.Prob[node] * buyingObjective[node];
					objfunc += nodeData.Prob[node] * sellingObjective[node];
				}
			}


			//------------------Finished objective function-----------------------------------


			//Creating cashflow expressions

			for (int j = 1; j < nodes; j++) {
				CashFlowIn[j] = revenueCObjective[j] + revenueOObjective[j] + layUpSavingsObjective[j];
				CashFlowOut[j] = spaceCharterCostObjective[j] + operatingCostObjective[j] + loopCostObjective[j];
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowIn[node] += (scrappingObjective[node] + sellingObjective[node]);
				}
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowOut[node] += buyingObjective[node];
				}
			}

			for (int t = 0; t < idata.Time; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowOut[node] += instalmentObjective[node];
				}
			}


			cout << "Running add option = 2" << endl;

			break;
		}

		case 3: {
			
			//Initializing variables
			for (int t = 1; t < idata.Time; t++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					map<int, int>::const_iterator iter2;
					for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
						int node = iter2->second;
						int s = iter2->first;
						stringstream strout;
						strout << "hI" << "(" << v << "," << t << "," << s << ")";
						const string name = strout.str();
						const char *cstr = name.c_str();
						hI[v][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr);	//float
						stringstream strout2;
						strout2 << "hO" << "(" << v << "," << t << "," << s << ")";
						const string name2 = strout2.str();
						const char *cstr2 = name2.c_str();
						hO[v][node] = IloNumVar(env, 0, IloInfinity, ILOFLOAT, cstr2);	//float
					}
				}
			}

			//Allocating memory
			charterCostMatrix.resize(idata.nShips);
			charterOutMatrix.resize(idata.nShips);

			for (int v = 0; v < idata.nShips; v++) {
				charterCostMatrix[v].resize(nodes);
				charterOutMatrix[v].resize(nodes);
				}

			charterCostObjective.resize(nodes);
			charterOutObjective.resize(nodes);
			

			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;

				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr initObjective(env, var);
					IloNumExpr initObjective2(env, var);

					charterCostObjective[node] = initObjective;
					charterOutObjective[node] = initObjective2;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						IloNumExpr init(env, var);
						IloNumExpr init2;
						charterCostMatrix[v][node] = init;
						charterOutMatrix[v][node] = init2;
					}

				}
			}

			

			//Adding elements to the objective function
			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						charterCostMatrix[v][node] = days*nodeData.CCI[v][node] * hI[v][node];
						charterCostObjective[node] += charterCostMatrix[v][node];

						charterOutMatrix[v][node] = days*nodeData.CCI[v][node] * hO[v][node];
						charterOutObjective[node] += charterOutMatrix[v][node];
					}
				}
			}

			

			for (int j = 1; j < nodes; j++) {
				objfunc -= nodeData.Prob[j] * charterCostObjective[j];
				objfunc += nodeData.Prob[j] * charterOutObjective[j];
			}

			

			//------------------Finished objective function-----------------------------------


			//Creating cashflow expressions

			for (int j = 1; j < nodes; j++) {
				CashFlowIn[j] = revenueCObjective[j] + revenueOObjective[j] + layUpSavingsObjective[j] + charterOutObjective[j];
				CashFlowOut[j] = spaceCharterCostObjective[j] + operatingCostObjective[j] + charterCostObjective[j] + loopCostObjective[j];
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowIn[node] += (scrappingObjective[node]);
				}
			}

			for (int t = 0; t < idata.Time; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowOut[node] += instalmentObjective[node];
				}
			}

			cout << "Running add option = 3" << endl;

			break;
		}

		default: {



			//Creating cashflow expressions

			for (int j = 1; j < nodes; j++) {
				CashFlowIn[j] = revenueCObjective[j] + revenueOObjective[j] + layUpSavingsObjective[j];
				CashFlowOut[j] = spaceCharterCostObjective[j] + operatingCostObjective[j] + loopCostObjective[j]; 
			}

			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowIn[node] += scrappingObjective[node];
				}
			}

			for (int t = 0; t < idata.Time; t++) {
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					CashFlowOut[node] += instalmentObjective[node];
				}
			}
		

			cout << "Running basic model" << endl;
		}
		}




	


		//----------------Constraints--------------------

		//Demand Constraints

		// 2

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
				int i = iterTrades->second;
					map<int, int>::const_iterator iterNode;
					for (iterNode = nodeData.NodesTime[t].begin(); iterNode != nodeData.NodesTime[t].end(); iterNode++) {
						int node = iterNode->second;
						for (int k = 0; k < idata.nProduct; k++) {
						IloNumExpr temp(env, var);
						map<string, int>::const_iterator iterShips;
						for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
							int v = iterShips->second;
							map<string, int>::const_iterator iterRoutes;
							for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
								int r = iterRoutes->second;
								temp += idata.Capacity[v][k] * x[v][r][node];
							}
						}

						basicModel.add(temp + n[i][k][node] >= nodeData.Demand[i][k][node]);
					}
				}
			}
		}

		// 3

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
				int i = iterTrades->second;
				
					map<int, int>::const_iterator iterNode;
					for (iterNode = nodeData.NodesTime[t].begin(); iterNode != nodeData.NodesTime[t].end(); iterNode++) {
						int node = iterNode->second;
						for (int k = 0; k < idata.nProduct; k++) {
						IloNumExpr temp(env, var);
						map<string, int>::const_iterator iterShips;
						for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
							int v = iterShips->second;
							map<string, int>::const_iterator iterRoutes;
							for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
								int r = iterRoutes->second;
								temp += idata.Capacity[v][k] * x[v][r][node];
							}
						}
						basicModel.add(temp >= (nodeData.Demand[i][k][node] * d[i][node]));
					}
				}
			}
		}

		//Capacity constraints

		//4

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
				int i = iterTrades->second;

				map<int, int>::const_iterator iterNode;
				for (iterNode = nodeData.NodesTime[t].begin(); iterNode != nodeData.NodesTime[t].end(); iterNode++) {
					int node = iterNode->second;
					IloNumExpr temp(env);
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						map<string, int>::const_iterator iterRoutes;
						for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
							int r = iterRoutes->second;
							temp += idata.TotalCapacity[v] * x[v][r][node];
						}
					}
					IloNumExpr temp2(env);
					IloNumExpr temp3(env);
					for (int k = 0; k < idata.nProduct; k++) {
						temp2 += n[i][k][node];
						temp3 += nodeData.Demand[i][k][node];
					}
					basicModel.add(temp + temp2 >= temp3);
				}
			}
		}

		//5

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
				int i = iterTrades->second;

				map<int, int>::const_iterator iterNode;
				for (iterNode = nodeData.NodesTime[t].begin(); iterNode != nodeData.NodesTime[t].end(); iterNode++) {
					int node = iterNode->second;
					IloNumExpr temp(env, var);
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						map<string, int>::const_iterator iterRoutes;
						for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
							int r = iterRoutes->second;
							temp += idata.TotalCapacity[v] * x[v][r][node];
						}
					}
					IloNumExpr temp2(env, var);
					for (int k = 0; k < idata.nProduct; k++) {
						temp2 += nodeData.Demand[i][k][node] * d[i][node];
					}
					basicModel.add(temp >= temp2);
				}
			}
		}

		//Frequency constraints

		//6
		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
				int i = iterTrades->second;

				map<int, int>::const_iterator iterNode;
				for (iterNode = nodeData.NodesTime[t].begin(); iterNode != nodeData.NodesTime[t].end(); iterNode++) {
					int node = iterNode->second;
					IloNumExpr temp(env, var);
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						map<string, int>::const_iterator iterRoutes;
						for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
							int r = iterRoutes->second;
							temp += x[v][r][node];
						}
					}
					if (nodeData.MinFreq[i][node] > 0) {
						basicModel.add(temp >= nodeData.MinFreq[i][node]);
					}
				}
			}
		}

		// 7

		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
				int i = iterTrades->second;

				map<int, int>::const_iterator iterNode;
				for (iterNode = nodeData.NodesTime[t].begin(); iterNode != nodeData.NodesTime[t].end(); iterNode++) {
					int node = iterNode->second;
					IloNumExpr temp(env, var);
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						map<string, int>::const_iterator iterRoutes;
						for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
							int r = iterRoutes->second;
							temp += x[v][r][node];
						}
					}
					if (nodeData.MinFreq[i][node] > 0) {
						basicModel.add(temp >= nodeData.MinFreq[i][node] * d[i][node]);
					}
				}
			}
		}

		
			//8

		for (int t = 1; t < idata.Time; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				map<string, int>::const_iterator iterShips;
				for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
					int v = iterShips->second;
					map<string, int>::const_iterator iterRoutes;
					IloNumExpr temp(env, var);
					for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
						int r = iterRoutes->second;
						temp += idata.Duration[v][r] * x[v][r][node];
					}
					switch (addOption) {
					case 1: {
						basicModel.add(temp <= (nodeData.availableTime[v] * (yP[v][node] + hI[v][node] - hO[v][node] - layUp[v][node])));
						break;
					}
					case 2: {
						basicModel.add(temp <= (nodeData.availableTime[v] * (yP[v][node] - layUp[v][node])));
						break;
					}
					case 3: {
						basicModel.add(temp <= (nodeData.availableTime[v] * (yP[v][node] + hI[v][node] - hO[v][node] - layUp[v][node])));
						break;
					}
					default: {
						basicModel.add(temp <= (nodeData.availableTime[v] * (yP[v][node] - layUp[v][node])));
					}
					}
				}
			}
		}
		//Time constraints

		

		// Optional trades constraints

		// 9

		for (int t = 2; t < idata.Time; t++) {
			map<string, int>::const_iterator iterTrades;
			for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
				int i = iterTrades->second;
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					map<int, int> temp;
					int node = iterNodes->second;
					temp = nodeData.AncestorNodes[t - 1];
					int a = temp[node];

					basicModel.add(d[i][a] <= d[i][node]);

				}
			}
		}

		//Optional trades constraints end


		//Pool constraints

		int tPrevious;
		for (int t = 0; t < idata.Time; t++) {
			tPrevious = t - 1;
			int tOrdered = t - buildTime;




			if (t == 0) {
				map<string, int>::const_iterator iterInitial;
				int node = 0;
				for (iterInitial = idata.VoF[t].begin(); iterInitial != idata.VoF[t].end(); iterInitial++) {
					int vInitial = iterInitial->second;
					basicModel.add(yP[vInitial][node] == idata.InitialPool[vInitial] + idata.InitialNB[vInitial][t]);		//Constraint (10) & (13)
				}
			}
			else {

				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					bool flag = false;

					if (t < idata.Horizon) {
						bool flagCont = false;
						map<string, int>::const_iterator iter2;
						for (iter2 = idata.VoF[t + 1].begin(); iter2 != idata.VoF[t + 1].end(); iter2++) {	//Checks if the ship is available in the next period
							int vNext = iter2->second;
							if (v == vNext) {
								flagCont = true;
								break;
							}
						}
						if (!flagCont) {
							map<int, int>::const_iterator iterNodes;
							for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
								int node = iterNodes->second;
								basicModel.add(yP[v][node] == ySC[v][node]); //(15)
							}
						}
					}

					map<string, int>::const_iterator iter2;
					for (iter2 = idata.VN[t].begin(); iter2 != idata.VN[t].end(); iter2++) {
						int vN = iter2->second;
						if (v == vN) {
							flag = true;
						}
					}
					if (flag) {
						if (tOrdered < 0) {
							map<int, int>::const_iterator iterNodes;
							for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
								int node = iterNodes->second;
								basicModel.add(yP[v][node] == idata.InitialNB[v][t]);			//(13)
							}
						}
						else {
							map<int, int>::const_iterator iterNodes;
							for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
								map<int, int> temp;
								int node = iterNodes->second;
								temp = nodeData.AncestorNodes[tOrdered];
								int a = temp[node];
								basicModel.add(yP[v][node] == yNB[v][a]);				//(12)
							}
						}
					}
					else {
						map<int, int>::const_iterator iterNodes;
						for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
							map<int, int> temp;
							int node = iterNodes->second;
							temp = nodeData.AncestorNodes[t - 1];
							int a = temp[node];
							switch(addOption) {
							case 1: {
								basicModel.add(yP[v][node] == (yP[v][a] - ySC[v][a]) - ySE[v][a] + ySH[v][a]);	//(11)
								break;
							}
							case 2: {
								basicModel.add(yP[v][node] == (yP[v][a] - ySC[v][a]) - ySE[v][a] + ySH[v][a]);
								break;
							}
							case 3: {
								basicModel.add(yP[v][node] == (yP[v][a] - ySC[v][a]));
								break;
							}
							default: {
								basicModel.add(yP[v][node] == (yP[v][a] - ySC[v][a]));	//(11)
							}
							}
							
						}
					}
				}
			}
		}



		//Constraint (14)
		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					switch (addOption) {
					case 1: {
						basicModel.add(layUp[v][node] - hI[v][node] + hO[v][node] <= yP[v][node]);
						break;
					}
					case 2: {
						basicModel.add(layUp[v][node] <= yP[v][node]);
						break;
					}
					case 3: {
						basicModel.add(layUp[v][node] - hI[v][node] + hO[v][node] <= yP[v][node]);
						break;
					}
					default: {
						basicModel.add(layUp[v][node] <= yP[v][node]);
					}
					}
					
				}
			}
		}
		// Pool constraints end



		//Charters constraints

		switch (addOption) {
		case 1: {
			// (16)
			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;

						basicModel.add(hI[v][node] <= idata.CharterInLimit[v]);
						basicModel.add(hO[v][node] <= idata.CharterOutLimit[v]);
					}
				}
			}
			
			// (17)
			for (int t = 1; t < idata.Time; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			IloNumExpr temp(env, var);
			IloNumExpr temp2(env, var);

			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
			int v = iterShips->second;
			temp += hI[v][node];
			temp2 += hO[v][node];

			}
			basicModel.add(temp <= idata.CharterInLimitTot);
			basicModel.add(temp2 <= idata.CharterOutLimitTot);
			}
			}


			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						basicModel.add(ySE[v][node] <= idata.sellingLimit[v]);
						basicModel.add(ySH[v][node] <= idata.buyingLimit[v]);
					}
				}
			}


			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr temp(env, var);
					IloNumExpr temp2(env, var);

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						temp += ySE[v][node];
						temp2 += ySH[v][node];

					}
					basicModel.add(temp <= idata.sellingLimitTot);
					basicModel.add(temp2 <= idata.buyingLimitTot);
				}
			}


			break;
		}
		case 2: {
			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						basicModel.add(ySE[v][node] <= idata.sellingLimit[v]);
						basicModel.add(ySH[v][node] <= idata.buyingLimit[v]);
					}
				}
			}


			for (int t = 0; t < idata.Horizon; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr temp(env, var);
					IloNumExpr temp2(env, var);

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						temp += ySE[v][node];
						temp2 += ySH[v][node];

					}
					basicModel.add(temp <= idata.sellingLimitTot);
					basicModel.add(temp2 <= idata.buyingLimitTot);
				}
			}

			break;
		}
		case 3: {
			// (16)
			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;

						basicModel.add(hI[v][node] <= idata.CharterInLimit[v]);
						basicModel.add(hO[v][node] <= idata.CharterOutLimit[v]);
					}
				}
			}

			// (17)
			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					IloNumExpr temp(env, var);
					IloNumExpr temp2(env, var);

					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						temp += hI[v][node];
						temp2 += hO[v][node];

					}
					basicModel.add(temp <= idata.CharterInLimitTot);
					basicModel.add(temp2 <= idata.CharterOutLimitTot);
				}
			}
			break;
		}
		default: {

		}
		}
	
		

		


	/*
		int number = 0;
		for (int t = 0; t < idata.Horizon-nodeData.buildTime+nodeData.M; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.InstalmentNodes[t].begin(); iterNodes != nodeData.InstalmentNodes[t].end(); iterNodes++) {
				int node = iterNodes->second;
				IloExpr Instalment = instalmentObjective[node];
				for (IloExpr::LinearIterator itb = Instalment.getLinearIterator();
					itb.ok(); ++itb) {
					cout << "Node: " << node << "\t";
					cout << "variable in obj: " << itb.getVar() <<
						"coef in obj:  " << itb.getCoef() << endl;
					number++;
				}
			}
		}
		cout << endl;
		cout << "Sunset value: " << endl;
		number = 0;
		
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[idata.Horizon].begin(); iterNodes != nodeData.NodesTime[idata.Horizon].end(); iterNodes++) {
				int node = iterNodes->second;
				IloExpr Instalment = sunsetObjective[node];
				for (IloExpr::LinearIterator itb = Instalment.getLinearIterator();
					itb.ok(); ++itb) {
					cout << number << "\t";
					cout << "variable in obj: " << itb.getVar() <<
						"coef in obj:  " << itb.getCoef() << endl;
					number++;
				}
			}

			
			*/

		switch (modelOption) {
		case 1: {
			cout << "Using Lt-constraints" << endl;

			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

						basicModel.add(CashFlowIn[node] >= Lt*CashFlowOut[node]);
				}
			}

			break;
		}
		case 2: {
			cout << "Using CVaR-constraints" << endl;

			
			
			basicModel.add(CashFlowIn[0] - CashFlowOut[0] + investmentFactor * revenueCObjective[1] >= Lt);	//Limiting cash flows in period 0

			for (int t = 1; t < nodeData.branchPeriod; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					basicModel.add(CashFlowIn[node] - CashFlowOut[node] >= Lt);
				}
			}

			

			for (int t = nodeData.branchPeriod; t < idata.Time; t++) {
				IloNumExpr cVar(env, var);
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					cVar += (nodeData.Prob[node]) * flow[node];	

					basicModel.add(flow[node] >= ((CashFlowOut[node] - CashFlowIn[node]) - zeta));
				}
				basicModel.add((zeta + (1 / (1 - confidenceLevel))*cVar) <= -Lt);
			}

			

			cout << "Cash flow limit: " << Lt << endl;

			break;
		}

		case 3: {
			cout << "Using absolute cashflow terms" << endl;

			basicModel.add(CashFlowIn[0] - CashFlowOut[0] + investmentFactor * revenueCObjective[1] >= Lt);	//Limiting cash flows in period 0

			for (int t = 1; t < idata.Time; t++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;

					basicModel.add(CashFlowIn[node] - CashFlowOut[node] >= Lt);
				}
			}
		}
		default: {
			cout << "Running basic model!" << endl;
			
		}
		}

/*
		if (modelOption == 2) {

			IloObjective objective = IloMinimize(env, -objfunc);

			basicModel.add(objective);

		}

		else {
			IloObjective objective = IloMaximize(env, objfunc);

			basicModel.add(objective);
		}
		*/


		IloObjective objective = IloMaximize(env, objfunc);

		basicModel.add(objective);

		//---------Solve--------------

		IloCplex cplex(basicModel);
		cplex.setParam(IloCplex::TiLim, 8000);	//Setting the time limit in seconds
		//cplex.setParam(IloCplex::PreInd, 0);	//Turns off the presolver
		cplex.setParam(IloCplex::EpGap, 0.01);	//Sets the allowed optimality gap
		string filePath = "nodeModel.lp";
		//cplex.exportModel(filePath.c_str());			//Printing the model

		cplex.solve();			//Solving the model

		cout << cplex.getCplexStatus() << endl;

		//cplex.writeSolution("Solution.sol");	//Writing the solution in a format used by cplex


		cout << "Total profit: " << cplex.getValue(objfunc) << endl;

		if (modelOption == 2) {
			cout << "VaR: " << cplex.getValue(zeta) << endl;
		}

		//Saving results for profit
		Profit = cplex.getValue(objfunc);

		for (int t = 1; t < idata.Time; t++) {
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				for (int k = 0; k < idata.nProduct; k++) {
					map<string, int>::const_iterator iterTrades;
					for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
						int i = iterTrades->second;
						revenueOResults[i][k][node] = cplex.getValue(revenueOMatrix[i][k][node]);
					}
					for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
						int i = iterTrades->second;
						revenueCResults[i][k][node] = revenueCMatrix[i][k][node];
						spaceCharterCostResults[i][k][node] = cplex.getValue(spaceCharterCostMatrix[i][k][node]);

					}
				}

				map<string, int>::const_iterator iterShips;
				for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
					int v = iterShips->second;

					//cout << "operating" << endl;
					operatingCostResults[v][node] = cplex.getValue(operatingCostMatrix[v][node]);

					//cout << "charter" << endl;
					switch (addOption) {
					case 1: {
						charterCostResults[v][node] = cplex.getValue(charterCostMatrix[v][node]);
						charterOutResults[v][node] = cplex.getValue(charterOutMatrix[v][node]);
						break;
					}
					case 3: {
						charterCostResults[v][node] = cplex.getValue(charterCostMatrix[v][node]);
						charterOutResults[v][node] = cplex.getValue(charterOutMatrix[v][node]);
						break;
					}
					default: {

					}
					}
					

					//cout << "layUp" << endl;

					layUpSavingsResults[v][node] = cplex.getValue(layUpSavingsMatrix[v][node]);

					map<string, int>::const_iterator iterRoutes;
					for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
						int r = iterRoutes->second;
						//cout << "loop" << endl;
						loopCostResults[v][r][node] = cplex.getValue(loopCostMatrix[v][r][node]);
					}
				}
			}
		}


		for (int tmarked = 0; tmarked < idata.Time - nodeData.buildTime; tmarked++) {
			map<string, int>::const_iterator iterShips;
			for (iterShips = idata.VN[tmarked + nodeData.buildTime].begin(); iterShips != idata.VN[tmarked + nodeData.buildTime].end(); iterShips++) {
				int v = iterShips->second;

				for (int t = tmarked + 1; t < tmarked + nodeData.M; t++) {	// +1 is used to access ancestor node correctly
					map<int, int>::const_iterator iterNodes;
					for (iterNodes = nodeData.InstalmentNodes[t].begin(); iterNodes != nodeData.InstalmentNodes[t].end(); iterNodes++) {
						int node = iterNodes->second;
						map<int, int> temp;
						temp = nodeData.AncestorInstalmentNodes[tmarked][t];
						int a = temp[node];
						instalmentCostResults[v][a][node] = cplex.getValue(instalmentCostMatrix[v][a][node]);
						instalmentCostResults[v][a][a] = cplex.getValue(instalmentCostMatrix[v][a][a]);
					}
				}
			}
		}


		for (iter = idata.VoF[idata.Horizon].begin(); iter != idata.VoF[idata.Horizon].end(); iter++) {
			int v = iter->second;
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[idata.Horizon].begin(); iterNodes != nodeData.NodesTime[idata.Horizon].end(); iterNodes++) {
				int node = iterNodes->second;
				sunsetValueResults[v][node] = cplex.getValue(sunsetValueMatrix[v][node]);
			}
		}

		for (int t = 0; t < idata.Horizon; t++) {		//Not possible to scrap in last period
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					switch (addOption) {
					case 1: {
						scrappingValueResults[v][node] = cplex.getValue(scrappingValueMatrix[v][node]);
						sellingResults[v][node] = cplex.getValue(sellingMatrix[v][node]);
						buyingResults[v][node] = cplex.getValue(buyingMatrix[v][node]);
						break;
					}
					case 2: {
						scrappingValueResults[v][node] = cplex.getValue(scrappingValueMatrix[v][node]);
						sellingResults[v][node] = cplex.getValue(sellingMatrix[v][node]);
						buyingResults[v][node] = cplex.getValue(buyingMatrix[v][node]);
						break;
					}
					default: {
						scrappingValueResults[v][node] = cplex.getValue(scrappingValueMatrix[v][node]);
		}
				}
				}
			}
		}

		//Saving results for variablels

		//New builds
		for (int t = 0; t < idata.Time - buildTime; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VN[t + buildTime].begin(); iter != idata.VN[t + buildTime].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					yNBResults[v][node] = cplex.getValue(yNB[v][node]);
				}
			}
		}

		//Scrappings
		for (int t = 0; t < idata.Horizon; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					switch (addOption) {
					case 1: {
						ySCResults[v][node] = cplex.getValue(ySC[v][node]);
						ySEResults[v][node] = cplex.getValue(ySE[v][node]);
						ySHResults[v][node] = cplex.getValue(ySH[v][node]);
						break;
					}
					case 2: {
						ySCResults[v][node] = cplex.getValue(ySC[v][node]);
						ySEResults[v][node] = cplex.getValue(ySE[v][node]);
						ySHResults[v][node] = cplex.getValue(ySH[v][node]);
						break;
					}
					default: {
						ySCResults[v][node] = cplex.getValue(ySC[v][node]);
					}
					}
					
				}
			}
		}

		//Optional trades
		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.TradesOOf[t].begin(); iter != idata.TradesOOf[t].end(); iter++) {
				int i = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					dResults[i][node] = cplex.getValue(d[i][node]);
				}
			}
		}

		//Space charter
		for (int t = 1; t < idata.Time; t++) {
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				map<string, int>::const_iterator iterTrades;
				for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
					int i = iterTrades->second;

					for (int k = 0; k < idata.nProduct; k++) {
						nResults[i][k][node] = cplex.getValue(n[i][k][node]);
					}
				}
			}
		}

		//Lay-Up
		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					layUpResults[v][node] = cplex.getValue(layUp[v][node]);
				}
			}
		}

		//Charter in
		switch (addOption) {
		case 1: {
			for (int t = 1; t < idata.Time; t++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					map<int, int>::const_iterator iter2;
					for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
						int node = iter2->second;
						hIResults[v][node] = cplex.getValue(hI[v][node]);
						hOResults[v][node] = cplex.getValue(hO[v][node]);
					}
				}
			}
			break;
		}
		case 3: {
			for (int t = 1; t < idata.Time; t++) {
				map<string, int>::const_iterator iter;
				for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
					int v = iter->second;
					map<int, int>::const_iterator iter2;
					for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
						int node = iter2->second;
						hIResults[v][node] = cplex.getValue(hI[v][node]);
						hOResults[v][node] = cplex.getValue(hO[v][node]);
					}
				}
			}
			break;
		}
		default: {

		}
		}


		//Pool
		for (int t = 0; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;
					yPResults[v][node] = cplex.getValue(yP[v][node]);
				}
			}
		}

		//Loops
		for (int t = 1; t < idata.Time; t++) {
			map<string, int>::const_iterator iter;
			for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
				int v = iter->second;
				map<int, int>::const_iterator iter2;
				for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
					int node = iter2->second;

					map<string, int>::const_iterator iterRoutes;
					for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
						int r = iterRoutes->second;
						xResults[v][r][node] = cplex.getValue(x[v][r][node]);
					}
				}
			}
		}

		//CashFlows
		cashInFlow.resize(nodes);
		cashOutFlow.resize(nodes);
		for (int node = 0; node < nodes; node++){
			cashInFlow[node] = cplex.getValue(CashFlowIn[node]);
			cashOutFlow[node] = cplex.getValue(CashFlowOut[node]);

		}


		switch (modelOption) {
		case 1: {
			cout << "Using L-constraints: " << Lt << endl;
			break;
		}
		case 2: {
			cout << "Using CVaR-constraints" << endl;
			break;
		}
		case 3: {
			cout << "Using absolute cash flow constraints: " << Lt << endl;
			break;
		}
		default: {

			calculateRobustnesFactor(nodeData);
			calculateWorstCashFlow(nodeData);


		}

		}


		//cout << "operating" << endl;	//For debugging

	}

	catch (IloException& e) {
		feasible = false;
		cerr << "Concert exception caught: " << e << endl;
	}
	catch (...) {
		feasible = false;
		cerr << "Unknown exception caught" << endl;
	}

	env.end();

}

void nodeModel::aggregateResults(const nodeData& nodeData, const ImportData& idata)
{
	//Allocating memory
	revenueOObjectiveResults.resize(nodeData.Nodes.size());
	revenueCObjectiveResults.resize(nodeData.Nodes.size());
	spaceCharterCostObjectiveResults.resize(nodeData.Nodes.size());
	operatingCostObjectiveResults.resize(nodeData.Nodes.size());
	charterCostObjectiveResults.resize(nodeData.Nodes.size());
	charterOutObjectiveResults.resize(nodeData.Nodes.size());
	loopCostObjectiveResults.resize(nodeData.Nodes.size());
	layUpSavingsObjectiveResults.resize(nodeData.Nodes.size());
	sellingObjectiveResults.resize(nodeData.Nodes.size());
	buyingObjectiveResults.resize(nodeData.Nodes.size());
	instalmentObjectiveResults.resize(nodeData.InstNodes.size());
	sunsetObjectiveResults.resize(nodeData.Nodes.size());
	scrappingObjectiveResults.resize(nodeData.Nodes.size());

	CashFlow.resize(nodeData.Nodes.size());

	for (int node = 0; node < nodeData.Nodes.size(); node++) {
		revenueOObjectiveResults[node] = 0;
		revenueCObjectiveResults[node] = 0;
		spaceCharterCostObjectiveResults[node] = 0;
		operatingCostObjectiveResults[node] = 0;
		charterCostObjectiveResults[node] = 0;
		charterOutObjectiveResults[node] = 0;
		loopCostObjectiveResults[node] = 0;
		layUpSavingsObjectiveResults[node] = 0;
		sunsetObjectiveResults[node] = 0;
		sellingObjectiveResults[node] = 0;
		buyingObjectiveResults[node] = 0;
		scrappingObjectiveResults[node] = 0;
		CashFlow[node] = 0;
	}
	for (int node = 0; node < nodeData.InstNodes.size(); node++) {
		instalmentObjectiveResults[node] = 0;
	}

	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iterTrades;
for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
	int i = iterTrades->second;
	for (int k = 0; k < idata.nProduct; k++) {
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			revenueOObjectiveResults[node] += revenueOResults[i][k][node];

		}
	}
}
	}

	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iterTrades;
		for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
			int i = iterTrades->second;
			for (int k = 0; k < idata.nProduct; k++) {
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					revenueCObjectiveResults[node] += revenueCResults[i][k][node];
					spaceCharterCostObjectiveResults[node] += spaceCharterCostResults[i][k][node];
				}
			}
		}
	}


	for (int node = 0; node < nodeData.Nodes.size(); node++) {
		for (int v = 0; v < idata.nShips; v++) {
			operatingCostObjectiveResults[node] += operatingCostResults[v][node];
			charterCostObjectiveResults[node] += charterCostResults[v][node];
			charterOutObjectiveResults[node] += charterOutResults[v][node];
			layUpSavingsObjectiveResults[node] += layUpSavingsResults[v][node];
			sunsetObjectiveResults[node] += sunsetValueResults[v][node];
			scrappingObjectiveResults[node] += scrappingValueResults[v][node];
			sellingObjectiveResults[node] += sellingResults[v][node];
			buyingObjectiveResults[node] += buyingResults[v][node];
			for (int r = 0; r < idata.nRoutes; r++) {
				loopCostObjectiveResults[node] += loopCostResults[v][r][node];
			}
		}
	}

	for (int v = 0; v < idata.nShips; v++) {
		for (int a = 0; a < nodeData.Nodes.size(); a++) {
			for (int node = 0; node < nodeData.InstNodes.size(); node++) {
				instalmentObjectiveResults[node] += instalmentCostResults[v][a][node];
			}
		}
	}

	for (int node = 0; node < nodeData.Nodes.size(); node++) {
		CashFlow[node] = cashInFlow[node] - cashOutFlow[node];

	}




	//----------------Aggregated decision variables for every node -------------------

	//Allocating memory
	ySENodes.resize(nodeData.Nodes.size());
	ySHNodes.resize(nodeData.Nodes.size());
	yNBNodes.resize(nodeData.Nodes.size());
	ySCNodes.resize(nodeData.Nodes.size());
	yPNodes.resize(nodeData.Nodes.size());
	hINodes.resize(nodeData.Nodes.size());
	hONodes.resize(nodeData.Nodes.size());
	layUpNodes.resize(nodeData.Nodes.size());
	xNodes.resize(nodeData.Nodes.size());
	nNodes.resize(nodeData.Nodes.size());
	dNodes.resize(nodeData.Nodes.size());

	//Initializing

	for (int i = 0; i < nodeData.Nodes.size(); i++) {
		ySENodes[i] = 0;
		ySHNodes[i] = 0;
		yNBNodes[i] = 0;
		ySCNodes[i] = 0;
		yPNodes[i] = 0;
		hINodes[i] = 0;
		hONodes[i] = 0;
		layUpNodes[i] = 0;
		xNodes[i] = 0;
		nNodes[i] = 0;
		dNodes[i] = 0;
	}


	//Aggregating

	//Scrapping, second hand market

	for (int t = 0; t < idata.Horizon; t++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
			int v = iterShips->second;

			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;

				ySENodes[node] += ySEResults[v][node];
				ySHNodes[node] += ySHResults[v][node];
				ySCNodes[node] += ySCResults[v][node];

			}
		}
	}

	//New build

	for (int t = 0; t < idata.Time - nodeData.buildTime; t++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VN[t+nodeData.buildTime].begin(); iterShips != idata.VN[t+nodeData.buildTime].end(); iterShips++) {
			int v = iterShips->second;

			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;

				yNBNodes[node] += yNBResults[v][node];

			}
		}
	}

	//Pool

	for (int t = 0; t < idata.Time; t++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
			int v = iterShips->second;

			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;

				yPNodes[node] += yPResults[v][node];

			}
		}
	}


	//Charters, layUp
	for (int t = 1; t < idata.Time; t++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
			int v = iterShips->second;

			map<int, int>::const_iterator iterNodes;
for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
	int node = iterNodes->second;

	hINodes[node] += hIResults[v][node];
	hONodes[node] += hOResults[v][node];
	layUpNodes[node] += layUpResults[v][node];

}
		}
	}

	//Loops

	for (int t = 1; t < idata.Time; t++) {

		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
			int v = iterShips->second;

			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;

				map<string, int>::const_iterator iterRoutes;
				for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
					int r = iterRoutes->second;

					xNodes[node] += xResults[v][r][node];

				}

			}
		}
	}

	//Space charter

	for (int t = 1; t < idata.Time; t++) {

		map<string, int>::const_iterator iterTrades;
		for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
			int i = iterTrades->second;

			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;

				for (int k = 0; k < idata.nProduct; k++) {

					nNodes[node] += nResults[i][k][node];

				}
			}
		}
	}

	//Optional trades

	for (int t = 1; t < idata.Time; t++) {

		map<string, int>::const_iterator iterTrades;
		for (iterTrades = idata.TradesOOf[t].begin(); iterTrades != idata.TradesOOf[t].end(); iterTrades++) {
			int i = iterTrades->second;

			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;

				dNodes[node] += dResults[i][node];

			}
		}
	}


}

void nodeModel::calculateRobustnesFactor(const nodeData& nodeData)
{
	L = (numeric_limits<double>::max)();

	for (int i = 1; i < nodeData.Nodes.size(); i++) {
		if (cashOutFlow[i] > 0) {
			double temp = cashInFlow[i] / cashOutFlow[i];

			if (temp < L) {
				L = temp;
			}
		}

	}

	cout << "Robustnes factor: " << L << endl;

}

void nodeModel::calculateWorstCashFlow(const nodeData & nodeData)
{
	worstCashFlow = (numeric_limits<double>::max)();

	for (int i = 1; i < nodeData.Nodes.size(); i++) {
		if (worstCashFlow > (cashInFlow[i] - cashOutFlow[i])) {
			worstCashFlow = (cashInFlow[i] - cashOutFlow[i]);
		}
		
}
	cout << "Worst case cash flow: " << worstCashFlow << endl;

}

template <class T>	
double nodeModel::average(const vector<vector<T>>& variable, const nodeData& nodeData, const int & time) {
	
	double average = 0;
	map<int, int>::const_iterator iterNodes;
	for (iterNodes = nodeData.NodesTime[time].begin(); iterNodes != nodeData.NodesTime[time].end(); iterNodes++) { 
		int node = iterNodes->second;
		for (int i = 0; i < variable.size(); i++) {
			average += variable[i][node];
		}
	}
	average = average / nodeData.NodesTime[time].size();

	return average;

}

template<class T>
double nodeModel::average(const vector<vector<vector<T>>>& variable, const nodeData& nodeData, const int & time) {
	
	double average = 0;
	map<int, int>::const_iterator iterNodes;
	for (iterNodes = nodeData.NodesTime[time].begin(); iterNodes != nodeData.NodesTime[time].end(); iterNodes++) {
		int node = iterNodes->second;
		for (int i = 0; i < variable.size(); i++) {
			for (int j = 0; j < variable[i].size(); j++) {
				average += variable[i][j][node];
			}
		}
	}
	average = average / nodeData.NodesTime[time].size();

	return average;
}

void nodeModel::writeToFile(string filename, const nodeData& nodeData, const ImportData& idata)
{
	ofstream output;
	output.open(filename);

	output << "Averaged results: " << endl;

	output << "t \t New builds \t Scrappings \t Second hand purchase \t Second hand selling \t Pool \t Lay-up \t Space charter \t Charter in \t Charter out \t Loops \t Optional trades" << endl;

	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t" << average(yNBResults, nodeData, t) << "\t" << average(ySCResults, nodeData, t) << "\t"
				<< average(ySHResults, nodeData, t) << "\t" << average(ySEResults, nodeData, t) << "\t" << average(yPResults, nodeData, t)
				<< "\t" << average(layUpResults, nodeData, t) << "\t" << average(nResults, nodeData, t) << "\t" << average(hIResults, nodeData, t)
				<< "\t" << average(hOResults, nodeData,t) << "\t" << average(xResults, nodeData, t) << "\t" << average(dResults, nodeData,t) << endl;
	}
	output << endl;

	output << "CashFlow: " << endl;
	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << CashFlow[node] << "\t";
		}
		output << endl;
	}

	output << "Cash Inflow: " << endl;
	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << cashInFlow[node] << "\t";
		}
		output << endl;
	}

	output << "Cash Outflow: " << endl;
	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << cashOutFlow[node] << "\t";
		}
		output << endl;
	}



	output << "Contracted revenue: " << endl;
	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << revenueCObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Optional revenue: " << endl;
	
	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << revenueOObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Space charter cost: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << spaceCharterCostObjectiveResults[node] << "\t";
		}
		output << endl;
	}


	output << "Operating cost: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << operatingCostObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Loop cost: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << loopCostObjectiveResults[node] << "\t";
		}
		output << endl;
	}


	output << "Charter in cost: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << charterCostObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Charter out revenue: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << charterOutObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Lay-up savings: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << layUpSavingsObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Scrapping revenue: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << scrappingObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Selling second hand ships: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << sellingObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Buying second hand ships: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << buyingObjectiveResults[node] << "\t";
		}
		output << endl;
	}

	output << "Instalment cost: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time-nodeData.buildTime+nodeData.M; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.InstalmentNodes[t].begin(); iterNodes != nodeData.InstalmentNodes[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << instalmentObjectiveResults[node] << "\t";
		}
		output << endl;
	}


	output << "Sunset value: " << endl;
	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	
	output << idata.Horizon << "\t";
	map<int, int>::const_iterator iterNodes;
	for (iterNodes = nodeData.NodesTime[idata.Horizon].begin(); iterNodes != nodeData.NodesTime[idata.Horizon].end(); iterNodes++) {
		int node = iterNodes->second;
		int scenario = iterNodes->first;
		output << sunsetObjectiveResults[node] << "\t";
	}
	output << endl;

	// Aggregated results - decision variables
	output << "Aggregated decision variables" << endl;

	output << "New builds " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time - nodeData.buildTime; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << yNBNodes[node] << "\t";
		}
		output << endl;
	}
	
	output << "Scrappings " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Horizon; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << ySCNodes[node] << "\t";
		}
		output << endl;
	}

	output << "Ships sold in the second hand market: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Horizon; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << ySENodes[node] << "\t";
		}
		output << endl;
	}


	output << "Ships bought in the second hand market: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Horizon; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << ySHNodes[node] << "\t";
		}
		output << endl;
	}


	output << "Pool: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 0; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << yPNodes[node] << "\t";
		}
		output << endl;
	}

	output << "Lay-up: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << layUpNodes[node] << "\t";
		}
		output << endl;
	}

	output << "Charter in: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << hINodes[node] << "\t";
		}
		output << endl;
	}


	output << "Charter out: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << hONodes[node] << "\t";
		}
		output << endl;
	}

	output << "Optional trades: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << dNodes[node] << "\t";
		}
		output << endl;
	}

	output << "Space charter: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << nNodes[node] << "\t";
		}
		output << endl;
	}


	output << "Loops: " << endl;

	output << "t\\s \t";
	for (int s = 0; s < nodeData.NodesTime[idata.Horizon].size(); s++) {
		output << s << "\t";
	}
	output << endl;
	for (int t = 1; t < idata.Time; t++) {
		output << t << "\t";
		map<int, int>::const_iterator iterNodes;
		for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
			int node = iterNodes->second;
			int scenario = iterNodes->first;
			output << xNodes[node] << "\t";
		}
		output << endl;
	}


	//Detailed results - decision variabels

	output << "Detailed results" << endl;

	output << "New builds:" << endl;
	for (int t = 0; t < idata.Time - nodeData.buildTime; t++) {
		map<string, int>::const_iterator iterShips;
		for (iterShips = idata.VN[t + nodeData.buildTime].begin(); iterShips != idata.VN[t + nodeData.buildTime].end(); iterShips++) {
			int v = iterShips->second;
			string ship = iterShips->first;
			map<int, int>::const_iterator iterNodes;
			for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
				int node = iterNodes->second;
				int scenario = iterNodes->first;
				output << "( " << ship << ", " << t << ", " << scenario << ")\t" << yNBResults[v][node] << endl;

			}
		}
	}

	output << "Selling second hand ships:" << endl;
	for (int t = 0; t < idata.Horizon; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int scenario = iter2->first;
				int node = iter2->second;

				output << "( " << ship << ", " << t << ", " << scenario << ")\t" << ySEResults[v][node] << endl;
			}
		}
	}

	output << "Buying second hand ships:" << endl;
	for (int t = 0; t < idata.Horizon; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int scenario = iter2->first;
				int node = iter2->second;

				output << "( " << ship << ", " << t << ", " << scenario << ")\t" << ySHResults[v][node] << endl;
			}
		}
	}

	output << "Scrappings:" << endl;
	for (int t = 0; t < idata.Horizon; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int scenario = iter2->first;
				int node = iter2->second;
				
				output << "( " << ship << ", " << t << ", " << scenario << ")\t" << ySCResults[v][node] << endl;
			}
		}
	}


	output << "Pool: " << endl;
	for (int t = 0; t < idata.Time; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				int scenario = iter2->first;
				output << "(" << ship << ", " << t << ", " << scenario << ")\t" << yPResults[v][node] << endl;
			}
		}
	}

	output << "Space charters: " << endl;
	for (int t = 1; t < idata.Time; t++) {
		map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				int scenario = iter2->first;
				map<string, int>::const_iterator iterTrades;
				for (iterTrades = idata.TradesCOf[t].begin(); iterTrades != idata.TradesCOf[t].end(); iterTrades++) {
					int i = iterTrades->second;
					string trade = iterTrades->first;

					for (int k = 0; k < idata.nProduct; k++) {
						output << "( " << trade << ", " << k << ", " << t << ", " << scenario << ")\t" << nResults[i][k][node] << endl;
					}
				}
			}
		}


	output << "Optional trades: " << endl;
	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.TradesOOf[t].begin(); iter != idata.TradesOOf[t].end(); iter++) {
			int i = iter->second;
			string trade = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				int scenario = iter2->first;
				output << "( " << trade << ", " << t << ", " << scenario << ")\t" << dResults[i][node] << endl;
			}
		}
	}

	output << "Lay-Up: " << endl;
	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				int scenario = iter2->first;
				output << "(" << ship << ", " << t << ", " << scenario << ")\t" << layUpResults[v][node] << endl;
			}
		}
	}

	output << "Charter in: " << endl;
	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				int scenario = iter2->first;
				output << "(" << ship << ", " << t << ", " << scenario << ")\t" << hIResults[v][node] << endl;
			}
		}
	}



	output << "Loops: " << endl;
	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iter;
		for (iter = idata.VoF[t].begin(); iter != idata.VoF[t].end(); iter++) {
			int v = iter->second;
			string ship = iter->first;
			map<int, int>::const_iterator iter2;
			for (iter2 = nodeData.NodesTime[t].begin(); iter2 != nodeData.NodesTime[t].end(); iter2++) {
				int node = iter2->second;
				int scenario = iter2->first;
				map<string, int>::const_iterator iterRoutes;
				for (iterRoutes = idata.RofVT[v][t].begin(); iterRoutes != idata.RofVT[v][t].end(); iterRoutes++) {
					int r = iterRoutes->second;
					string route = iterRoutes->first;
					output << "(" << ship << ", " << route << ", " << t << ", " << scenario << ")\t" << xResults[v][r][node] << endl;
				}
			}
		}
	}

	output.close();
}

void nodeModel::appendToFile(string filename, const nodeData& nodeData, const ImportData& idata)
{
}

void nodeModel::compareDemandToCap(const string& filename, const nodeData& nodeData,  const ImportData & idata)
{
	ofstream output(filename);

	output << "\t\t\tDemand\t" << "Capacity\t" << "Difference" << endl;
	for (int t = 1; t < idata.Time; t++) {
		map<string, int>::const_iterator iterTrades;
		for (iterTrades = idata.TradesOf[t].begin(); iterTrades != idata.TradesOf[t].end(); iterTrades++) {
			int i = iterTrades->second;
			string trade = iterTrades->first;
			for (int k = 0; k < idata.nProduct; k++) {
				
				map<int, int>::const_iterator iterNodes;
				for (iterNodes = nodeData.NodesTime[t].begin(); iterNodes != nodeData.NodesTime[t].end(); iterNodes++) {
					int node = iterNodes->second;
					int s = iterNodes->first;

					double temp = 0;
					map<string, int>::const_iterator iterShips;
					for (iterShips = idata.VoF[t].begin(); iterShips != idata.VoF[t].end(); iterShips++) {
						int v = iterShips->second;
						string ship = iterShips->first;

						map<string, int>::const_iterator iterRoutes;
						for (iterRoutes = idata.RofIVT[i][v][t].begin(); iterRoutes != idata.RofIVT[i][v][t].end(); iterRoutes++) {
							int r = iterRoutes->second;
							string loop = iterRoutes->first;

							temp += idata.Capacity[v][k] * xResults[v][r][node];
						}
					}
					double diff = temp - nodeData.Demand[i][k][node];
					output << "(" << trade << ", "<< k << ", " << t << ", " << s << ")\t\t" << nodeData.Demand[i][k][node] << "\t" << temp << "\t" << diff << endl;
				}
			}
		}
	}

	output.close();

}