#pragma once

#include "nodeData.h"
#include <ilcplex/ilocplex.h>

class nodeModel {
public:

	nodeModel();
	nodeModel(const nodeData& nodeData, const ImportData& idata): modelOption(0), addOption(0) { model(nodeData, idata);}
	nodeModel(const nodeData& nodeData, const ImportData& idata, const int& addOption) : modelOption(0), addOption(addOption){ model(nodeData, idata); }
	nodeModel(const nodeData& nodeData, const ImportData& idata, const int& modelOption, const int& addOption, const double& Lt) : Lt(Lt), modelOption(modelOption), addOption(addOption) { model(nodeData, idata); }
	nodeModel(const nodeData& nodeData, const ImportData& idata, const int& modelOption, const int& addOption, const double& Lt, const double & confidenceLevel) : confidenceLevel(confidenceLevel), Lt(Lt), modelOption(modelOption), addOption(addOption) { model(nodeData, idata); }

	void model(const nodeData& nodeData, const ImportData& idata);

	void aggregateResults(const nodeData& nodeData, const ImportData& idata);
	void calculateRobustnesFactor(const nodeData& nodeData);
	void calculateWorstCashFlow(const nodeData& nodeData);

	void writeToFile(string filename, const nodeData& nodeData, const ImportData& idata);
	void appendToFile(string filename, const nodeData& nodeData, const ImportData& idata);


	double Lt;
	double confidenceLevel;
	
	int modelOption;	// 1 = Using the hard constraint ratio, 2 = Using CVaR-constraint, 3 = using absolute cash flow constraints default/any other number = basic model
	int addOption;		// What to include in the basic model, i.e. second hand purchase, charter in/charter out:
						// 1 = add all, 2 = add second hand only, 3 = charters only, default = none
	bool feasible;		// Set to false if the model doesn't finish, i.e. gives an error or is infeasible

	

	//Results profit
	//Declaring expressions
	double Profit;

	vector<vector<double>> charterCostResults;				//Charter in cost, v, n
	vector<vector<double>> charterOutResults;				//Charter out cost, v,n
	vector<vector<double>> buyingResults;						//Second hand buying cost, v, n
	vector<vector<double>> sellingResults;						//Second hand selling cost, v, n
	vector<vector<vector<double>>> revenueOResults;					//i,k,n
	vector<vector<vector<double>>> revenueCResults;					//i,n
	vector<vector<vector<double>>> spaceCharterCostResults;	//i,k,n
	vector<vector<vector<double>>> loopCostResults;					//v,r,n
	vector<vector<double>> operatingCostResults;				//v,n
	vector<vector<double>> layUpSavingsResults;				//v,n
	vector<vector<double>> scrappingValueResults;			//v,n
	vector<vector<vector<double>>> instalmentCostResults;	//v,a,n
	vector<vector<double>> sunsetValueResults;				//v,n

	vector<double> nodeObjectiveResults;	//Storing the objective function for every node
	vector<double> charterOutObjectiveResults;
	vector<double> buyingObjectiveResults;
	vector<double> sellingObjectiveResults;
	vector<double> revenueOObjectiveResults;
	vector<double> revenueCObjectiveResults;
	vector<double> spaceCharterCostObjectiveResults;
	vector<double> operatingCostObjectiveResults;
	vector<double> charterCostObjectiveResults;
	vector<double> loopCostObjectiveResults;
	vector<double> layUpSavingsObjectiveResults;
	vector<double> instalmentObjectiveResults;
	vector<double> sunsetObjectiveResults;
	vector<double> scrappingObjectiveResults;

	vector<IloNumExpr> CashFlowIn;
	vector<IloNumExpr> CashFlowOut;
	vector<double> CashFlow;

	vector<double> cashInFlow;
	vector<double> cashOutFlow;

	double L; //The robustnes factor
	double worstCashFlow;	//The worst cash flow

	//Result parameters
	vector<vector<double>> ySEResults;
	vector<vector<double>> ySHResults;
	vector<vector<double>> yNBResults;
	vector<vector<double>> ySCResults;
	vector<vector<double>> yPResults;
	vector<vector<double>> hIResults;
	vector<vector<double>> hOResults;
	vector<vector<double>> layUpResults;
	vector<vector<vector<double>>> xResults;
	vector<vector<vector<double>>> nResults;
	vector<vector<double>> dResults;

	//Aggregated results parameters for every node
	vector<double> ySENodes;
	vector<double> ySHNodes;
	vector<double> yNBNodes;
	vector<double> ySCNodes;
	vector<double> yPNodes;
	vector<double> hINodes;
	vector<double> hONodes;
	vector<double> layUpNodes;
	vector<double> xNodes;
	vector<double> nNodes;
	vector<double> dNodes;



	void compareDemandToCap(const string& filename, const nodeData& nodeData, const ImportData& idata);
private:
	template<class T>
	double average(const vector<vector<T>>& variable, const nodeData& nodeData, const int & time);

	template<class T>
	double average(const vector<vector<vector<T>>>& variable, const nodeData& nodeData, const int & time);

	
};